<?php

namespace app\components;

use Yii;
use yii\base\Component;
use yii\base\InvalidConfigException;
use yii\helpers\ArrayHelper;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Clase para la consulta directa en la base de datos de SIESA
 *
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
 * @version   Release: $Id$
 * @link      http://www.ingeneo.com.co
 */
class B2BISiesa extends Component {

    public $fechaIn;
    public $fechaFn;
    public $fechaEn;

    /**
     * Retorna el listado de los Centros de operacion
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getListCO() {
        $query = 'SELECT f285_id AS id, f285_descripcion as nombre FROM t285_co_centro_op';
        $result = Yii::$app->dbsql->createCommand($query)->queryAll();
        if (count($result) > 0) {
            return ArrayHelper::map($result, 'id', 'nombre');
        } else {
            return [];
        }
    }

    /**
     * Retorna las ventas de cada cliente deacuerdo al listado de referencias 
     * 
     * @param string $fechaIn    Fecha inicial
     * @param string $fechaFin   Fecha Final
     * @param array $referencias Array de referencias 
     * @param array $select     Columnas seleccionadas
     * @param string $andWhere   Parametros para el filtro
     * @param array $groupBy    Agrupacion de parametros
     * @param array $orderBy    Orden de los parametros
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getVentas($referencias, $select, $andWhere = '', $groupBy, $orderBy, $branchOffice = null) {
        if (is_array($referencias) && count($referencias) > 0 
                && is_array($select) && count($select) > 0 ) {
            //Creamos el select ------------------------------------------------
            $sqlSelect = implode(",", $select);
            //Creamos el group -------------------------------------------------             
            $tmpGroupBy = implode(",", array_unique(array_map('trim', explode('/', implode("/", $groupBy)))));
            $sqlGroupBy = (!empty($tmpGroupBy))? 'GROUP BY '.$tmpGroupBy : '';
            //Creamos el order -------------------------------------------------                        
            $tempOrderBy = implode(",", array_unique(array_map('trim', explode('/', implode("/", $orderBy)))));             
            $sqlOrderBy = (!empty($tmpOrderBy))? 'ORDER BY '.$tmpOrderBy : '';
            //Creamos el where
            $andWhere .= " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            if(empty($branchOffice['branches_office']) == false){
                $andWhere .= " AND FV.f461_id_sucursal_fact IN ('" . implode("','", $branchOffice['branches_office']) . "')";
            }

            if(empty($branchOffice['providers_nit']) == false){
                $andWhere .= " AND T200.f200_nit IN ('" . implode("','", $branchOffice['providers_nit']) . "')";
            }

            if(empty($branchOffice['cities_name']) == false){
                $andWhere .= " AND T103.f013_descripcion IN ('" . implode("','", $branchOffice['cities_name']) . "')";
            }

            $query = "
            SET NOCOUNT ON;
            SELECT                
                " . $sqlSelect . "

            FROM t470_cm_movto_invent MFV 

            INNER JOIN t461_cm_docto_factura_venta FV
                ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = FV.f461_rowid_docto

            -- LEFT JOIN t121_mc_items_extensiones T121
            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MFV.f470_rowid_item_ext

            -- LEFT JOIN t120_mc_items T120
            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item            

            -- LEFT JOIN t200_mm_terceros T200
            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = FV.f461_rowid_tercero_fact
            
            LEFT JOIN t200_mm_terceros T200VEN
                ON T200VEN.f200_rowid = FV.f461_rowid_tercero_vendedor

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id

            LEFT JOIN t431_cm_pv_movto
                ON f470_rowid_pv_movto = f431_rowid

            LEFT JOIN T430_CM_PV_DOCTO PV 
                ON f431_rowid_pv_docto = f430_rowid

            LEFT JOIN T132_Mc_items_instalacion T132 
                ON T132.f132_rowid_item_ext = MFV.f470_rowid_item_ext
                AND T132.f132_id_instalacion = MFV.f470_id_instalacion                
                AND T132.f132_id_instalacion is not null 
            
            INNER JOIN t285_co_centro_op
                ON f285_id = MFV.f470_id_co_movto

            WHERE                
                DOCFAC.f350_id_tipo_docto IN ('FV', 'NC', 'NCD', 'NCE', 'FVE', 'NFC', 'NEF', 'NDF', 'FEV', 'NDE', 'NED', 'NFE', 'DEF', 'NCF')                           
                " . $andWhere . "
                " . $sqlGroupBy . "
                " . $sqlOrderBy . "
                ";
			\Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r('# consulta de ventas #'.$query, true) . '}');
            $arrayResultQuery = Yii::$app->dbsql->createCommand($query)->queryAll();              
            return $arrayResultQuery;
        } else {
            return [];
        }
    }

    /**
     * Retorna las compras realizadas por el proveedor
     * 
     * @param string $fechaIn    Fecha inicial
     * @param string $fechaFin   Fecha Final
     * @param array $referencias Array de referencias 
     * @param string $select     Columnas seleccionadas
     * @param string $andWhere   Parametros para el filtro
     * @param string $groupBy    Agrupacion de parametros
     * @param string $orderBy    Orden de los parametros
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getCompras($referencias, $select, $andWhere = '', $groupBy, $orderBy) {
        if (is_array($referencias) && count($referencias) > 0 
                && is_array($select) && count($select) > 0 ) {
            //Creamos el select ------------------------------------------------
            $sqlSelect = implode(",", $select);
            //Creamos el group -------------------------------------------------
            $tmpGroupBy = implode(",", array_unique(array_map('trim', explode('/', implode("/", $groupBy)))));
            $sqlGroupBy = (!empty($tmpGroupBy)) ? 'GROUP BY ' . $tmpGroupBy : '';
            //Creamos el order -------------------------------------------------                        
            $tempOrderBy = implode(",", array_unique(array_map('trim', explode('/', implode("/", $orderBy)))));
            $sqlOrderBy = (!empty($tmpOrderBy)) ? 'ORDER BY ' . $tmpOrderBy : '';
            //Creamos el where
            $andWhere .= " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            $query = "
            SET NOCOUNT ON;
            SELECT                
                " . $sqlSelect . "

            FROM 
                t455_cm_factura_docto FC

            JOIN t451_cm_docto_compras DC
                ON f451_rowid_docto_factura = FC.f455_rowid_docto
            
            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = DC.f451_rowid_docto

            INNER JOIN t470_cm_movto_invent MI
                ON f451_rowid_docto = f470_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MI.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item

            LEFT JOIN  t421_cm_oc_movto MC
                ON MC.f421_rowid = MI.f470_rowid_oc_movto

            LEFT JOIN t420_cm_oc_docto T420
               ON T420.f420_rowid = f451_rowid_oc_docto

            LEFT JOIN t285_co_centro_op
                ON f285_id = MC.f421_id_co_movto
                
            LEFT JOIN t200_mm_terceros T200
                ON T200.f200_rowid = DC.f451_rowid_tercero_prov

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND DC.f451_id_sucursal_prov = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 

            WHERE                                
                f470_id_concepto IN (401,402)
                " . $andWhere . "
                      
                " . $sqlGroupBy . "
            
                " . $sqlOrderBy . "
                ";

            $arrayResultQuery = Yii::$app->dbsql->createCommand($query)->queryAll();
            return $arrayResultQuery;
        } else {
            return [];
        }
    }

    /**
     * Retorna el inventario actual de SIESA
     *      
     * @param array $referencias Array de referencias      
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getInventario($referencias, $andWhere = '', $branchOffice = null) {
        if (is_array($referencias) && count($referencias) > 0) {  
            $inventario = new \app\models\Inventarios();
            $fechaEn = date('d-m-Y');
            $fechaFn = date('d-m-Y', mktime(0, 0, 0, date('m')-0, 0, date('Y')));                
            $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m')-1, 1, date('Y')));

            if(empty($branchOffice['cities_name']) == false){
                $andWhere .= " AND qry.ciudad IN ('" . implode("','", $branchOffice['cities_name']) . "')";
            }

            $query = "
            SET NOCOUNT ON;			
            Declare 
                @Nit_Proveedor char(13), 
                @Id_Sucursal char(3),
                @Id_Cia int, 
                @Id_bodega Char(13),
                @Fecha_I Datetime, 
                @Fecha_F Datetime,
                @Fecha_E Datetime

            set @Nit_proveedor =''
            set @Id_Sucursal = ''
            set @Id_Cia = 1
            Set @Id_bodega = '' 
            Set @Fecha_F = '" . $fechaFn . "'
            Set @Fecha_I = '" . $fechaIn . "'
            Set @Fecha_E = '" . $fechaEn . "'

            SELECT                
                RTRIM(LTRIM(T120.f120_referencia)) as referencia_dist,                
                T120.f120_descripcion as desc_ref_dist, 
                ".$inventario->getStrCategoriaProv()." as categoria_prov,
                ".$inventario->getStrCategoriaDist()." as categoria_dist,
                RTRIM(LTRIM(ISNULL(qry.Bodega,' '))) as bodega,
                RTRIM(LTRIM(ISNULL(qry.co,' '))) as co,
                -- RTRIM(LTRIM(ISNULL(qry_prov.proveedor,' '))) as nit_proveedor,
                -- ISNULL(qry_prov.Nombre_proveedor,' ') as proveedor,
                -- ISNULL(qry_prov.Sucursal_proveedor,' ') as sucursal_id,
                -- ISNULL(qry_prov.Nombre_Sucursal,' ') as sucursal,
                'UNID' AS unidad_medida,
                SUM(qry.costo) AS costo,
                SUM(qry.total) AS total,
                CONVERT(INT, ISNULL(SUM(qry.Cantidad_Disponible),0)) As cantidad_disponible,
                CONVERT(INT, ISNULL(SUM(qry.Cantidad_comprometida),0)) As Cantidad_comprometida, 
                CONVERT(INT, ISNULL((CASE WHEN (SUM(qry.Cantidad_Disponible))=0 THEN 0 ELSE ((SUM(qry.Cantidad_Disponible)) - (SUM(qry.Cantidad_comprometida))) END),0)) AS cantidad	
            FROM (
            
            -- QUERY PEDIDOS
            -- SELECT                
            --    MPV.f431_rowid_item_ext AS RItem,
            --    T150.f150_id AS Bodega,
            --    T150.f150_id_co AS co,
            --    f285_descripcion AS co,
            --    T150.F150_id_instalacion As Instalacion,
            --    SUM(MPV.F431_CANT1_PEDIDA) AS Cantidad_comprometida,
            --    0 AS Cantidad_Disponible
            -- FROM
            -- T430_CM_PV_DOCTO PV 
            --
            -- INNER JOIN T431_CM_PV_MOVTO MPV
            --    ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO
            --
            -- LEFT JOIN t121_mc_items_extensiones T121
            --    ON T121.f121_rowid = MPV.f431_rowid_item_ext
            --
            -- LEFT JOIN t120_mc_items T120
            --    ON T120.f120_rowid = T121.f121_rowid_item
            --
            -- LEFT JOIN t150_mc_bodegas T150
            --    ON MPV.F431_ROWID_BODEGA = T150.f150_rowid
            --  
            -- JOIN 
            --    t285_co_centro_op
            --    ON T150.f150_id_co = f285_id
            --
            -- WHERE
            --    PV.f430_id_cia = @Id_Cia
            --    AND PV.f430_ind_estado NOT IN(4,9)
            --     AND PV.f430_id_fecha BETWEEN @fecha_I AND @fecha_F
            --    AND PV.f430_ind_facturado = 0
            --    AND PV.f430_ind_remisionado = 0
            --    
            -- GROUP BY                                
            --    F150_id_instalacion,
            --    MPV.f431_rowid_item_ext,
            --    T150.f150_id_co,
            --    f285_descripcion,
            --    T150.f150_id
            --
            -- UNION    

            --QUERY INVENTARIOS Act
                Select
                    T470.F470_ROWID_ITEM_EXT AS RItem,
                    T150.F150_ID As Bodega,
                    f285_descripcion AS co,
                    T470.F470_id_instalacion As Instalacion,	  
                    0 AS Cantidad_comprometida,
                    ISNULL(T132.f132_ultimo_costo_local,0) AS costo,
                    SUM((CASE WHEN T470.F470_ID_CONCEPTO = '502' THEN (T470.f470_vlr_bruto)*-1 ELSE T470.f470_vlr_bruto END)) AS total,
                    Sum(CASE WHEN T470.F470_IND_NATURALEZA = 1 THEN T470.F470_CANT_1 ELSE T470.F470_CANT_1*-1 END) As Cantidad_Disponible,
                    f013_descripcion AS ciudad	 	  	  
                FROM
                    (T150_MC_BODEGAS T150 Right Outer Join
                    T470_CM_MOVTO_INVENT T470 
                    On T150.F150_ROWID = T470.F470_ROWID_BODEGA)
                -- Cruzamos con las bodegas principales
                JOIN 
                    (SELECT f150_descripcion AS bodega, f150_id AS id_bodega
                            FROM t150_mc_bodegas
                            JOIN t152_mc_bodega_grupo_bodega ON f152_rowid_bodega = f150_rowid
                            JOIN t151_mc_grupo_bodega ON f152_id_grupo_bodega = f151_id
                            WHERE f151_id IN ('BPP') ) AS GB
                    ON GB.id_bodega = T150.F150_ID
                    
                JOIN 
                    t285_co_centro_op
                    ON T150.f150_id_co = f285_id
                
                LEFT JOIN T132_Mc_items_instalacion T132 
                ON T132.f132_rowid_item_ext = T470.f470_rowid_item_ext
                AND T132.f132_id_instalacion = T470.f470_id_instalacion                
                AND T132.f132_id_instalacion is not null 
                
                LEFT JOIN t015_mm_contactos ON f015_rowid = t285_co_centro_op.f285_rowid_contacto
                JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 
                
                WHERE
                    T470.F470_ID_CIA = @Id_CIA
                    -- AND T150.F150_ID = @Id_bodega
                    AND T470.F470_ID_fecha <= @Fecha_E 
                    AND T470.F470_IND_ESTADO_CM  <> '2' 
                    AND T470.F470_IND_TIPO_ITEM = 1	  	  
                GROUP BY
                    T470.F470_ROWID_ITEM_EXT,
                    -- T150.f150_id_co,
                    f285_descripcion,
                    T150.F150_ID,
                    T470.F470_id_instalacion,
                    T132.f132_ultimo_costo_local,
                    f013_descripcion
                ) qry
	
            FULL JOIN t121_mc_items_extensiones T121
                On qry.ritem = T121.f121_rowid
	
            INNER JOIN t120_mc_items T120
                On T121.f121_rowid_item = T120.f120_rowid
                AND T120.F120_IND_TIPO_ITEM = '1'
        
            WHERE 
                qry.Bodega IS NOT NULL                 
                " . $andWhere . "
	
            GROUP BY 
                T120.f120_referencia,
                T120.f120_id,
                T120.f120_descripcion, 
                T120.F120_IND_TIPO_ITEM,
                qry.Bodega,
                qry.co                

            ORDER BY 
                T120.f120_referencia, 
                T120.f120_id,
                qry.Bodega";

           return Yii::$app->dbsql->createCommand($query)->queryAll();
        } else {
            return [];
        }
    }
    
    /**
     * Retorna el cliente con mayor venta
     * 
     * @param string $fechaIn    Fecha inicial
     * @param string $fechaFin   Fecha Final
     * @param array $referencias Array de referencias      
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getClienteMayorVenta($fechaIn, $fechaFin, $referencias, $branchOffice) {
        if (is_array($referencias) && count($referencias) > 0) {
            //Creamos el where
            $andWhere = " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            if(empty($branchOffice['branches_office']) == false){
                $andWhere .= " AND FV.f461_id_sucursal_fact IN ('" . implode("','", $branchOffice['branches_office']) . "')";
            }

            if(empty($branchOffice['providers_nit']) == false){
                $andWhere .= " AND T200.f200_nit IN ('" . implode("','", $branchOffice['providers_nit']) . "')";
            }

            if(empty($branchOffice['cities_name']) == false){
                $andWhere .= " AND T103.f013_descripcion IN ('" . implode("','", $branchOffice['cities_name']) . "')";
            }

            $query = "
            SET NOCOUNT ON;


            SELECT TOP(1)
                T200.f200_nit as nit,
                LTRIM(RTRIM(T200.f200_razon_social)) as razon_social,
                -- SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea END)) AS total
                CAST((SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto)*-1 ELSE MFV.f470_vlr_neto END)) -
                SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END)))AS BIGINT) AS total

            FROM t470_cm_movto_invent MFV 

            INNER JOIN t461_cm_docto_factura_venta FV
                ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = FV.f461_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MFV.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item            

            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = FV.f461_rowid_tercero_fact  
            
            LEFT JOIN t200_mm_terceros T200VEN
                ON T200VEN.f200_rowid = FV.f461_rowid_tercero_vendedor

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 
               
            WHERE
                FV.f461_id_cia = 1 
                AND DOCFAC.f350_id_tipo_docto IN ('FV', 'NC', 'NCD', 'NCE', 'FVE', 'NFC', 'NEF', 'NDF', 'FEV', 'NDE', 'NED', 'NFE', 'DEF', 'NCF')
                AND DOCFAC.f350_fecha BETWEEN '" . $fechaIn . "' AND '" . $fechaFin . "'            
                " . $andWhere . "

            GROUP BY                
                T200.f200_nit,T200.f200_razon_social
                
            ORDER BY total DESC ";
            $arrayResultQuery = Yii::$app->dbsql->createCommand($query)->queryOne();
            return $arrayResultQuery;
        } else {
            return [];
        }
    }
    
    /**
     * Retorna la referencia masvendida
     * 
     * @param string $fechaIn    Fecha inicial
     * @param string $fechaFin   Fecha Final
     * @param array $referencias Array de referencias      
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getRefefrenciaMayorVenta($fechaIn, $fechaFin, $referencias, $branchOffice = null) {

        if (is_array($referencias) && count($referencias) > 0) {
            //Creamos el where
            $andWhere = " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            if(empty($branchOffice['branches_office']) == false){
                $andWhere .= " AND FV.f461_id_sucursal_fact IN ('" . implode("','", $branchOffice['branches_office']) . "')";
            }

            if(empty($branchOffice['providers_nit']) == false){
                $andWhere .= " AND T200.f200_nit IN ('" . implode("','", $branchOffice['providers_nit']) . "')";
            }

            if(empty($branchOffice['cities_name']) == false){
                $andWhere .= " AND T103.f013_descripcion IN ('" . implode("','", $branchOffice['cities_name']) . "')";
            }

            $query = "
            SET NOCOUNT ON;

            SELECT TOP(1)
                LTRIM(RTRIM(T120.f120_referencia)) AS referencia,
                LTRIM(RTRIM(T120.f120_descripcion)) AS descripcion,
                -- SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea END)) AS total
                -- SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_bruto)*-1 ELSE MFV.f470_vlr_bruto END)) AS total
                CAST((SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto)*-1 ELSE MFV.f470_vlr_neto END)) -
				SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END))) AS BIGINT) AS total
                
            FROM t470_cm_movto_invent MFV 

            INNER JOIN t461_cm_docto_factura_venta FV
                ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = FV.f461_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MFV.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item            

            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = FV.f461_rowid_tercero_fact   

            LEFT JOIN t200_mm_terceros T200VEN
                ON T200VEN.f200_rowid = FV.f461_rowid_tercero_vendedor

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 

            WHERE
                FV.f461_id_cia = 1 
                AND DOCFAC.f350_id_tipo_docto IN ('FV', 'NC', 'NCD', 'NCE', 'FVE', 'NFC', 'NEF', 'NDF', 'FEV', 'NDE', 'NED', 'NFE', 'DEF', 'NCF')
                AND DOCFAC.f350_fecha BETWEEN '" . $fechaIn . "' AND '" . $fechaFin . "'            
                " . $andWhere . "

            GROUP BY                
                T120.f120_referencia,T120.f120_descripcion
                
            ORDER BY total DESC
                ";
            $arrayResultQuery = Yii::$app->dbsql->createCommand($query)->queryOne();            
            return $arrayResultQuery;
        } else {
            return [];
        }
    }

    /**
     * Retorna el listado de sucursales asociadas a un nit
     *
     * @param integer $nit Nit del cliente
     *
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getSucursales($nit) {
        # Consulra a la base de datos SQL server -------------------------------
        $sql = "SELECT f201_id_sucursal as cod_sucursal, 
            f201_descripcion_sucursal as nombre_sucursal
            FROM t201_mm_clientes
            LEFT JOIN t200_mm_terceros AS suc ON f200_rowid = f201_rowid_tercero
            WHERE f200_nit ='" . $nit . "' AND f200_ind_estado = '1'
            GROUP BY f201_id_sucursal, f201_descripcion_sucursal
            ORDER BY f201_descripcion_sucursal ASC";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }

    /**
     * Retorna el listado de ciudades
     *
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getCities() {
        # Consulra a la base de datos SQL server -------------------------------
        $sql = "SELECT f013_id as code, 
            f013_descripcion as name
            FROM t013_mm_ciudades
            GROUP BY f013_id, f013_descripcion
            ORDER BY f013_descripcion";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }

    /**
     * Retorna el listado de ciudades
     *
     * @return array
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getRequestedCitiesDrop() {
        # Consulra a la base de datos SQL server -------------------------------
        $sql = "SELECT                
                f013_id_depto + '-' + T103.f013_id as code,
                T103.f013_descripcion as name
             FROM t200_mm_terceros T200
        
               INNER JOIN T201_MM_CLIENTES C201
                   ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                   
               LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
               JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                   AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 
                
             GROUP BY                                
                T103.f013_id,
                f013_id_depto,
                T103.f013_descripcion
                
               ORDER BY
                T103.f013_descripcion";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }

    /**
     * 
     * @param type $descripcion
     * @return type
     */
    public function getCodeCities($descripcion) {
        # Consulra a la base de datos SQL server -------------------------------
        $sql = "SELECT f013_id as code
            FROM t013_mm_ciudades
            WHERE f013_descripcion LIKE '".$descripcion."'";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }
    
    /**
     * 
     * @param type $descripcion
     * @return type
     */
    public function getClients($descripcion) {
        $sql = "SELECT f200_nit as code, f200_razon_social as business_name
            FROM t200_mm_terceros
            WHERE f200_ind_cliente = 1 and f200_razon_social LIKE '%".$descripcion."%'";
        return Yii::$app->dbsql->createCommand($sql)->queryAll();
    }


    /**
     * Retorna el total de las ventas
     * 
     * @param string $fechaIn    Fecha inicial
     * @param string $fechaFin   Fecha Final
     * @param array $referencias Array de referencias      
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getTotalVenta($fechaIn, $fechaFin, $referencias, $branchOffice = null) {

        if (is_array($referencias) && count($referencias) > 0) {
            //Creamos el where
            $andWhere = " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            if(empty($branchOffice['branches_office']) == false){
                $andWhere .= " AND FV.f461_id_sucursal_fact IN ('" . implode("','", $branchOffice['branches_office']) . "')";
            }

            if(empty($branchOffice['providers_nit']) == false){
                $andWhere .= " AND T200.f200_nit IN ('" . implode("','", $branchOffice['providers_nit']) . "')";
            }

            if(empty($branchOffice['cities_name']) == false){
                $andWhere .= " AND T103.f013_descripcion IN ('" . implode("','", $branchOffice['cities_name']) . "')";
            }

            $query = "
            SET NOCOUNT ON;

            SELECT                             
                -- SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_bruto)*-1 ELSE MFV.f470_vlr_bruto END)) AS total
                CAST((SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto)*-1 ELSE MFV.f470_vlr_neto END)) -
				SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END))) AS BIGINT) AS total
                
            FROM t470_cm_movto_invent MFV 

            INNER JOIN t461_cm_docto_factura_venta FV
                ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = FV.f461_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MFV.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item            

            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = FV.f461_rowid_tercero_fact   
            
            LEFT JOIN t200_mm_terceros T200VEN
                ON T200VEN.f200_rowid = FV.f461_rowid_tercero_vendedor

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 

            WHERE
                FV.f461_id_cia = 1 
                AND DOCFAC.f350_id_tipo_docto IN ('FV', 'NC', 'NCD', 'NCE', 'FVE', 'NFC', 'NEF', 'NDF', 'FEV', 'NDE', 'NED', 'NFE', 'DEF', 'NCF')
                AND DOCFAC.f350_fecha BETWEEN '" . $fechaIn . "' AND '" . $fechaFin . "'            
                " . $andWhere . "           
                ";
            $arrayResultQuery = Yii::$app->dbsql->createCommand($query)->queryOne();              
            if(!isset($arrayResultQuery['total']) || $arrayResultQuery['total'] == null){
                $arrayResultQuery['total'] = 0;
            }
            return $arrayResultQuery;
        } else {
            return [];
        }
    }
    
    /**
     * Retorna el total de las compras 
     * 
     * @param string $fechaIn    Fecha inicial
     * @param string $fechaFin   Fecha Final
     * @param array $referencias Array de referencias      
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getTotalCompra($fechaIn, $fechaFin, $referencias) {

        if (is_array($referencias) && count($referencias) > 0) {            
            //Creamos el where
            $andWhere = " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            $query = "
            SET NOCOUNT ON;

            SELECT
		-- SUM((CASE WHEN MI.f470_id_concepto = '402' THEN (MI.f470_vlr_neto - MI.f470_vlr_imp - MI.f470_vlr_dscto_linea)*-1 ELSE MI.f470_vlr_neto - MI.f470_vlr_imp - MI.f470_vlr_dscto_linea END)) AS total
		CAST(SUM((CASE WHEN MI.f470_id_concepto = '402' THEN (MI.f470_vlr_bruto)*-1 ELSE MI.f470_vlr_bruto END)) AS BIGINT) AS total

            FROM 
                t455_cm_factura_docto FC

            JOIN t451_cm_docto_compras DC
                ON f451_rowid_docto_factura = FC.f455_rowid_docto
                
            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = DC.f451_rowid_docto

            INNER JOIN t470_cm_movto_invent MI
                ON f451_rowid_docto = f470_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MI.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item

            LEFT JOIN  t421_cm_oc_movto MC
                ON MC.f421_rowid = MI.f470_rowid_oc_movto

            LEFT JOIN t420_cm_oc_docto T420
               ON T420.f420_rowid = f451_rowid_oc_docto

            LEFT JOIN t285_co_centro_op
                ON f285_id = MC.f421_id_co_movto
                
            LEFT JOIN t200_mm_terceros T200
                ON T200.f200_rowid = DC.f451_rowid_tercero_prov

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND DC.f451_id_sucursal_prov = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 

            WHERE                
                DOCFAC.f350_fecha BETWEEN '" . $fechaIn . "' AND '" . $fechaFin . "'
                AND f470_id_concepto IN (401,402)
                " . $andWhere . "           
                ";
            $arrayResultQuery = Yii::$app->dbsql->createCommand($query)->queryOne();              
            if(!isset($arrayResultQuery['total']) || $arrayResultQuery['total'] == null){
                $arrayResultQuery['total'] = 0;
            }
            return $arrayResultQuery;
        } else {
            return [];
        }
    }
    
    
    
    
    /**
     * Retorna el total de las ventas de los ultimos meses 
     *           
     * @param array $referencias Array de referencias      
     * @param int $noMeses   
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getUltimasVentasMeses($referencias, $noMeses, $branchOffice = null) {

        if (is_array($referencias) && count($referencias) > 0
            && is_int($noMeses)) {
            $fechaFin = date('d-m-Y');            
            $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m')-$noMeses, 1, date('Y')));
            $fechaFinAn = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d'), date('Y')-1));
            $fechaInAn = date('d-m-Y', mktime(0, 0, 0, date('m')-$noMeses, 1, date('Y')-1));
            //Creamos el where -------------------------------------------------            
            $andWhere = " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            if(empty($branchOffice['branches_office']) == false){
                $andWhere .= " AND FV.f461_id_sucursal_fact IN ('" . implode("','", $branchOffice['branches_office']) . "')";
            }

            if(empty($branchOffice['providers_nit']) == false){
                $andWhere .= " AND T200.f200_nit IN ('" . implode("','", $branchOffice['providers_nit']) . "')";
            }

            if(empty($branchOffice['cities_name']) == false){
                $andWhere .= " AND T103.f013_descripcion IN ('" . implode("','", $branchOffice['cities_name']) . "')";
            }

            $query = "
            SET NOCOUNT ON;

            SELECT                
                YEAR(DOCFAC.f350_fecha) as anio,
                MONTH(DOCFAC.f350_fecha) as mes,
                -- SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea END)) AS total
                -- SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_bruto)*-1 ELSE MFV.f470_vlr_bruto END)) AS total
                CAST((SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto)*-1 ELSE MFV.f470_vlr_neto END)) -
				SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END))) AS BIGINT) AS total,
                SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END)) AS imp
				
            FROM t470_cm_movto_invent MFV 

            INNER JOIN t461_cm_docto_factura_venta FV
                ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = FV.f461_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MFV.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item            

            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = FV.f461_rowid_tercero_fact 
                
            INNER JOIN t200_mm_terceros T200VEN
                ON T200VEN.f200_rowid = FV.f461_rowid_tercero_vendedor

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 
            
            WHERE
                FV.f461_id_cia = 1 
                AND DOCFAC.f350_id_tipo_docto IN ('FV', 'NC', 'NCD', 'NCE', 'FVE', 'NFC', 'NEF', 'NDF', 'FEV', 'NDE', 'NED', 'NFE', 'DEF', 'NCF')
                -- AND f461_id_concepto IN (501,502)
                AND DOCFAC.f350_fecha BETWEEN '" . $fechaIn . "' AND '" . $fechaFin . "'            
                " . $andWhere . "
            GROUP BY 
                YEAR(DOCFAC.f350_fecha),
                MONTH(DOCFAC.f350_fecha)
            ORDER BY 
            	 MONTH(DOCFAC.f350_fecha)            
                ";
            $arrayYear = Yii::$app->dbsql->createCommand($query)->queryAll();
            
            $query = "
            SET NOCOUNT ON;
            
            SELECT
                YEAR(DOCFAC.f350_fecha) as anio,
                MONTH(DOCFAC.f350_fecha) as mes,
                CAST((SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto)*-1 ELSE MFV.f470_vlr_neto END)) -
				SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END))) AS BIGINT) AS total,
				SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END)) AS imp
				
            FROM t470_cm_movto_invent MFV 

            INNER JOIN t461_cm_docto_factura_venta FV
                ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = FV.f461_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MFV.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item         
      
            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = FV.f461_rowid_tercero_fact 
                
            INNER JOIN t200_mm_terceros T200VEN
                ON T200VEN.f200_rowid = FV.f461_rowid_tercero_vendedor

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 
            WHERE
                FV.f461_id_cia = 1 
                AND DOCFAC.f350_id_tipo_docto IN ('FV', 'NC', 'NCD', 'NCE', 'FVE', 'NFC', 'NEF', 'NDF', 'FEV', 'NDE', 'NED', 'NFE', 'DEF', 'NCF')
                -- AND f461_id_concepto IN (501,502)
                AND DOCFAC.f350_fecha BETWEEN '" . $fechaInAn . "' AND '" . $fechaFinAn. "'
                " . $andWhere . "
            GROUP BY 
                YEAR(DOCFAC.f350_fecha),
                MONTH(DOCFAC.f350_fecha)
            ORDER BY 
            	 MONTH(DOCFAC.f350_fecha)
                ";
            $arrayLastYear = Yii::$app->dbsql->createCommand($query)->queryAll(); 
            $arrayResultQuery = array_merge($arrayYear, $arrayLastYear);            
            if(count($arrayResultQuery)>0){                
                return ArrayHelper::index($arrayResultQuery, 'mes', ['anio']);
            }else{
                return $arrayResultQuery;
            }            
        } else {
            return [];
        }
    }
    
    /**
     * Retorna el total de las compras de los ultimos meses
     *      
     * @param array   $referencias Array de referencias      
     * @param integer $noMeses     No de meses a consultar
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getUltimasComprasMeses($referencias, $noMeses) {

        if (is_array($referencias) && count($referencias) > 0
            && is_int($noMeses)) {
            //Creamos el where
            $andWhere = " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            $fechaFin = date('d-m-Y');            
            $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m')-$noMeses, 1, date('Y')));
            $fechaFinAn = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d'), date('Y')-1));
            $fechaInAn = date('d-m-Y', mktime(0, 0, 0, date('m')-$noMeses, 1, date('Y')-1));

            $query = "
            SET NOCOUNT ON;  
            
            SELECT
                YEAR(DOCFAC.f350_fecha) as anio,
                MONTH(DOCFAC.f350_fecha) as mes,
		-- SUM((CASE WHEN MI.f470_id_concepto = '402' THEN (MI.f470_vlr_neto - MI.f470_vlr_imp - MI.f470_vlr_dscto_linea)*-1 ELSE MI.f470_vlr_neto - MI.f470_vlr_imp - MI.f470_vlr_dscto_linea END)) AS total
		SUM((CASE WHEN MI.f470_id_concepto = '402' THEN (MI.f470_vlr_bruto)*-1 ELSE MI.f470_vlr_bruto END)) AS total
            FROM 
                t455_cm_factura_docto FC

            JOIN t451_cm_docto_compras DC
            ON f451_rowid_docto_factura = FC.f455_rowid_docto
            
            INNER JOIN t350_co_docto_contable DOCFAC
            ON DOCFAC.f350_rowid = DC.f451_rowid_docto

            INNER JOIN t470_cm_movto_invent MI
            ON f451_rowid_docto = f470_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
            ON T121.f121_rowid = MI.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
            ON T120.f120_rowid = T121.f121_rowid_item

            INNER JOIN  t421_cm_oc_movto MC
                ON MC.f421_rowid = MI.f470_rowid_oc_movto

            INNER JOIN t420_cm_oc_docto T420
               ON T420.f420_rowid = f451_rowid_oc_docto

            INNER JOIN t285_co_centro_op
                ON f285_id = MC.f421_id_co_movto
                
            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = DC.f451_rowid_tercero_prov

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND DC.f451_id_sucursal_prov = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 

            WHERE                
                DOCFAC.f350_fecha BETWEEN '" . $fechaIn . "' AND '" . $fechaFin . "'
                AND f470_id_concepto IN (401,402)
                " . $andWhere . "
            GROUP BY 
                YEAR(DOCFAC.f350_fecha),
                MONTH(DOCFAC.f350_fecha)
            ORDER BY 
            	 MONTH(DOCFAC.f350_fecha)
                ";  
            $arrayYear = Yii::$app->dbsql->createCommand($query)->queryAll();
            
            $query = "
            SET NOCOUNT ON;  
            
            SELECT
                YEAR(DOCFAC.f350_fecha) as anio,
                MONTH(DOCFAC.f350_fecha) as mes,
		SUM((CASE WHEN MI.f470_id_concepto = '402' THEN (MI.f470_vlr_neto - MI.f470_vlr_imp - MI.f470_vlr_dscto_linea)*-1 ELSE MI.f470_vlr_neto - MI.f470_vlr_imp - MI.f470_vlr_dscto_linea END)) AS total
            FROM 
                t455_cm_factura_docto FC

            JOIN t451_cm_docto_compras DC
            ON f451_rowid_docto_factura = FC.f455_rowid_docto
            
            INNER JOIN t350_co_docto_contable DOCFAC
            ON DOCFAC.f350_rowid = DC.f451_rowid_docto

            INNER JOIN t470_cm_movto_invent MI
            ON f451_rowid_docto = f470_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
            ON T121.f121_rowid = MI.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
            ON T120.f120_rowid = T121.f121_rowid_item

            INNER JOIN  t421_cm_oc_movto MC
             ON MC.f421_rowid = MI.f470_rowid_oc_movto

            INNER JOIN t420_cm_oc_docto T420
             ON T420.f420_rowid = MC.f421_rowid_oc_docto

            INNER JOIN t285_co_centro_op
            ON f285_id = MC.f421_id_co_movto
            
            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = DC.f451_rowid_tercero_prov

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND DC.f451_id_sucursal_prov = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 

            WHERE             
            DOCFAC.f350_fecha BETWEEN '" . $fechaInAn . "' AND '" . $fechaFinAn. "'
                " . $andWhere . "
            GROUP BY 
                YEAR(DOCFAC.f350_fecha),
                MONTH(DOCFAC.f350_fecha)
            ORDER BY 
            	 MONTH(DOCFAC.f350_fecha)
                ";  
            $arrayLastYear = Yii::$app->dbsql->createCommand($query)->queryAll();
            
            $arrayResultQuery = array_merge($arrayYear, $arrayLastYear);
            if(count($arrayResultQuery)>0){                
                return ArrayHelper::index($arrayResultQuery, 'mes', ['anio']);
            }else{
                return $arrayResultQuery;
            }
        } else {
            return [];
        }
    }
    
    /**
     * Retorna el total de las ventas de los ultimos meses 
     *           
     * @param array $referencias Array de referencias      
     * @param int $noMeses   
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getDashVentas($referencias, $noMeses) {

        if (is_array($referencias) && count($referencias) > 0
            && is_int($noMeses)) {
            $fechaFin = date('d-m-Y');            
            $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m')-$noMeses, 1, date('Y')));
            //Creamos el where -------------------------------------------------            
            $andWhere = " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            $query = "
            SET NOCOUNT ON;

            SELECT
                T.anio,T.mes, 
                nit, 
                razon_social, 
                sucursal, 
                referencia, 
                descripcion, 
                ciudad, 
                SUM(T.total) - SUM(T.imp) AS total  
            FROM (
            SELECT                
                YEAR(DOCFAC.f350_fecha) as anio,
                MONTH(DOCFAC.f350_fecha) as mes,
                T200.f200_nit as nit,
                T200.f200_razon_social as razon_social,
                f461_id_sucursal_fact as sucursal,
                T120.f120_referencia AS referencia,
                T120.f120_descripcion AS descripcion,
                T103.f013_descripcion as ciudad,
                -- SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea END)) AS total
                -- SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_bruto)*-1 ELSE MFV.f470_vlr_bruto END)) AS total
                SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto)*-1 ELSE MFV.f470_vlr_neto END)) AS total,
                SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END)) AS imp
				
            FROM t470_cm_movto_invent MFV 

            INNER JOIN t461_cm_docto_factura_venta FV
                ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = FV.f461_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MFV.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item            

            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = FV.f461_rowid_tercero_fact 
                
            INNER JOIN t200_mm_terceros T200VEN
                ON T200VEN.f200_rowid = FV.f461_rowid_tercero_vendedor

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 
            
            WHERE
                FV.f461_id_cia = 1 
                AND DOCFAC.f350_id_tipo_docto IN ('FV', 'NC', 'NCD', 'NCE', 'FVE', 'NFC', 'NEF', 'NDF', 'FEV', 'NDE', 'NED', 'NFE', 'DEF', 'NCF')
                -- AND f461_id_concepto IN (501,502)
                AND DOCFAC.f350_fecha BETWEEN '" . $fechaIn . "' AND '" . $fechaFin . "'            
                " . $andWhere . "
            GROUP BY 
                DOCFAC.f350_fecha, T200.f200_nit, FV.f461_id_sucursal_fact,
                T200.f200_razon_social, T120.f120_referencia,
                T120.f120_descripcion, T103.f013_descripcion
                 
                 ) T                 
            GROUP BY T.anio,T.mes, nit, razon_social, sucursal, referencia, descripcion, ciudad
            ORDER BY T.anio,T.mes, nit, razon_social, sucursal, referencia, descripcion, ciudad
                ";
            try{
                $data = Yii::$app->dbsql->createCommand($query)->queryAll();
            } catch (\Exception $ex) {
                throw new \Exception();
            }
                       
        } else {
            return [];
        }
    }
    
    /**
     * Retorna el total de las ventas de los ultimos meses 
     *           
     * @param array $referencias Array de referencias      
     * @param int $noMeses   
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getDashVentasUltAnio($referencias, $noMeses) {

        if (is_array($referencias) && count($referencias) > 0
            && is_int($noMeses)) {
            $fechaFinAn = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d'), date('Y')-1));
            $fechaInAn = date('d-m-Y', mktime(0, 0, 0, date('m')-$noMeses, 1, date('Y')-1));
            //Creamos el where -------------------------------------------------            
            $andWhere = " AND T120.f120_referencia IN ('" . implode("','", $referencias) . "')";

            $query = "
            SET NOCOUNT ON;

            SELECT
                T.anio,T.mes, 
                nit, 
                razon_social, 
                sucursal, 
                referencia, 
                descripcion, 
                ciudad, 
                SUM(T.total) - SUM(T.imp) AS total  
            FROM (
 				
            SELECT
                YEAR(DOCFAC.f350_fecha) as anio,
                MONTH(DOCFAC.f350_fecha) as mes,
                T200.f200_nit as nit,
                T200.f200_razon_social as razon_social,
                f461_id_sucursal_fact as sucursal,
                T120.f120_referencia AS referencia,
                T120.f120_descripcion AS descripcion,
                T103.f013_descripcion as ciudad,
                -- SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_bruto)*-1 ELSE MFV.f470_vlr_bruto END)) AS total
                SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto)*-1 ELSE MFV.f470_vlr_neto END)) AS total,
				SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END)) AS imp
				
            FROM t470_cm_movto_invent MFV 

            INNER JOIN t461_cm_docto_factura_venta FV
                ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

            INNER JOIN t350_co_docto_contable DOCFAC
                ON DOCFAC.f350_rowid = FV.f461_rowid_docto

            INNER JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MFV.f470_rowid_item_ext

            INNER JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item         
      
            INNER JOIN t200_mm_terceros T200
                ON T200.f200_rowid = FV.f461_rowid_tercero_fact 
                
            INNER JOIN t200_mm_terceros T200VEN
                ON T200VEN.f200_rowid = FV.f461_rowid_tercero_vendedor

            INNER JOIN T201_MM_CLIENTES C201
                ON T200.F200_ROWID = C201.F201_ROWID_TERCERO
                AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL 
                
            LEFT JOIN t015_mm_contactos ON f015_rowid = C201.f201_rowid_contacto
            JOIN t013_mm_ciudades T103 ON f015_id_pais = f013_id_pais 
                AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id 
            WHERE
                FV.f461_id_cia = 1 
                AND DOCFAC.f350_id_tipo_docto IN ('FV', 'NC', 'NCD', 'NCE', 'FVE', 'NFC', 'NEF', 'NDF', 'FEV', 'NDE', 'NED', 'NFE', 'DEF', 'NCF')
                -- AND f461_id_concepto IN (501,502)
                AND DOCFAC.f350_fecha BETWEEN '" . $fechaInAn . "' AND '" . $fechaFinAn. "'
                " . $andWhere . "
            GROUP BY 
                DOCFAC.f350_fecha, T200.f200_nit, FV.f461_id_sucursal_fact,
                T200.f200_razon_social, T120.f120_referencia,
                T120.f120_descripcion, T103.f013_descripcion
                                  
                 ) T                 
            GROUP BY T.anio,T.mes, nit, razon_social, sucursal, referencia, descripcion, ciudad
            ORDER BY T.anio,T.mes, nit, razon_social, sucursal, referencia, descripcion, ciudad
                ";
            try{
                $data = Yii::$app->dbsql->createCommand($query)->queryAll();
            } catch (\Exception $ex) {
                throw new \Exception();
            }
                       
        } else {
            return [];
        }
    }

}
