<?php
namespace app\components;
 
use Yii;
use yii\base\Component;
use yii\base\InvalidConfigException;
use yii\helpers\ArrayHelper;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Clase para la consulta directa en la base de datos de SIESA
 *
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2017 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
 * @version   Release: $Id$
 * @link      http://www.ingeneo.com.co
 */
class SQLSiesa extends Component {            
    
    public $fechaIn;
    public $fechaFn;
    public $fechaEn;
    
    /**
     * Retorna el listado de los Centros de operacion
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getListCO(){
        $query = 'SELECT f285_id AS id, f285_descripcion as nombre FROM t285_co_centro_op';
        $result = Yii::$app->dbsql->createCommand($query)->queryAll();
        if(count($result)>0){
            return ArrayHelper::map($result, 'id', 'nombre');
        }else{
            return [];
        }
    }            
    
    /**
     * Retorna el listado de las ciudades
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getCiudades(){        
        $query = '
            SELECT 
            f013_id_pais + f013_id_depto + f013_id AS id, 
            f013_descripcion as nombre 
            FROM t013_mm_ciudades';
        return Yii::$app->dbsql->createCommand($query)->queryAll();                
    }
    
    /**
     * Obtiene el listado de items por el bombre del fabricante
     * 
     * @param string $fabricantes Nombre de los fabricantes
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getItemsXFrabricante($fabricantes){
        $andWhere = '';
        if(is_array($fabricantes) && count($fabricantes)>0){
            $andWhere = " AND f106_descripcion IN('".implode("','", $fabricantes)."')";
        }
        $query = "
            SELECT 
                RTRIM(LTRIM(f120_referencia)) AS codigo_producto, 
                f120_descripcion AS nombre,
                'UNID' AS unidad_medida,
                f106_descripcion AS compania
            FROM t120_mc_items 
            JOIN t125_mc_items_criterios ON f120_rowid = f125_rowid_item
            JOIN t106_mc_criterios_item_mayores 
                ON f106_id_plan = f125_id_plan 
                AND f106_id = f125_id_criterio_mayor
            WHERE f106_id_plan = 1 ".$andWhere."
            ORDER BY f120_descripcion";
        return Yii::$app->dbsql->createCommand($query)->queryAll(); 
    }    
    
    /**
     * Retorna la informacion de cada NIT
     * 
     * @param array $nits Array de Nit
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getClientes($nits){ 
        $andWhere = '';
        if(is_array($nits) && count($nits)>0){
            $andWhere = " AND f200_nit IN('".implode("','", $nits)."')";
        }
        $query = "
            SELECT 
                RTRIM(LTRIM(f200_rowid)) as codigo,
                f200_razon_social as nombre,
                f200_fecha_nacimiento as fecha_ingreso,                
                f200_nit as nit,            
                f015_direccion1 as direccion, 
                f015_id_pais + f015_id_depto + f015_id_ciudad as codigo_municipio,
                f204_descripcion,
                f207_id_plan_criterios,
                f207_id_criterio_mayor as id_tipo_negocio,
                f206_descripcion
            FROM t200_mm_terceros
            JOIN t015_mm_contactos ON f015_rowid = f200_rowid_contacto
            JOIN t013_mm_ciudades ON f015_id_pais = f013_id_pais AND f015_id_depto = f013_id_depto AND f015_id_ciudad = f013_id
            JOIN t207_mm_criterios_clientes ON f207_rowid_tercero = f200_rowid AND f207_id_sucursal = '001'
            JOIN t204_mm_planes_criterios ON f204_id = f207_id_plan_criterios AND f204_id = '014'
            JOIN t206_mm_criterios_mayores ON f206_id = f207_id_criterio_mayor
            WHERE 
                f200_nit IS NOT NULL AND f200_ind_estado = 1 
                AND f200_ind_cliente = 1
                ".$andWhere."
            ORDER BY f200_razon_social ASC";
        return Yii::$app->dbsql->createCommand($query)->queryAll();                
    }
    
    /**
     * Retorna el listado de los tipos de negocio
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getTiposNegocio(){                
        $query = "
            SELECT 
                RTRIM(LTRIM(f206_id)) AS codigo,
                f206_descripcion as nombre
            FROM dbo.t206_mm_criterios_mayores 
            WHERE f206_id_plan = '014'";
        return Yii::$app->dbsql->createCommand($query)->queryAll();                
    }
    
    /**
     * Retorna el listado de los vendedores
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getVendedores($vendedores){  
        $andWhere = '';
        if(is_array($vendedores) && count($vendedores)>0){
            $andWhere = " WHERE tec.f200_rowid IN('".implode("','", $vendedores)."')";
        }
        $query = "
            SELECT 
                RTRIM(LTRIM(ven.f210_rowid_tercero)) as codigo,
                tec.f200_nit as identificacion, 
                tec.f200_razon_social as nombre,                 
                (CASE WHEN sup.f200_nit IS NULL THEN '98545776' ELSE sup.f200_nit END)  as codigo_supervisor
            FROM t200_mm_terceros tec
            LEFT JOIN t210_mm_vendedores ven ON ven.f210_rowid_tercero = tec.f200_rowid
            LEFT JOIN t200_mm_terceros sup ON ven.f210_rowid_terc_supervisor = sup.f200_rowid
            ".$andWhere."
            ORDER BY tec.f200_razon_social";
        
        return Yii::$app->dbsql->createCommand($query)->queryAll();                
    }
        
    /**
     * Retorna el listado de los supervisores
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getSupervisores(){                
        $query = "SELECT 
                RTRIM(LTRIM(f200_nit)) as codigo, 
                f200_razon_social as nombre
            FROM t200_mm_terceros 
            JOIN t210_mm_vendedores ON f210_rowid_terc_supervisor = f200_rowid
            WHERE f210_rowid_terc_supervisor IS NOT NULL
            GROUP BY f200_nit, f200_razon_social
            ORDER BY f200_razon_social";
        return Yii::$app->dbsql->createCommand($query)->queryAll();                       
    }    
    
    /**
     * Metodo para la consulta de inventario
     * 
     * @param array $referencias Array de las referencias
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getInventario($referencias){
        $andWhere = "";
        if(is_array($referencias) && count($referencias)>0){
            $andWhere .= " AND T120.f120_referencia IN ('".implode("','", $referencias)."')";
        }
        $fechaE = date('d-m-Y');            
        $fechaIn = date('d-m-Y', strtotime('-1 month'));        
        $query = "
            SET NOCOUNT ON;			
            Declare 
                @Nit_Proveedor char(13), 
                @Id_Sucursal char(3),
                @Id_Cia int, 
                @Id_bodega Char(13),
                @Fecha_I Datetime, 
                @Fecha_F Datetime,
                @Fecha_E Datetime

            set @Nit_proveedor =''
            set @Id_Sucursal = ''
            set @Id_Cia = 1
            Set @Id_bodega = '' 
            Set @Fecha_F = '".$this->fechaFn."'
            Set @Fecha_I = '".$this->fechaIn."'
            Set @Fecha_E = '".$this->fechaEn."'

            SELECT
                RTRIM(LTRIM(T120.f120_referencia)) as codigo_producto,
                T120.f120_id as item, 
                T120.f120_descripcion as nombre, 
                RTRIM(LTRIM(ISNULL(qry.Bodega,' '))) as bodega,
                RTRIM(LTRIM(ISNULL(qry.co,' '))) as co,
                RTRIM(LTRIM(ISNULL(qry_prov.proveedor,' '))) as nit_proveedor,
                ISNULL(qry_prov.Nombre_proveedor,' ') as proveedor,
                -- ISNULL(qry_prov.Sucursal_proveedor,' ') as sucursal_id,
                -- ISNULL(qry_prov.Nombre_Sucursal,' ') as sucursal,
                'UNID' AS unidad_medida,		
                CONVERT(INT, ISNULL(SUM(qry.Cantidad_Disponible),0)) As cantidad_disponible,
                CONVERT(INT, ISNULL(SUM(qry.Cantidad_comprometida),0)) As Cantidad_comprometida, 
                CONVERT(INT, ISNULL((CASE WHEN (SUM(qry.Cantidad_Disponible))=0 THEN 0 ELSE ((SUM(qry.Cantidad_Disponible)) - (SUM(qry.Cantidad_comprometida))) END),0)) AS cantidad	
            FROM (
            
            -- QUERY PEDIDOS
            SELECT                
                MPV.f431_rowid_item_ext AS RItem,
                T150.f150_id AS Bodega,
                T150.f150_id_co AS co,
                T150.F150_id_instalacion As Instalacion,
                SUM(MPV.F431_CANT1_PEDIDA) AS Cantidad_comprometida,
                0 AS Cantidad_Disponible
            FROM
            T430_CM_PV_DOCTO PV 
            
            INNER JOIN T431_CM_PV_MOVTO MPV
                ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

            LEFT JOIN t121_mc_items_extensiones T121
                ON T121.f121_rowid = MPV.f431_rowid_item_ext

            LEFT JOIN t120_mc_items T120
                ON T120.f120_rowid = T121.f121_rowid_item

            LEFT JOIN t150_mc_bodegas T150
                ON MPV.F431_ROWID_BODEGA = T150.f150_rowid

            WHERE
                PV.f430_id_cia = @Id_Cia
                AND PV.f430_ind_estado NOT IN(4,9)
                AND PV.f430_id_fecha BETWEEN @fecha_I AND @fecha_F
                AND PV.f430_ind_facturado = 0
                AND PV.f430_ind_remisionado = 0
                
            GROUP BY                                
                F150_id_instalacion,
                MPV.f431_rowid_item_ext,
                T150.f150_id_co,
                T150.f150_id

            UNION    

            --QUERY INVENTARIOS Act
                Select
                    T470.F470_ROWID_ITEM_EXT AS RItem,
                    T150.F150_ID As Bodega,
                    T150.f150_id_co AS co,
                    T470.F470_id_instalacion As Instalacion,	  
                    0 AS Cantidad_comprometida,
                    Sum(CASE WHEN T470.F470_IND_NATURALEZA = 1 THEN T470.F470_CANT_1 ELSE T470.F470_CANT_1*-1 END) As Cantidad_Disponible	  	  
                FROM
                    (T150_MC_BODEGAS T150 Right Outer Join
                    T470_CM_MOVTO_INVENT T470 
                    On T150.F150_ROWID = T470.F470_ROWID_BODEGA)
                -- Cruzamos con las bodegas principales
                JOIN 
                    (SELECT f150_descripcion AS bodega, f150_id AS id_bodega
                            FROM t150_mc_bodegas
                            JOIN t152_mc_bodega_grupo_bodega ON f152_rowid_bodega = f150_rowid
                            JOIN t151_mc_grupo_bodega ON f152_id_grupo_bodega = f151_id
                            WHERE f151_id IN ('BPP') ) AS GB
                    ON GB.id_bodega = T150.F150_ID
                WHERE
                    T470.F470_ID_CIA = @Id_CIA
                    -- AND T150.F150_ID = @Id_bodega
                    AND T470.F470_ID_fecha <= @Fecha_E 
                    AND T470.F470_IND_ESTADO_CM  <> '2' 
                    AND T470.F470_IND_TIPO_ITEM = 1	  	  
                GROUP BY
                    T470.F470_ROWID_ITEM_EXT,
                    T150.f150_id_co,
                    T150.F150_ID,
                    T470.F470_id_instalacion
                ) qry
	
            FULL JOIN t121_mc_items_extensiones T121
                On qry.ritem = T121.f121_rowid
	
            INNER JOIN t120_mc_items T120
                On T121.f121_rowid_item = T120.f120_rowid
                AND T120.F120_IND_TIPO_ITEM = '1'
	  
            --QUERY PROVEEDORES 
            LEFT JOIN (
                SELECT 
                    t470_comp.f470_rowid_item_ext AS RItem,
                    T200.f200_id AS Proveedor,
                    T200.f200_razon_social AS Nombre_Proveedor,
                    dtocmp.f451_id_sucursal_prov AS Sucursal_proveedor,
                    T202.f202_descripcion_sucursal as Nombre_Sucursal,
                    RANK() OVER (PARTITION BY t470_comp.f470_rowid_item_ext ORDER BY SUM(CASE WHEN t470_comp.f470_ind_naturaleza = 2 THEN (-t470_comp.f470_cant_1) ELSE t470_comp.f470_cant_1 END) DESC) as Rango
                FROM t470_cm_movto_invent t470_comp
		INNER JOIN t451_cm_docto_compras dtocmp ON dtocmp.f451_rowid_docto = t470_comp.f470_rowid_docto
		INNER JOIN t350_co_docto_contable t350_t1 ON t350_t1.f350_rowid = t470_comp.f470_rowid_docto
		LEFT JOIN t200_mm_terceros T200 ON dtocmp.f451_rowid_tercero_prov = T200.f200_rowid
		LEFT JOIN t202_mm_proveedores T202 ON dtocmp.f451_id_sucursal_prov = T202.f202_id_sucursal AND T200.f200_rowid = T202.f202_rowid_tercero
                WHERE 
                    t350_t1.f350_id_clase_docto IN (408, 409, 410, 411, 412, 413, 414, 417, 420)
                    -- AND (t470_comp.f470_id_fecha BETWEEN DATEADD(month, -6,@Fecha_F) AND @Fecha_F )
                    -- AND T200.f200_id = @Nit_proveedor
                    -- AND dtocmp.f451_id_sucursal_prov = @id_sucursal 
                GROUP BY 
                    t470_comp.f470_rowid_item_ext, 
                    T200.f200_id,
                    T200.f200_razon_social,
                    T202.f202_descripcion_sucursal,
                    dtocmp.f451_id_sucursal_prov
                ) qry_prov
                ON qry.ritem = qry_prov.ritem
                AND qry_prov.RANGO = 1
        
            WHERE 
                qry.Bodega IS NOT NULL 
                AND qry_prov.proveedor IS NOT NULL
                ".$andWhere."
	
            GROUP BY 
                T120.f120_referencia,
                T120.f120_id,
                T120.f120_descripcion, 
                T120.F120_IND_TIPO_ITEM,
                qry.Bodega,
                qry.co,
                qry_prov.proveedor,
                qry_prov.Nombre_proveedor,
                qry_prov.Sucursal_proveedor,
                qry_prov.Nombre_Sucursal

            ORDER BY 
                T120.f120_referencia, 
                T120.f120_id,
                qry.Bodega,
                qry_prov.proveedor,
                qry_prov.Nombre_proveedor,
                qry_prov.Sucursal_proveedor                
	";         
        return Yii::$app->dbsql->createCommand($query)->queryAll();
    }
    
    /**
     * Retorna las ventas de cada cliente deacuerdo al listado de referencias 
     * 
     * @param array $referencias Array de referencias
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getVentas($referencias){                                                                
        $andWhere = "";
        if(is_array($referencias) && count($referencias)>0){
            $andWhere .= " AND T120.f120_referencia IN ('".implode("','", $referencias)."')";
        }
        $query =
        "DECLARE @Nit_cliente CHAR(13), @C_Sucursal VARCHAR(3), @fecha_dto_ini DATETIME, @fecha_dto_fin DATETIME, @Id_Cia INT            

        SET @fecha_dto_ini = '" . $this->fechaIn . "'
        SET @fecha_dto_fin = '" . $this->fechaFn . "'
        SET @Id_Cia = 1

        SELECT
            f470_rowid AS id,
            ClienteFact.f200_rowid AS codigo_cliente,
            FV.f461_rowid_tercero_vendedor AS codigo_vendedor,            
            RTRIM(LTRIM(f350_id_tipo_docto)) + '-' + CONVERT(VARCHAR,(REPLICATE('0', (8 - LEN(f350_consec_docto))) + CONVERT(VARCHAR,f350_consec_docto)))  AS numero_documento,
            ClienteFact.f200_nit as nit,
            FV.F461_ID_SUCURSAL_FACT as sucursal,
            C201.F201_DESCRIPCION_SUCURSAL AS nombre_sucursal,
            T120.f120_descripcion as descripcion,
            CONVERT(VARCHAR,DOCFAC.f350_fecha,105) as fecha,
            RTRIM(LTRIM(T120.f120_referencia)) as codigo_producto,
            ISNULL(T132.f132_ultimo_costo_local,0) AS costo,                
            SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_cant_1*-1 ELSE MFV.f470_cant_1 END)) AS cantidad,
            SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea END)) AS total_item_vendido,
            MFV.f470_id_co_movto AS co,
            'UNID' AS unidad_medida,            
            (CASE WHEN RTRIM(LTRIM(f350_id_tipo_docto)) IN ('FV', 'FVE', 'FEV') THEN 0 ELSE 1 END) AS tipo,
	    RTRIM(LTRIM(f430_id_tipo_docto)) + '-' + CONVERT(VARCHAR,(REPLICATE('0', (8 - LEN(PV.f430_consec_docto))) + CONVERT(VARCHAR,PV.f430_consec_docto)))  AS numero_pedido

        FROM t470_cm_movto_invent MFV 

        INNER JOIN t461_cm_docto_factura_venta FV
            ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto

        INNER JOIN t350_co_docto_contable DOCFAC
            ON DOCFAC.f350_rowid = FV.f461_rowid_docto

        -- LEFT JOIN t121_mc_items_extensiones T121
        INNER JOIN t121_mc_items_extensiones T121
            ON T121.f121_rowid = MFV.f470_rowid_item_ext

        -- LEFT JOIN t120_mc_items T120
        INNER JOIN t120_mc_items T120
            ON T120.f120_rowid = T121.f121_rowid_item            

        -- LEFT JOIN t200_mm_terceros ClienteFact
        INNER JOIN t200_mm_terceros ClienteFact
            ON ClienteFact.f200_rowid = FV.f461_rowid_tercero_fact

        INNER JOIN T201_MM_CLIENTES C201
            ON ClienteFact.F200_ROWID = C201.F201_ROWID_TERCERO
            AND FV.F461_ID_SUCURSAL_FACT = C201.F201_ID_SUCURSAL    
            
        LEFT JOIN t431_cm_pv_movto
            ON f470_rowid_pv_movto = f431_rowid
			
        LEFT JOIN T430_CM_PV_DOCTO PV 
            ON f431_rowid_pv_docto = f430_rowid

        LEFT JOIN T132_Mc_items_instalacion T132 
            ON T132.f132_rowid_item_ext = MFV.f470_rowid_item_ext
            AND T132.f132_id_instalacion = MFV.f470_id_instalacion
            AND T132.f132_id_cia =  @Id_CIA
            AND T132.f132_id_instalacion is not null 

        WHERE
            FV.f461_id_cia = @Id_Cia 
            AND DOCFAC.f350_id_tipo_docto IN ('FV', 'NC', 'NCD', 'NCE', 'FVE', 'NFC', 'NEF', 'NDF', 'FEV', 'NDE', 'NED', 'NFE', 'DEF', 'NCF')
            AND DOCFAC.f350_fecha BETWEEN @fecha_dto_ini AND @fecha_dto_fin            
            ".$andWhere."

        GROUP BY
            f470_rowid,
            ClienteFact.f200_rowid,
            f461_rowid_docto,
            ClienteFact.f200_nit, 
            FV.F461_ID_SUCURSAL_FACT, 
            C201.F201_DESCRIPCION_SUCURSAL, 
            T120.f120_referencia, 
            T120.f120_descripcion, 
            DOCFAC.f350_fecha,
            FV.f461_rowid_tercero_vendedor,
            MFV.f470_id_co_movto,
            T132.f132_ultimo_costo_local,
            f350_id_tipo_docto,
            f350_consec_docto,
            f430_id_tipo_docto,
            PV.f430_consec_docto
            
        ORDER BY
            ClienteFact.f200_nit, 
            FV.F461_ID_SUCURSAL_FACT, 
            T120.f120_referencia, 
            T120.f120_descripcion, 
            DOCFAC.f350_fecha
            ";   
			
        $arrayResultQuery = Yii::$app->dbsql->createCommand($query)->queryAll();        
        return $arrayResultQuery;
    }
}
