<?php

namespace app\controllers;

use Yii;
use app\models\Referencias;
use app\models\ReferenciasSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\web\UploadedFile;
use Akeneo\Component\SpreadsheetParser\SpreadsheetParser;
use app\models\Logcargas;
use yii\bootstrap\Html;

/**
 * ReferenciasController implements the CRUD actions for Referencias model.
 */
class ReferenciasController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Referencias models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ReferenciasSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Referencias model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Referencias model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Referencias();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        } else {
            return $this->render('create', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Updates an existing Referencias model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        } else {
            return $this->render('update', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Deletes an existing Referencias model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Referencias model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Referencias the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Referencias::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }
    
    public function actionUploadfile(){
        $model = new Referencias();
        $model->scenario = 'importar';
        $banderaCarga = false;
        $error = false;
        
        if ($model->load(Yii::$app->request->post())) {            
            $file = UploadedFile::getInstance($model, 'fileInput');   
            $model->fileInput = $file;
            if ($model->fileInput && $model->validate()) {
                $arrayExt = explode('.', $file->name);
                $extension = end($arrayExt);
                $blockDataInsert = [];
                $dateFile = date('YmdHis');
                $basePath = Yii::getAlias('@app');
                $pathfile = $basePath . DIRECTORY_SEPARATOR.'uploads'
                        .DIRECTORY_SEPARATOR.'carga_' . $dateFile
                        . '.' . $extension;
                $fileType = $file->type;
                $fileName = $file->tempName;                
                $model->fileInput->saveAs($pathfile);                
                if ($fileType === 'application/vnd.ms-excel' || $fileType ===
                            'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet') {
                    //Leemos el archivo ------------------------------------
                    $workbook = SpreadsheetParser::open($pathfile);
                    $myWorksheetIndex = $workbook->getWorksheetIndex('myworksheet');                    
                    $sw = 0;
                    //Leemos fila por fila ---------------------------------
                    ob_start();
                    $fila = 1;
                    Logcargas::deleteAll(['usuario_id'=>\Yii::$app->user->getId()]);                                       
                    foreach ($workbook->createRowIterator($myWorksheetIndex) as $rowIndex => $values) {                                                                          
                        if($fila != 1){   
                            $data = $model->limpiarData($values);     
                            $data[] =  $model->distribuidor_id;
                            $data[] =  $model->proveedor_id;  
                            $referencias = new Referencias;
                            //$maquina->scenario = 'carga';
                            $referencias->ref_proveedor = $data[0];
                            $referencias->ref_descripcion_proveedor = $data[1];
                            $referencias->categoria_proveedor = $data[2];
                            $referencias->ref_distribuidor = $data[3];
                            $referencias->ref_descripcion_distribuidor = $data[4];
                            $referencias->categoria_distribuidor = $data[5];
                            $referencias->distribuidor_id = $model->distribuidor_id;
                            $referencias->proveedor_id = $model->proveedor_id;                            
                            if (!$referencias->validate()) {
                                $attributes = $model->getAttributes();
                                $arrayReplace = array_keys($attributes);                                
                                $arrayReplace[] = '[';
                                $arrayReplace[] = ']';
                                $arrayReplace[] = '{';
                                $arrayReplace[] = '}';
                                $arrayReplace[] = '"';
                                $arrayReplace[] = ':';
                                //Eliminamos la columna id ---------------------
                                unset($arrayReplace[0]);
                                $error = true;
                                $msg = json_encode($referencias->getErrors(),
                                        JSON_UNESCAPED_UNICODE);
                                $cleanMsg = str_replace($arrayReplace, '', $msg);                                
                                //Guardamos el error ---------------------------
                                $log = new Logcargas;                                
                                $log->fila = $fila;
                                $log->mensaje = $cleanMsg;
                                $log->usuario_id = \Yii::$app->user->getId();
                                $log->save();
                            }else{                                                                
                                $dataString = str_replace("''", "NULL",
                                            implode("','", $data));                                
                                $blockDataInsert[] = '('                                            
                                            . "'" . $dataString . "'" . ')';                                      
                                if (count($blockDataInsert) == 500) {
                                    $sqlInsert = 'INSERT INTO referencias'
                                        . ' (ref_proveedor, '
                                        . 'ref_descripcion_proveedor, '
                                        . 'categoria_proveedor, '
                                        . 'ref_distribuidor, '
                                        . 'ref_descripcion_distribuidor, '
                                        . 'categoria_distribuidor, '
                                        . 'distribuidor_id, '
                                        . 'proveedor_id ) '
                                        . 'VALUES '
                                        . implode(',', $blockDataInsert).'';
                                    try {
                                        \Yii::$app->db->createCommand($sqlInsert)->execute();
                                        $blockDataInsert = [];
                                    } catch (\Exception $exc) {
                                        $log = new Logcargas;                                
                                        $log->fila = $fila;
                                        $log->mensaje = $exc->getTraceAsString();
                                        $log->usuario_id = \Yii::$app->user->getId();
                                        $log->save();
                                    }                                    
                                }
                            }                            
                        }
                        $fila++;
                    }
                    //Gaurdamos los restantes
                    if (count($blockDataInsert) > 0) {                          
                        $sqlInsert = 'INSERT INTO referencias'
                            . ' (ref_proveedor, '
                            . 'ref_descripcion_proveedor, '
                            . 'categoria_proveedor, '
                            . 'ref_distribuidor, '
                            . 'ref_descripcion_distribuidor, '
                            . 'categoria_distribuidor, '
                            . 'distribuidor_id, '
                            . 'proveedor_id ) '
                            . 'VALUES '
                            . implode(',', $blockDataInsert).'';
                        try {
                            \Yii::$app->db->createCommand($sqlInsert)->execute();
                            $blockDataInsert = [];
                        } catch (\Exception $exc) {
                            $log = new Logcargas;                                
                            $log->fila = 0;
                            $log->mensaje = $exc->getMessage();
                            $log->usuario_id = \Yii::$app->user->getId();
                            $log->save();
                        }                                    
                    }
                    
                    if($error){
                        Yii::$app->session->setFlash('danger', 
                            'Se cargó el archivo con errores, '
                            .Html::a(Yii::t('app', 'Ver Errores'),
                                ['logcargas/index'],['target' => '_blank']));
                    }else{
                        Yii::$app->session->setFlash('success', 'Se cargó el archivo con éxito');
                    }
                }else{
                    Yii::$app->session->setFlash('danger', 'Extensión inválida');
                }                                                
            }           
        }
        return $this->render('uploadfile',['model'=>$model,'bandera'=>$banderaCarga]);
    }
}
