<?php

namespace app\controllers;

use Yii;
use yii\web\Controller;
use app\models\Ventas;
use app\models\Compras;
use app\models\Inventarios;
use app\models\Parametrosreportes;

class ReportesController extends Controller {    

    /**
     * Accion del reporte de ventas
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function actionVentas(){
        $model = new Ventas();
        $valueListAdd = '';
        $valueListDefecto = '';
        //Obtenemos los parametros de la busqueda ------------------------------
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        //Consutlamos los parametros -------------------------------------------
        $params = Parametrosreportes::find()
                    ->where(['reporte'=>Ventas::NM_REPORTE, 
                        'usuario_id'=>Yii::$app->user->identity->id])
                    ->one();    
        //proveedor ------------------------------------------------------------        
        $idProveedor = null;                 
        if(isset($_REQUEST['proveedor_id'])){              
            $idProveedor = $_REQUEST['proveedor_id'];             
            $session = Yii::$app->session;
            $session->set('proveedor_id', $idProveedor);
        }        
        //----------------------------------------------------------------------
        if(empty($params)){
            $valueListDefecto = $model->listParamsDefecto();            
            $valueListAdd = $model->listParamsAdd();                            
            //Guardamos los parametros -----------------------------------------
            $paramsReporte = new Parametrosreportes;
            $paramsReporte->reporte = Ventas::NM_REPORTE;
            $paramsReporte->paramsDefecto = $valueListDefecto;
            $paramsReporte->paramsAdd = $valueListAdd;
            $paramsReporte->save();
        }else{
            $valueListDefecto = $params->paramsDefecto;            
            $valueListAdd = $params->paramsAdd;
        }                            
        //Tomamos el nuevo orden de las busquedas ------------------------------
        if(isset($_POST['listAdd']) && isset($_POST['listDefecto']) 
                && !empty($_POST['listDefecto'])){
            $valueListAdd = $_POST['listAdd'];
            $valueListDefecto = $_POST['listDefecto'];    
            //Guardamos el orden en la bd --------------------------------------
            Parametrosreportes::updateAll(
                ["paramsDefecto" => $valueListDefecto, "paramsAdd" => $valueListAdd], 
                ['reporte'=>Ventas::NM_REPORTE, 
                    'usuario_id'=>Yii::$app->user->identity->id]);            
        }        
        //Sacamos el listado de los parametros ---------------------------------
        $listParamsAdd = $model->getTempList($valueListAdd);
        $listParamsDefecto = $model->getTempList($valueListDefecto);
        $columns =  $model->getListGrid($valueListDefecto);        
        //----------------------------------------------------------------------        
        $dataProvider = $model->reporteVentas($valueListDefecto, $queryParams);

        return $this->render('ventas', [
            'model' => $model,
            'dataProvider' => $dataProvider,
            'listParamsDefecto' => $listParamsDefecto,
            'listParamsAdd' => $listParamsAdd,
            'valueListAdd' => $valueListAdd,
            'valueListDefecto' => $valueListDefecto,
            'columns' => $columns,
            'idProveedor' => $idProveedor,
        ]);
    }        
    
    /**
     * Accion del reporte de compras
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function actionCompras(){
        $model = new Compras();
        $valueListAdd = '';
        $valueListDefecto = '';
        //Obtenemos los parametros de la busqueda ------------------------------
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        //Consutlamos los parametros -------------------------------------------
        $params = Parametrosreportes::find()
                    ->where(['reporte'=>Compras::NM_REPORTE, 
                        'usuario_id'=>Yii::$app->user->identity->id])
                    ->one();   
        //proveedor ------------------------------------------------------------        
        $idProveedor = null;                 
        if(isset($_REQUEST['proveedor_id'])){              
            $idProveedor = $_REQUEST['proveedor_id'];             
            $session = Yii::$app->session;
            $session->set('proveedor_id', $idProveedor);
        }        
        //----------------------------------------------------------------------

        if(empty($params)){
            $valueListDefecto = $model->listParamsDefecto();            
            $valueListAdd = $model->listParamsAdd();                            
            //Guardamos los parametros -----------------------------------------
            $paramsReporte = new Parametrosreportes;
            $paramsReporte->reporte = Compras::NM_REPORTE;
            $paramsReporte->paramsDefecto = $valueListDefecto;
            $paramsReporte->paramsAdd = $valueListAdd;
            $paramsReporte->save();
        }else{
            $valueListDefecto = $params->paramsDefecto;            
            $valueListAdd = $params->paramsAdd;
        }
        //Tomamos el nuevo orden de las busquedas ------------------------------
        if(isset($_POST['listAdd']) && isset($_POST['listDefecto']) 
                && !empty($_POST['listDefecto'])){
            $valueListAdd = $_POST['listAdd'];
            $valueListDefecto = $_POST['listDefecto'];    
            //Guardamos el orden en la bd --------------------------------------
            Parametrosreportes::updateAll(
                ["paramsDefecto" => $valueListDefecto, "paramsAdd" => $valueListAdd], 
                ['reporte'=>Compras::NM_REPORTE, 
                    'usuario_id'=>Yii::$app->user->identity->id]);            
        }        
        //Sacamos el listado de los parametros ---------------------------------
        $listParamsAdd = $model->getTempList($valueListAdd);
        $listParamsDefecto = $model->getTempList($valueListDefecto);
        $columns =  $model->getListGrid($valueListDefecto);
        //----------------------------------------------------------------------        
        $dataProvider = $model->reporteCompras($valueListDefecto, $queryParams);

        return $this->render('compras', [
            'model' => $model,
            'dataProvider' => $dataProvider,
            'listParamsDefecto' => $listParamsDefecto,
            'listParamsAdd' => $listParamsAdd,
            'valueListAdd' => $valueListAdd,
            'valueListDefecto' => $valueListDefecto,
            'columns' => $columns,
            'idProveedor' => $idProveedor,
        ]);
    }
    
    /**
     * Accion del reporte de inventario
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function actionInventario(){
        $model = new Inventarios();        
        //Obtenemos los parametros de la busqueda ------------------------------
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());                                                        
        //proveedor ------------------------------------------------------------        
        $idProveedor = null;                 
        if(isset($_REQUEST['proveedor_id'])){              
            $idProveedor = $_REQUEST['proveedor_id'];             
            $session = Yii::$app->session;
            $session->set('proveedor_id', $idProveedor);
        }        
        //----------------------------------------------------------------------
        $dataProvider = $model->reporteInventario($queryParams);

        return $this->render('inventario', [
            'model' => $model,
            'dataProvider' => $dataProvider,            
        ]);
    }
    
    
    /**
     * 
     * @return type
     */
    public function actionDetallecliente(){
        $fechaVenta = '';        
        if (isset($_POST['expandRowKey'])) {
            $model = \app\models\Maquinas::findOne($_POST['expandRowKey']); 
            $searchModel = new MaquinasSearch();
            $searchModel->nit = $model->nit;            
            $searchModel->fecha_venta = $_POST['fecha_venta']; 
            $dataProvider = $searchModel->detalleVenta();
            return $this->renderPartial('_detalleCliente', ['dataProvider' => $dataProvider]);
        }        
    }

}

?>
