<?php

namespace app\controllers;

use app\models\UsrBranchOffice;
use Yii;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\filters\VerbFilter;
use app\models\LoginForm;
use app\models\ContactForm;
use app\models\Ventas;
use app\models\Compras;
use app\models\Proveedores;
use app\components\B2BISiesa;
use app\models\Referencias;
use yii\helpers\ArrayHelper;
use app\models\Usuarios;
use app\models\Proyecciones;

class SiteController extends Controller {

    /**
     * @inheritdoc
     */
    public function behaviors() {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'only' => ['logout'],
                'rules' => [
                    [
                        'actions' => ['logout'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'logout' => ['post'],
                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function actions() {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    /**
     * Displays homepage.
     *
     * @return string
     */
    public function actionIndex() {
        $ventas = new Ventas;
        $compras = new Compras;
        $modelProv = new Proveedores;
        $idProveedor = null;
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');

        if ($modelProv->load(Yii::$app->request->post())) {
            if(isset($_POST['Proveedores']['id'])
                && !empty($_POST['Proveedores']['id'])
                && is_numeric($_POST['Proveedores']['id'])){
                $idProveedor = $_POST['Proveedores']['id'];
                $modelProv->id = $idProveedor;
            }
        }

//        $ventasInfo = $ventas->getInfoDasboard($idProveedor);
//        $comprasInfo = $compras->getInfoDasboard($idProveedor);
        return $this->render('index',[
//            'ventasInfo'=>$ventasInfo,
//            'comprasInfo'=>$comprasInfo,
            'modelProv'=>$modelProv,
            'idProveedor'=>$idProveedor,
            'branchOffice' => $branchOffice
        ]);
    }

    /**
     * Login action.
     *
     * @return string
     */
    public function actionLogin() {
        if (!Yii::$app->user->isGuest) {
            return $this->goHome();
        }

        $model = new LoginForm();
        if ($model->load(Yii::$app->request->post()) && $model->login()) {
            return $this->goBack();
        }
        return $this->render('login', [
                    'model' => $model,
        ]);
    }    

    /**
     * Logout action.
     *
     * @return string
     */
    public function actionLogout() {
        Yii::$app->user->logout();

        return $this->goHome();
    }

    /**
     * Displays contact page.
     *
     * @return string
     */
    public function actionContact() {
        $model = new ContactForm();
        if ($model->load(Yii::$app->request->post()) && $model->contact(Yii::$app->params['adminEmail'])) {
            Yii::$app->session->setFlash('contactFormSubmitted');

            return $this->refresh();
        }
        return $this->render('contact',
                        [
                    'model' => $model,
        ]);
    }

    /**
     * Displays about page.
     *
     * @return string
     */
    public function actionAbout() {
        return $this->render('about');
    }
    
    

    /**
     * Metodo para cambio de contraseña          
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2016 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */   
    public function actionRepass() {        
        $this->layout = 'main-login';
        $model = new LoginForm(); 
        $model->scenario = 'scnrepass';
        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            $model->sendMailRePass();
        }
        return $this->render('repass', [
                    'model' => $model,
        ]);
    } 
    
    /**
     * toal ventas del dashboard
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2016 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function actionTotalventasmes(){
        $andWhere = '';
        $b2bSiesa = new B2BISiesa;
        $fechaFin = date('d-m-Y'); 
		$fechaFin = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d')-1, date('Y')));
        $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));
        $idProveedor = Yii::$app->request->post('proveedor');
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $ventas = new Ventas;      
        $modelReferencias = $ventas->getReferences($idProveedor, $branchOffice);
        $referencias = ArrayHelper::map($modelReferencias, 
                    'ref_distribuidor', 'ref_distribuidor');
        $clnVenta = $b2bSiesa->getTotalVenta($fechaIn, $fechaFin, $referencias, $branchOffice);
        return json_encode($clnVenta);
    }
    
    /**
     * Cliente mayor venta dashboard
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2016 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function actionClientemayorventa(){
        $andWhere = '';
        $b2bSiesa = new B2BISiesa;
        $fechaFin = date('d-m-Y');    
		$fechaFin = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d')-1, date('Y')));
        $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));
        $idProveedor = Yii::$app->request->post('proveedor');
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $ventas = new Ventas;      
        $modelReferencias = $ventas->getReferences($idProveedor, $branchOffice);
        $referencias = ArrayHelper::map($modelReferencias, 
                    'ref_distribuidor', 'ref_distribuidor');
        $clnVenta = $b2bSiesa->getClienteMayorVenta($fechaIn, $fechaFin, $referencias, $branchOffice);
        return json_encode($clnVenta);
    }
    
    /**
     * Referencia mayor venta dashboard
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2016 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function actionReferenciamayorventa(){
        $andWhere = '';
        $b2bSiesa = new B2BISiesa;
        $fechaFin = date('d-m-Y');  
		$fechaFin = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d')-1, date('Y')));
        $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));
        $idProveedor = Yii::$app->request->post('proveedor');
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $ventas = new Ventas;      
        $modelReferencias = $ventas->getReferences($idProveedor, $branchOffice);
        $referencias = ArrayHelper::map($modelReferencias, 
                    'ref_distribuidor', 'ref_distribuidor');
        $refVenta = $b2bSiesa->getRefefrenciaMayorVenta($fechaIn, $fechaFin, $referencias, $branchOffice);
        return json_encode($refVenta);
    }
    
    /**
     * Compra mayor venta dashboard
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2016 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function actionTotalcomprasmes(){
        $andWhere = '';
        $b2bSiesa = new B2BISiesa;
        $fechaFin = date('d-m-Y'); 
		$fechaFin = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d')-1, date('Y')));
        $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));
        $idProveedor = Yii::$app->request->post('proveedor');
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $compras = new Compras;      
        $modelReferencias = $compras->getReferences($idProveedor, $branchOffice);
        $referencias = ArrayHelper::map($modelReferencias, 
                    'ref_distribuidor', 'ref_distribuidor');
        $totCompra = $b2bSiesa->getTotalCompra($fechaIn, $fechaFin, $referencias);
        return json_encode($totCompra);
    }
    
    /**
     * ventas ultimos meses dashboard
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2016 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function actionVentasultimosmeses(){
        $andWhere = '';
        $b2bSiesa = new B2BISiesa;
        $fechaFin = date('d-m-Y'); 
		$fechaFin = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d')-1, date('Y')));
        $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));
        $idProveedor = Yii::$app->request->post('proveedor');
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $ventas = new Ventas;      
        $modelReferencias = $ventas->getReferences($idProveedor, $branchOffice);
        $referencias = ArrayHelper::map($modelReferencias, 
                    'ref_distribuidor', 'ref_distribuidor');
        $ultVentas = $b2bSiesa->getUltimasVentasMeses($referencias, 3, $branchOffice);
        
        //Traemos las metas del proveedor --------------------------------------
        $proyecciones = [];
        $arrayProy = [];        
        if(is_null($idProveedor)){
            $proyecciones = Proyecciones::find()->where(
                ['usuario_id'=>Yii::$app->user->identity->id])->asArray()->all();            
        }else{
            $usuario = Usuarios::findOne(['proveedor_id'=>$idProveedor]);
            if(!is_null($usuario)){
                $proyecciones = Proyecciones::find()
                    ->where(['usuario_id'=>$usuario->id])->asArray()->all();            
            }            
        }
        
        if(count($proyecciones)>0){
            $arrayProy = ArrayHelper::index($proyecciones, 'mes', ['anio']);
        }        
        //----------------------------------------------------------------------
        $dataMeses = [];
        $dataTotales = [];
        if(count($ultVentas)>0){ 
            $meses = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 
            'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
            foreach ($ultVentas as $anio => $datos) { 
                foreach ($datos as $data) {                 
                    $dataMeses[$anio][] = $meses[$data['mes']-1];                
                    $dataTotales[$anio][] = $data['total'];                               
                }            
            }            
        }
        $infoDashboard['meses_grafica']= $dataMeses; 
        $infoDashboard['totales_grafica']= $dataTotales; 
        $infoDashboard['proyecciones']= $arrayProy; 
        //TotalVenta -----------------------------------------------------------
        $infoDashboard['totalVeta'] = 0;
        $infoDashboard['cliente_nit'] = '';
        $infoDashboard['cliente_razon_social'] = '';
        $infoDashboard['cliente_total'] = '0';
        $infoDashboard['referencia_ref'] = '';
        $infoDashboard['referencia_desc'] = '';
        $infoDashboard['referencia_total'] = '0';
        
        if(isset($refVenta['referencia']) && isset($refVenta['descripcion']) && isset($refVenta['total'])){
            $infoDashboard['referencia_ref'] = $refVenta['referencia'];
            $infoDashboard['referencia_desc'] = $refVenta['descripcion'];
            $infoDashboard['referencia_total'] = $refVenta['total'];
        }
        
        return $this->renderAjax('_saleslastmonths', [
                'ventasInfo'=>$infoDashboard,
                'idProveedor'=>$idProveedor,
            ]);
    }
    
    /**
     * compras ultimos meses dashboard
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2016 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function actionComprasultimosmeses(){
        $andWhere = '';
        $b2bSiesa = new B2BISiesa;
        $fechaFin = date('d-m-Y');
		$fechaFin = date('d-m-Y', mktime(0, 0, 0, date('m'), date('d')-1, date('Y')));
        $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));
        $idProveedor = Yii::$app->request->post('proveedor');
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $compras = new Compras;      
        $modelReferencias = $compras->getReferences($idProveedor, $branchOffice);
        $referencias = ArrayHelper::map($modelReferencias, 
                    'ref_distribuidor', 'ref_distribuidor');                       
        $ultCompras = $b2bSiesa->getUltimasComprasMeses($referencias, 3);
        //Traemos las metas del proveedor --------------------------------------
        $proyecciones = [];
        $arrayProy = [];        
        if(is_null($idProveedor)){
            $proyecciones = Proyecciones::find()->where(
                ['usuario_id'=>Yii::$app->user->identity->id])->asArray()->all();            
        }else{
            $usuario = Usuarios::findOne(['proveedor_id'=>$idProveedor]);
            if(!is_null($usuario)){
                $proyecciones = Proyecciones::find()
                    ->where(['usuario_id'=>$usuario->id])->asArray()->all();            
            }            
        }
        
        if(count($proyecciones)>0){
            $arrayProy = ArrayHelper::index($proyecciones, 'mes', ['anio']);
        }        
        //----------------------------------------------------------------------
        $dataMeses = [];
        $dataTotales = [];
        if(count($ultCompras)>0){ 
            $meses = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 
            'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
            foreach ($ultCompras as $anio => $datos) { 
                foreach ($datos as $data) {                                                          
                    $dataMeses[$anio][] = $meses[$data['mes']-1];                
                    $dataTotales[$anio][] = $data['total'];                
                }            
            }            
        }
        $infoDashboard['meses_grafica']= $dataMeses; 
        $infoDashboard['totales_grafica']= $dataTotales;
        $infoDashboard['proyecciones']= $arrayProy; 
        //TotalVenta -----------------------------------------------------------
        $infoDashboard['totalCompra'] = 0;
        
        return $this->renderAjax('_purchaseslastmonths', [
            'comprasInfo'=>$infoDashboard,
            'idProveedor'=>$idProveedor,
        ]);
    }
}
