<?php

namespace app\controllers;

use app\components\B2BISiesa;
use app\models\Configpdv;
use app\models\Distribuidores;
use app\models\Proveedores;
use app\models\Usuarios;
use Yii;
use app\models\UsrBranchOffice;
use app\models\UsrBranchOfficeSearch;
use yii\helpers\ArrayHelper;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\Json;


/**
 * UsrpdvController implements the CRUD actions for UsrBranchOffice model.
 */
class UsrBranchOfficeController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all UsrBranchOffice models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new UsrBranchOfficeSearch();
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        $users = ArrayHelper::map(Usuarios::find()->all(), 'id', 'nombre');

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'users' => $users,
        ]);
    }

    /**
     * Displays a single UsrBranchOffice model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new UsrBranchOffice model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new UsrBranchOffice();
        $users = ArrayHelper::map(Usuarios::find()->all(), 'id', 'nombre');
        $providers = [];
        $cities = ArrayHelper::map($model->getRequestedCitiesDrop(), 'code', 'name');

        if ($model->load(Yii::$app->request->post())) {
            $post = Yii::$app->request->post();
            $model->code_branch_office = implode(',', $post['UsrBranchOffice']['code_branch_office']);
            $model->code_city = implode(',', $post['UsrBranchOffice']['code_city']);
            
            $model->save();
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
            'users' => $users,
            'cities' => $cities,
            'providers' => $providers
        ]);
    }

    /**
     * Updates an existing UsrBranchOffice model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $users = ArrayHelper::map(Usuarios::find()->all(), 'id', 'nombre');
        $providers = [];
        $cities = ArrayHelper::map($model->getRequestedCitiesDrop(), 'code', 'name');

        if ($model->load(Yii::$app->request->post())) {
            $post = Yii::$app->request->post();
            $b2bSiesa = new B2BISiesa();
            $city = $b2bSiesa->getCodeCities($post['UsrBranchOffice']['name_city']);
            $prov = Proveedores::find()->where(['nit' => $post['UsrBranchOffice']['code_provider']])->one();

            if (empty($prov) == false) {
                $model->id_provider = $prov->id;
            }

            $model->code_branch_office = implode(',', $post['UsrBranchOffice']['code_branch_office']);
            $model->code_city = implode(',', $post['UsrBranchOffice']['code_city']);
            $model->code_branch_office = implode(',', $post['UsrBranchOffice']['code_branch_office']);

            $model->save();
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
            'users' => $users,
            'cities' => $cities,
            'providers' => $providers
        ]);
    }

    /**
     * Deletes an existing UsrBranchOffice model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the UsrBranchOffice model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return UsrBranchOffice the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = UsrBranchOffice::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }

    /**
     * Sucursales AJAX Response of BD SIESA
     */
    public function actionGetBranchesOffice() {
        $out = [];
        $params = $_POST['depdrop_all_params'];
        $selected = '';

        if (isset($params['usrbranchoffice-code_provider'])) {
            $model = new UsrBranchOffice();
            $branchesOffice = $model->getSucursalesDrop($params['usrbranchoffice-code_provider']);

            if(empty($params['usrbranchoffice-id']) == false){
                $selected = explode(',', UsrBranchOffice::find()->where(['id' => $params['usrbranchoffice-id']])->one()->code_branch_office);
            }

            if(count($branchesOffice)>0){
                foreach ($branchesOffice as $value) {
                    $out[] = ['id' => $value['cod_sucursal'], 'name' => $value['nombre_sucursal']];
                }
            }
            echo Json::encode(['output'=>$out, 'selected'=> $selected]);
            return;
        }
        echo Json::encode(['output'=>'', 'selected'=>'']);
        return;
    }

    /**
     * Filtro por proveedor y sucursal
     */
    public function actionGetBranchUser(){
        $result = [
            'providers' => [],
            'cities' => [],
            'branches_office' => []
        ];

        $branchOffice = UsrBranchOffice::find()->where(['id_user' =>  \Yii::$app->user->identity->id])->all();
        $cont = 0;

        if(empty($branchOffice) == false){
            foreach ($branchOffice as $value){
                if(empty($value->code_branch_office) == false) {
                    $branches = explode(',', $value->code_branch_office);
                    $cities = explode(',', $value->name_city);

                    //$result['providers'][] = $value->code_provider;
                    $result['providers_nit'][] = $value->code_provider;
                    $result['cities'][] = $value->code_city;
                    $cont++;
                    if(empty($branches) == false){
                        foreach ($branches as $branch){
                            $result['branches_office'][] = $branch;
                            $cont++;
                        }
                    }
                    if(empty($cities) == false){
                        foreach ($cities as $city){
                            $result['cities_name'][] = $city;
                        }
                    }
                }
            }
        }
        return $result;
    }
    
    /**
     * 
     * @param type $q
     * @param type $id
     * @return type
     */
    public function actionListclients($q = null, $id = null) {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $out = ['results' => ['id' => '', 'text' => '']];
        if (!is_null($q)) {
            $model = new UsrBranchOffice;
            $result = $model->getClients($q);
            $data = [];
            if(count($result)>0){
                foreach ($result as $value) {
                    $data[] = ['id' => $value['code'], 'text' => $value['business_name']];
                }
            }
            $out['results'] = $data;
        }

        return $out;
    }
}

