<?php

namespace app\controllers;

use Yii;
use app\models\Usuarios;
use app\models\UsuariosSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

/**
 * UsuariosController implements the CRUD actions for Usuarios model.
 */
class UsuariosController extends Controller {

    /**
     * @inheritdoc
     */
    public function behaviors() {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Usuarios models.
     * @return mixed
     */
    public function actionIndex() {
        $searchModel = new UsuariosSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
                    'searchModel' => $searchModel,
                    'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Usuarios model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($id = null) {
        if (\Yii::$app->user->identity->isSuperAdmin() || \Yii::$app->user->identity->isAdmin()) {
            if(empty($id) || is_null($id)){
                $id = \Yii::$app->user->getId();
            }            
        }else{
            $id = \Yii::$app->user->getId();            
        }
        $model = $this->findModel($id);  
        return $this->render('view', [
                    'model' => $model,
        ]);
    }

    /**
     * Creates a new Usuarios model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate() {
        $model = new Usuarios();        
        $model->scenario = "crearuUs";        
        
        if ($model->load(Yii::$app->request->post())) {              
            if(!empty($model->proveedor_id)){
                $model->scenario = "crearuUsProv";        
            }
            
            if(!empty($model->distribuidor_id)){
                $model->scenario = "crearuUsDis";        
            }
            if ($model->validate()) {                
                $model->password = md5($model->password);
                $model->password_repeat = md5($model->password_repeat);  
                
                if ($model->save()) {
                    return $this->redirect(['view', 'id' => $model->id]);
                }
            } else {
                return $this->render('create', ['model' => $model]);
            }
        } else {
            return $this->render('create', ['model' => $model]);
        }
    }        

    /**
     * Updates an existing Usuarios model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id = null) {
        if (\Yii::$app->user->identity->isSuperAdmin() || \Yii::$app->user->identity->isAdmin()) {
            if(empty($id) || is_null($id)){
                $id = \Yii::$app->user->getId();
            }            
        }else{
            $id = \Yii::$app->user->getId();            
        }
        $model = $this->findModel($id);            

        $changeLogin = false;
        $beforePassword = $model->password;
        $OldLogin = $model->username;
        $model->password = "";                        
        
        $model->scenario = "updateUs";        

        if ($model->load(Yii::$app->request->post())) {
            //VERIFICO SI SE CAMBIO EL LOGIN
            if ($OldLogin != $model->username) {
                $changeLogin = true;
            }
            $password = '';
            $password_repeat = '';
            if (isset($_POST['Usuarios']['password']) && isset($_POST['Usuarios']['password_repeat'])) {                
                $password = $_POST['Usuarios']['password'];
                $password_repeat = $_POST['Usuarios']['password_repeat'];
            }
            $password = trim($password);
            $password_repeat = trim($password_repeat);
            if (empty($password)) {
                $model->password = $model->password_repeat = $beforePassword;
            } else {
                $model->password = md5($password);
                $model->password_repeat = md5($password_repeat);
            }

            if(!empty($model->proveedor_id)){
                $model->scenario = "updateUsProv";        
            }
            
            if(!empty($model->distribuidor_id)){
                $model->scenario = "updateUsDis";        
            }
            
            if ($model->validate()) {
                if ($model->save()) {
                    if ($changeLogin) {
                        Yii::$app->user->logout();
                        return $this->redirect(['site/login']);
                    }                    
                    return $this->redirect(['view', 'id' => $model->id]);
                }
            } else {
                $model->password = $password;
                $model->password_repeat = $password_repeat;
                return $this->render('create', ['model' => $model]);
            }                        
            return $this->redirect(['view', 'id' => $model->id]);
        } else {
            return $this->render('update', ['model' => $model]);
        }
    }

    /**
     * Deletes an existing Usuarios model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($id) {
        $this->findModel($id)->delete();
        return $this->redirect(['index']);
    }

    /**
     * Finds the Usuarios model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Usuarios the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id) {
        if (($model = Usuarios::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }

}
