<?php

namespace app\models;

use Yii;
use yii\base\Model;
use app\components\B2BISiesa;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "compras".
 *
 * @property integer $id 
 * @property string $referencia_dist
 * @property string $desc_ref_dist
 * @property string $fecha
 * @property string $factura
 * @property string $cantidad_pedida
 * @property string $cantidad_entrada
 * @property double $total
 * @property string $tipo
 * @property double $costo
 * @property string $unidad_medida
 * @property string $pedido
 * @property string $co
 * @property string $oc 
 * 
 */
class Compras extends Model {
    
    const NM_REPORTE = 'compras';
                
    public $referencia;
    public $referencia_dist;
    public $desc_ref_dist;
    public $referencia_prov;
    public $desc_ref_prov;
    public $categoria_prov;
    public $categoria_dist;
    public $fecha;    
    public $cantidad_pedida;
    public $cantidad_entrada;    
    public $total;
    public $precio_uni;
    //public $tipo;
    //public $costo;
    public $unidad_medida;
    public $oc;
    public $co;  
    public $mes;
    public $anio;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [            
            [['cantidad_pedida','cantidad_entrada'], 'integer'],            
            [['total', 'costo', 'anio', 'mes', 'precio_uni'], 'number'],
            [['referencia_dist', 'tipo', 'unidad_medida', 'co', 'oc',
                'categoria_prov', 'categoria_dist'], 'string', 'max' => 45],            
            [['desc_ref_dist', 'referencia_dist', 'referencia_prov'], 'string', 'max' => 50],            
            [['desc_ref_prov'], 'string', 'max' => 50],     
            [['fecha','nit','referencia_dist','cantidad','tipo','costo',
                'unidad_medida','oc','co', 'desc_ref_dist', 'total', 
                'referencia_prov', 'desc_ref_prov','anio', 'mes', 
                'categoria_prov', 'categoria_dist'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),                                    
            'referencia_dist' => Yii::t('app', 'Referencia Dist'),
            'referencia_prov' => Yii::t('app', 'Referencia Prov'),
            'desc_ref_dist' => Yii::t('app', 'Desc Referencia Dist'),
            'desc_ref_prov' => Yii::t('app', 'Desc Referencia Prov'), 
            'categoria_prov' => Yii::t('app', 'Categoria Prov'),            
            'categoria_dist' => Yii::t('app', 'Categoria Dist'),
            'fecha' => Yii::t('app', 'Fecha'),
            'oc' => Yii::t('app', 'OC'),
            'cantidad_pedida' => Yii::t('app', 'Cantidad Pedida'),
            'cantidad_entrada' => Yii::t('app', 'Cantidad Entrada'),
            'total' => Yii::t('app', 'Total'),
            'precio_uni' => Yii::t('app', 'Unit price'),
            //'costo' => Yii::t('app', 'Costo'),
            //'unidad_medida' => Yii::t('app', 'Unidad Medida'),
            'co' => Yii::t('app', 'Co'), 
            'mes' => Yii::t('app', 'Mes'),            
            'anio' => Yii::t('app', 'Anio'),
        ];
    }
    
    /**
     * Crea el reporte de ventas 
     * 
     * @return \yii\data\ArrayDataProvider
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function reporteCompras($valueListDefecto, $params){
        $b2bSiesa = new B2BISiesa();        
        $andWhere = '';
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $this->load($params);

        if($this->desc_ref_dist != "") 
            $andWhere .= " AND T120.f120_descripcion LIKE '%$this->desc_ref_dist%'";        
        if($this->cantidad_pedida != "") 
            $andWhere .= " AND MC.f421_cant_pedida LIKE '%$this->cantidad_pedida%'";        
        if($this->cantidad_entrada != "") 
            $andWhere .= " AND MC.f421_cant_entrada LIKE '%$this->cantidad_entrada%'";                        
        if($this->oc != "") 
            $andWhere .= " AND T420.f420_consec_docto LIKE '%$this->oc%'";         
        if($this->co != "") 
            $andWhere .= " AND f285_descripcion LIKE '%$this->co%'";
        if($this->mes != "") {
            if(is_array($this->mes) && count($this->mes)>0){
                foreach ($this->mes as $value){
                   $tempMes[] = $value +1; 
                }
                $andWhere .= " AND CONVERT(VARCHAR,MONTH(DOCFAC.f350_fecha)) IN('"
                        .implode("','",$tempMes)."')";                
            }
        }        
        if($this->anio != ""){
            $tempAnio = explode(',', $this->anio);
            $andWhere .= " AND CONVERT(VARCHAR,YEAR(DOCFAC.f350_fecha)) IN('"
                    .implode("','",$tempAnio)."')";
        }             
        //Creamos las fechas ---------------------------------------------------        
        if($this->fecha != ""){             
            list($fechaIn, $fechaFin) = explode(' - ', $this->fecha);
            $andWhere .= " AND DOCFAC.f350_fecha BETWEEN '$fechaIn' AND '$fechaFin'";                      
        }else{            
            if($this->mes == '' && $this->anio == ''){
                $fechaFin = date('d-m-Y');                
                $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));            
                $andWhere .= "AND DOCFAC.f350_fecha BETWEEN '$fechaIn' AND '$fechaFin'";                      
            }            
        }
        //Obtenemos las referencias --------------------------------------------
        if(empty(Yii::$app->session['proveedor_id'])){              
            $modelReferencias = $this->_getReferencias();
        }else{
            //Buscamos las referencias del proveedor en sesion -----------------
            $modelReferencias = Referencias::find()
                    ->where(['proveedor_id'=>Yii::$app->session['proveedor_id']])
                    ->asArray()->all();  
        }        
        $referencias = ArrayHelper::map($modelReferencias, 
                        'ref_distribuidor', 'ref_distribuidor');
        $andWhere .= " AND T120.f120_referencia IN ('".  implode("','", $referencias)."')";
        
        if($this->referencia_dist != ""){                
            $modelReferencias = $this->_getReferencias(
                    "ref_distribuidor LIKE '%$this->referencia_dist%'");  
            if(count($referencias)>0){
                $andWhere .= " AND T120.f120_referencia LIKE '%$this->referencia_dist%'";
            }else{
                $andWhere .= " AND T120.f120_referencia = '0'";
            }         
        }
        //Referencias proveedor ------------------------------------------------
        if($this->referencia_prov != ""){            
            if(strpos($valueListDefecto, 'referencia_prov') !== false){                
                $modelReferencias = $this->_getReferencias(
                        "ref_proveedor LIKE '%$this->referencia_prov%'");                
                $referencias = ArrayHelper::map($modelReferencias, 
                            'ref_distribuidor', 'ref_distribuidor');            
                $andWhere .= " AND T120.f120_referencia IN ('".  implode("','", $referencias)."')";
            }                        
        }
        
        if($this->desc_ref_prov != ""){                        
            if(strpos($valueListDefecto, 'desc_ref_prov') !== false){
                $modelReferencias = $this->_getReferencias(
                        "ref_descripcion_proveedor LIKE '%$this->desc_ref_prov%'");
                $referencias = ArrayHelper::map($modelReferencias, 
                            'ref_distribuidor', 'ref_distribuidor');            
                $andWhere .= " AND T120.f120_referencia IN ('".  implode("','", $referencias)."')";
            }            
        }        
        //Filtro de categorias -------------------------------------------------        
        if($this->categoria_prov != ""){                        
            if(strpos($valueListDefecto, 'categoria_prov') !== false ){                                
                $modelReferencias = $this->_getReferencias(
                        "categoria_proveedor LIKE '%$this->categoria_prov%'");                
                $referencias = ArrayHelper::map($modelReferencias, 
                            'ref_distribuidor', 'ref_distribuidor');            
                $andWhere .= " AND T120.f120_referencia IN ('"
                        .  implode("','", $referencias)."')";
            }            
        }
        if($this->categoria_dist != ""){                        
            if(strpos($valueListDefecto, 'categoria_dist') !== false ){                                
                $modelReferencias = $this->_getReferencias(
                        "categoria_distribuidor LIKE '%$this->categoria_dist%'");                
                $referencias = ArrayHelper::map($modelReferencias, 
                            'ref_distribuidor', 'ref_distribuidor');            
                $andWhere .= " AND T120.f120_referencia IN ('"
                        .  implode("','", $referencias)."')";
            }            
        }        
        //Creamos las variables de la consulta ---------------------------------
        $select = $this->_getListSelect($valueListDefecto);
        //Los parametros del group y ordern son similares ----------------------
        $groupBy = $orderBy =$this->_getListGroupBy($valueListDefecto);

        //Consultamos los datos en siesa ---------------------------------------              
        $arrayResult = $b2bSiesa->getCompras($referencias, $select, $andWhere, 
                $groupBy, $orderBy);
        //Validamos si se presentan todos los datos ----------------------------
        $paramShowAll = '_tog';
        $exportAll = 'all';        
        $pagination = ['pageSize' => 20];
        foreach ($_GET as $key => $value) {
            if (strpos($key, $paramShowAll) !== false){
                if($value == $exportAll){
                    $pagination = false;        
                }
            }                    
        }
        //Adicionamos las referencias del proveedor ----------------------------
        if(strpos($valueListDefecto, 'referencia_prov') !== false){  
            $tempArrayResult = [];
            //Agregamos las referencias del proveedor al resultado -------------
            if(count($arrayResult)>0){
                $tempRefencias = ArrayHelper::index($modelReferencias, 'ref_distribuidor');
                foreach ($arrayResult as $value) { 
                    if(isset($value['referencia_dist']) 
                            && isset($tempRefencias[$value['referencia_dist']]['ref_proveedor'])){
                        $value['referencia_prov'] = $tempRefencias[$value['referencia_dist']]['ref_proveedor'];
                    }else{
                        $value['referencia_prov'] = '';
                    }                    
                    $tempArrayResult[] = $value;
                }                
                $arrayResult = $tempArrayResult;
            }
        }          
        if(strpos($valueListDefecto, 'desc_ref_prov') !== false){  
            $tempArrayResult = [];
            //Agregamos las referencias del proveedor al resultado -------------
            if(count($arrayResult)>0){
                $tempRefencias = ArrayHelper::index($modelReferencias, 'ref_distribuidor');             
                foreach ($arrayResult as $value) {                    
                    if(isset($value['referencia_dist']) 
                            && isset($tempRefencias[$value['referencia_dist']]['ref_descripcion_proveedor'])){
                        $value['desc_ref_prov'] = $tempRefencias[$value['referencia_dist']]['ref_descripcion_proveedor'];                    
                    }else{
                        $value['desc_ref_prov'] = '';                        
                    }                    
                    $tempArrayResult[] = $value;
                }                
                $arrayResult = $tempArrayResult;                
            }
        }        
        //Retornamos los datos -------------------------------------------------        
        return new \yii\data\ArrayDataProvider([
            'allModels' => $arrayResult,
            'sort'  => [
                'attributes' => explode(',', $valueListDefecto),
            ],            
            'pagination' => $pagination
        ]);        
    }
    
    /**
     * Retorna le listaod de las referencias por defecto segun el tipo de 
     * usuario
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getReferencias($addAndCondicion = ''){
        $modelReferencias = Referencias::find();
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');

        if (\Yii::$app->user->identity->isProveedor()){
            $idProveedor = Yii::$app->user->identity->getIdProveedor();
            $modelReferencias->where(['proveedor_id'=>$idProveedor]);                
        }elseif (\Yii::$app->user->identity->isDistribuidor()){
            $idDistribuidor = Yii::$app->user->identity->getIdDistribuidor();
            $modelReferencias->where(['distribuidor_id'=>$idDistribuidor]);
        }

        if(empty($branchOffice['providers']) == false){
            $modelReferencias->where(['in', 'proveedor_id', $branchOffice['providers']]);
        }
        
        if(!empty($addAndCondicion)){            
            $modelReferencias->andWhere($addAndCondicion);
        }
        return $modelReferencias->asArray()->all();        
    }
    
    /**
     * Listado de los parametros por defecto sin espacios
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsDefecto(){          
            return 'fecha,oc,referencia_dist,referencia_prov,cantidad_pedida';        
    }
    
    /**
     * Listado de los parametros adicionales sin espacios
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsAdd(){
        if (\Yii::$app->user->identity->isSuperAdmin()){
            return 'cantidad_entrada,total,co,desc_ref_dist,desc_ref_prov,mes,'
            . 'anio,categoria_prov,categoria_dist,precio_uni';
        }else{
            return 'cantidad_entrada,total,co,desc_ref_dist,desc_ref_prov,mes,'
            . 'anio,categoria_prov,categoria_dist,precio_uni';
        }        
    }
    
    /**
     * Lisrados de parametros configurables en el reporte
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParams(){
        return [
            'precio_uni' => ['content' => Yii::t('app', 'Unit price')],
            'referencia_dist' => ['content' => Yii::t('app', 'Referencia Dist')],
            'desc_ref_dist' => ['content' => Yii::t('app', 'Desc Referencia Dist')],   
            'referencia_prov' => ['content' => Yii::t('app', 'Referencia Prov')],   
            'desc_ref_prov' => ['content' => Yii::t('app', 'Desc Referencia Prov')],
            'categoria_prov' => ['content' => Yii::t('app', 'Categoria Prov')],   
            'categoria_dist' => ['content' => Yii::t('app', 'Categoria Dist')],   
            'cantidad_pedida' => ['content' => Yii::t('app', 'Cantidad Pedida')],      
            'cantidad_entrada' => ['content' => Yii::t('app', 'Cantidad Entrada')],                                     
            'fecha' => ['content' => Yii::t('app', 'Fecha')],                                 
            'total' => ['content' => Yii::t('app', 'Total')],                                                                                        
            'co' => ['content' => Yii::t('app', 'Co')],     
            'oc' => ['content' => Yii::t('app', 'OC')], 
            'mes' => ['content' => Yii::t('app', 'Mes')],     
            'anio' => ['content' => Yii::t('app', 'Anio')],
        ];
    }
    
    /**
     * Metodo que retorna el listado de las columnas para el grid
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsGrid(){
        $meses = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 
            'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
        return [                        
            'oc' => 'oc',
            'referencia_dist' => 'referencia_dist',
            'desc_ref_dist' => 'desc_ref_dist',   
            'referencia_prov' => 'referencia_prov',   
            'desc_ref_prov' => 'desc_ref_prov', 
            'categoria_prov' => 'categoria_prov',   
            'categoria_dist' => 'categoria_dist',   
            'cantidad_pedida' => 
            [                
                'attribute' => 'cantidad_pedida',
                'value' => 'cantidad_pedida',                
                'format'=>['decimal', 2],
                'pageSummary'=>true
            ],            
            'cantidad_entrada' => 
            [                
                'attribute' => 'cantidad_entrada',
                'value' => 'cantidad_entrada',                
                'format'=>['decimal', 2],
                'pageSummary'=>true
            ],
            'mes' =>
            [                
                'attribute' => 'mes',
                'width' => '150px',
                'filter'=>$meses, 
                'filterType'=>  \kartik\grid\GridView::FILTER_SELECT2,
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'pluginOptions'=>['allowClear'=>true,'tags' => true,],
                ],
                'value' => function($model) use($meses){
                    return $meses[$model['mes']-1];
                },                                
                'group'=>true, 
                'groupedRow'=>false,
                'subGroupOf'=>'anio', // supplier column index is the parent group,
                'groupFooter'=>function ($model, $key, $index, $widget) { // Closure method                    
                    return [
                        'mergeColumns'=>[['total', 'cantidad', 'anio']], // columns to merge in summary
                        'content'=>[// content to show in each summary cell
                            'mes' => Yii::t('app', 'Total Mes'),
                            'precio_uni'=>  \kartik\grid\GridView::F_SUM,
                            'cantidad'=>  \kartik\grid\GridView::F_SUM,
                        ],
                        'contentFormats' => [ // content reformatting for each summary cell
                            'precio_uni' => ['format' => 'number', 'decimals' => 0],
                            'total' => ['format' => 'number', 'decimals' => 0],
                            'cantidad' => ['format' => 'number', 'decimals' => 0],
                        ],                  
                        'options' => ['class' => 'danger', 'style' => 'font-weight:bold;']
                    ];
                }
            ],
            'anio' =>
            [                
                'attribute' => 'anio',
                'value' => 'anio',                                
                'group'=>true, 
                'groupedRow'=>false,                
                'groupFooter'=>function ($model, $key, $index, $widget) { // Closure method                    
                    return [
                        'mergeColumns'=>[['total', 'cantidad', 'mes']], // columns to merge in summary
                        'content'=>[// content to show in each summary cell 
                            'anio' => Yii::t('app', 'Total').' '.Yii::t('app', 'Anio'),
                            'total'=>  \kartik\grid\GridView::F_SUM,
                            'cantidad'=>  \kartik\grid\GridView::F_SUM,
                        ],
                        'contentFormats' => [ // content reformatting for each summary cell
                            'total' => ['format' => 'number', 'decimals' => 0],
                            'precio_uni' => ['format' => 'number', 'decimals' => 0],
                            'cantidad' => ['format' => 'number', 'decimals' => 0],
                        ],                  
                        'options' => ['class' => 'danger', 'style' => 'font-weight:bold;']
                    ];
                }
            ],
            'fecha' => 
            [
                'attribute' => 'fecha',
//                'value' => function($data){                    
//                    if(empty($this->fecha_venta)) {
//                        return date('Y-m-d').' - '.date('Y-m-d', strtotime('now - 12 month'));
//                    }else{
//                        return $data->fecha_venta;
//                    }
//                },
                'filterType' => \kartik\grid\GridView::FILTER_DATE_RANGE,
                'filterWidgetOptions' => [
                    'options' => ['placeholder' => 'Rango Fechas', 'readonly'=>true],
                    'pluginOptions' => [
                        'allowClear' => true,
                        'locale'=>[                            
                            'format' => 'DD-MM-YYYY',
                            'separator'=>' - ',
                        ],                        
                    ],
                ],                
                'width' => '200px',
            ],                                 
            'total' =>
            [                
                'attribute' => 'total',
                'value' => 'total',                
                'format'=>['decimal', 2],
                'pageSummary'=>true
            ],
            'precio_uni' =>
            [
                'attribute' => 'precio_uni',
                'value' => 'precio_uni',
                'format'=>['decimal', 2],
                'pageSummary'=>true
            ],
            'tipo' => 'tipo',            
            'costo' =>            
            [                
                'attribute' => 'costo',
                'value' => 'costo',                
                'format'=>['decimal', 2],
                'pageSummary'=>true
            ],
            'unidad_medida' => 'unidad_medida',            
            'co' => 'co'
        ];
    }
    
    private static function _getProveedores(){
        return ArrayHelper::map(Proveedores::find()->all(), 'id', 'razon_social');
    }
    
    /**
     * Retorna los parametrso que van en el select del B2BiSiesa
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsSelect(){
        return [
            'id'=>'f420_rowid AS id',
            'precio_uni'=>'MI.f470_precio_uni AS precio_uni',
            'oc' => "RTRIM(LTRIM(T420.f420_id_tipo_docto)) + '-' + CONVERT(VARCHAR,(REPLICATE('0', (8 - LEN(T420.f420_consec_docto))) + CONVERT(VARCHAR,T420.f420_consec_docto)))  AS oc",                        
            'desc_ref_dist' => 'T120.f120_descripcion as desc_ref_dist',
            'desc_ref_prov' => 'RTRIM(LTRIM(T120.f120_referencia)) as referencia_dist,T120.f120_descripcion as desc_ref_dist',            
            'fecha' => 'CONVERT(VARCHAR,DOCFAC.f350_fecha,105) as fecha',
            'anio' => 'YEAR(DOCFAC.f350_fecha) as anio',            
            'mes' => 'MONTH(DOCFAC.f350_fecha) as mes',
            'referencia_dist' => 'RTRIM(LTRIM(T120.f120_referencia)) as referencia_dist',
            'referencia_prov' => 'RTRIM(LTRIM(T120.f120_referencia)) as referencia_dist',
            'categoria_prov' => $this->_getStrCategoriaProv().' as categoria_prov',
            'categoria_dist' => $this->_getStrCategoriaDist().' as categoria_dist',
            //'costo' => 'ISNULL(T132.f132_ultimo_costo_local,0) AS costo', 
            'cantidad_pedida' => "SUM((CASE WHEN MC.f421_id_concepto = '402' THEN MC.f421_cant_pedida*-1 ELSE MC.f421_cant_pedida END)) AS cantidad_pedida",
            'cantidad_entrada' => "SUM((CASE WHEN MI.F470_ID_CONCEPTO = '402' THEN MI.f470_cant_1*-1 ELSE MI.f470_cant_1 END)) AS cantidad_entrada",
            //'total' => "SUM((CASE WHEN MI.f470_id_concepto = '402' THEN (MI.f470_vlr_neto - MI.f470_vlr_imp - MI.f470_vlr_dscto_linea)*-1 ELSE MI.f470_vlr_neto - MI.f470_vlr_imp - MI.f470_vlr_dscto_linea END)) AS total",
            'total' => "SUM((CASE WHEN MI.f470_id_concepto = '402' THEN (MI.f470_vlr_bruto)*-1 ELSE MI.f470_vlr_bruto END)) AS total",
            'co' => 'f285_descripcion AS co',
            //'unidad_medida' => "'UNID' AS unidad_medida", 
            //'tipo' => "(CASE WHEN RTRIM(LTRIM(f350_id_tipo_docto)) = 'FV' THEN 0 ELSE 1 END) AS tipo",	    
        ];
    }
    
    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsGroupBy(){
        #Es necesaio el / para la separacion de columnas no debe ser modificado
        return [
            'precio_uni'=>'MI.f470_precio_uni',
            'id'=>'f420_rowid',
            'desc_ref_dist' => 'T120.f120_descripcion',
            'desc_ref_prov' => 'T120.f120_referencia / T120.f120_descripcion',            
            'fecha' => 'DOCFAC.f350_fecha',
            'referencia_dist' => 'T120.f120_referencia',
            'referencia_prov' => 'T120.f120_referencia',
            //'costo' => 'T132.f132_ultimo_costo_local',             
            'co' => 'f285_descripcion',                        
	    'oc' => "T420.f420_id_tipo_docto / T420.f420_consec_docto",
            'anio' => 'YEAR(DOCFAC.f350_fecha)',            
            'mes' => 'MONTH(DOCFAC.f350_fecha)',
            'categoria_prov' => $this->_getStrCategoriaProv(),
            'categoria_dist' => $this->_getStrCategoriaDist(),
        ];
    }
    
    /**
     * Retorna el listado a partir de un array y un string dado
     * 
     * @param string $string String separado por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getTempList($string){        
        return $this->_getDataFromList($string, $this->listParams());
    }
    
    /**
     * Retorna el array de los parametros que iran el en select de la consulta 
     * de ventas
     * 
     * @param string $params Parametros separados por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListSelect($params){
        return $this->_getDataFromList($params, $this->_paramsSelect());
    }
    
    /**
     * Retorna el array de los parametros que iran el en group de la consulta 
     * de ventas
     * 
     * @param string $params Parametros separados por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListGroupBy($params){
        return $this->_getDataFromList($params, $this->_paramsGroupBy());
    }
    
    /**
     * Retorna el array de los parametros que iran el en grid de la vista     
     * 
     * @param string $params Parametros separados por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getListGrid($params){
        return $this->_getDataFromList($params, $this->listParamsGrid());
    }        
    
    /**
     * Funcion que busca los datos de un string en un array
     * 
     * @param string $string Datos separados por comas    
     * @param array  $list   Array donde se buscaran los datos
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getDataFromList($string, $list){
        $array = explode(',', $string);
        $arrayResul = [];
        if(count($array)>0){            
            foreach ($array as $value) {
                $trimValue = trim($value);
                if(isset($list[$trimValue])){
                    $arrayResul[$trimValue] = $list[$trimValue];
                }                
            }
        }
        return $arrayResul;
    }
    
    /**
     * Retorna toda la informacion del dashboard para la seccion de compras
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getInfoDasboard($idProveedorPost = null){
        $andWhere = '';
        $b2bSiesa = new B2BISiesa;
        $fechaFin = date('d-m-Y');                
        $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $modelReferencias = $this->getReferences($idProveedorPost, $branchOffice);
        $referencias = ArrayHelper::map($modelReferencias, 
                    'ref_distribuidor', 'ref_distribuidor');                       
        $ultCompras = $b2bSiesa->getUltimasComprasMeses($referencias, 3);
        //Traemos las metas del proveedor --------------------------------------
        $proyecciones = [];
        $arrayProy = [];        
        if(is_null($idProveedorPost)){
            $proyecciones = Proyecciones::find()->where(
                ['usuario_id'=>Yii::$app->user->identity->id])->asArray()->all();            
        }else{
            $usuario = Usuarios::findOne(['proveedor_id'=>$idProveedorPost]);
            if(!is_null($usuario)){
                $proyecciones = Proyecciones::find()
                    ->where(['usuario_id'=>$usuario->id])->asArray()->all();            
            }            
        }
        
        if(count($proyecciones)>0){
            $arrayProy = ArrayHelper::index($proyecciones, 'mes', ['anio']);
        }        
        //----------------------------------------------------------------------
        $dataMeses = [];
        $dataTotales = [];
        if(count($ultCompras)>0){ 
            $meses = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 
            'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
            foreach ($ultCompras as $anio => $datos) { 
                foreach ($datos as $data) {                                                          
                    $dataMeses[$anio][] = $meses[$data['mes']-1];                
                    $dataTotales[$anio][] = $data['total'];                
                }            
            }            
        }
        $infoDashboard['meses_grafica']= $dataMeses; 
        $infoDashboard['totales_grafica']= $dataTotales;
        $infoDashboard['proyecciones']= $arrayProy; 
        //TotalVenta -----------------------------------------------------------
        $infoDashboard['totalCompra'] = 0;                            
        return $infoDashboard;
    }
    
    /**
     * Retorna el string para la consulta de categorias por proveedor
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getStrCategoriaProv(){
        $referencas = $this->_getReferencias();        
        $groupCategoria = ArrayHelper::index($referencas, 'id', ['categoria_proveedor']);        
        $str = "(CASE";
        if(count($groupCategoria) > 0){            
            foreach ($groupCategoria as $categoria => $data ) {                
                $referencias = ArrayHelper::map($data, 'id', 'ref_distribuidor');
                $nomCategroria = !empty($categoria) ? trim($categoria) : 'null';                
                $str .= " WHEN f120_referencia IN('".  implode("','", $referencias)."') THEN '".$nomCategroria."' ";                
            }
        }
        $str .= "ELSE 'null' END)";
        return $str;
    }
    
    /**
     * Retorna el string para la consulta de categorias por distribuidor
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getStrCategoriaDist(){
        $referencas = $this->_getReferencias();        
        $groupCategoria = ArrayHelper::index($referencas, 'id', ['categoria_distribuidor']); 
        $str = "";
        if(count($groupCategoria) > 0){            
            $str .= "(CASE";
            foreach ($groupCategoria as $categoria => $data ) {                
                $referencias = ArrayHelper::map($data, 'id', 'ref_distribuidor');
                $nomCategroria = !empty($categoria) ? trim($categoria) : null;                
                $str .= " WHEN f120_referencia IN('".  implode("','", $referencias)."') THEN '".$nomCategroria."' ";                
            }
            $str .= "ELSE null END)";
        }                
        return $str;
    }
    
    /**
     * Retorna las referencias 
     * 
     * @param integer $idProveedor  
     * @param string  $branchOffice
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2016 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getReferences($idProveedor, $branchOffice){
        $modelReferencias = [];
        if(is_null($idProveedor) || empty($idProveedor)){
            if (\Yii::$app->user->identity->isProveedor()){
                $idProveedor = Yii::$app->user->identity->getIdProveedor();
                $modelReferencias = Referencias::find()
                        ->where(['proveedor_id'=>$idProveedor])
                        ->asArray()->all();                
            }elseif (\Yii::$app->user->identity->isDistribuidor()){
                $idDistribuidor = Yii::$app->user->identity->getIdDistribuidor();

                if(empty($branchOffice['providers']) == false){
                    $modelReferencias = Referencias::find()
                        ->where(['in', 'proveedor_id', $branchOffice['providers']])
                        ->andWhere(['distribuidor_id'=>$idDistribuidor])
                        ->asArray()->all();
                }else{
                    $modelReferencias = Referencias::find()
                        ->where(['distribuidor_id'=>$idDistribuidor])
                        ->asArray()->all();
                }
            }else{
                if(empty($branchOffice['providers']) == false){
                    $modelReferencias = Referencias::find()
                        ->asArray()
                        ->where(['in', 'proveedor_id', $branchOffice['providers']])
                        ->all();
                }else{
                    $modelReferencias = Referencias::find()
                        ->asArray()->all();
                }
            }
        }else{
            if(empty($branchOffice['providers']) == false){
                $modelReferencias = Referencias::find()
                    ->where(['proveedor_id'=>$idProveedor])
                    ->andWhere(['in', 'proveedor_id', $branchOffice['providers']])
                    ->asArray()->all();
            }else{
                $modelReferencias = Referencias::find()
                    ->where(['proveedor_id'=>$idProveedor])
                    ->asArray()->all();
            }
        }
        return $modelReferencias;
    }
    
}
