<?php

namespace app\models;

use Yii;
use yii\base\Model;
use app\components\B2BISiesa;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "compras".
 *
 * @property integer $id 
 * @property string $referencia_dist
 * @property string $desc_ref_dist
 * @property string $bodega 
 * @property integer $cantidad  
 * @property string $co 
 * 
 */
class Inventarios extends Model {
    
    const NM_REPORTE = 'compras';
                    
    public $referencia_dist;
    public $desc_ref_dist;
    public $referencia_prov;
    public $desc_ref_prov; 
    public $categoria_prov;
    public $categoria_dist;
    public $cantidad;        
    public $bodega;        
    public $co;                

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [            
            [['cantidad'], 'integer'],                                
            [['bodega', 'co', 'categoria_prov', 'categoria_dist'], 'string', 'max' => 45],            
            [['referencia_dist', 'referencia_prov'], 'string', 'max' => 50],            
            [['desc_ref_prov', 'desc_ref_dist'], 'string', 'max' => 50],     
            [['referencia_dist', 'desc_ref_dist', 'referencia_prov', 
                'desc_ref_prov', 'cantidad', 'bodega', 'co', 'categoria_prov', 
                'categoria_dist'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),                                    
            'referencia_dist' => Yii::t('app', 'Referencia Dist'),
            'referencia_prov' => Yii::t('app', 'Referencia Prov'),
            'desc_ref_dist' => Yii::t('app', 'Desc Referencia Dist'),
            'desc_ref_prov' => Yii::t('app', 'Desc Referencia Prov'),            
            'categoria_prov' => Yii::t('app', 'Categoria Prov'),            
            'categoria_dist' => Yii::t('app', 'Categoria Dist'),            
            'cantidad' => Yii::t('app', 'Cantidad'),                                           
            'co' => Yii::t('app', 'Co'),                        
        ];
    }
    
    /**
     * Crea el reporte de ventas 
     * 
     * @return \yii\data\ArrayDataProvider
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function reporteInventario($params){
        $b2bSiesa = new B2BISiesa();        
        $andWhere = '';
        
        $this->load($params);
                                                  
        if($this->desc_ref_dist != "") 
            $andWhere .= " AND T120.f120_descripcion LIKE '%$this->desc_ref_dist%'";        
        if($this->cantidad != "") 
            $andWhere .= " AND qry.Cantidad_Disponible '%$this->cantidad%'";                
//        if($this->bodega != "") 
//            $andWhere .= " AND f150_descripcion '%$this->cantidad%'";                
        if($this->co != "") 
            $andWhere .= " AND qry.co LIKE '%$this->co%'";                        
        //Obtenemos las referencias --------------------------------------------
        if(empty(Yii::$app->session['proveedor_id'])){              
            $modelReferencias = $this->_getReferencias();
        }else{
            //Buscamos las referencias del proveedor en sesion -----------------
            $modelReferencias = Referencias::find()
                    ->where(['proveedor_id'=>Yii::$app->session['proveedor_id']])
                    ->asArray()->all();  
        }        
        $referencias = ArrayHelper::map($modelReferencias, 
                        'ref_distribuidor', 'ref_distribuidor');
        $andWhere .= " AND T120.f120_referencia IN ('"
                .  implode("','", $referencias)."')";
        //Filtrado de referencias ----------------------------------------------
        if($this->referencia_dist != ""){                
            $modelReferencias = $this->_getReferencias(
                    "ref_distribuidor LIKE '%$this->referencia_dist%'");
            if(count($referencias)>0){
                $andWhere .= " AND T120.f120_referencia LIKE '%$this->referencia_dist%'";
            }else{
                $andWhere .= " AND T120.f120_referencia = 0";
            }         
        }
        //Referencias proveedor ------------------------------------------------
        if($this->referencia_prov != ""){                        
            $modelReferencias = $this->_getReferencias(
                        "ref_proveedor LIKE '%$this->referencia_prov%'");
            $referencias = ArrayHelper::map($modelReferencias, 
                        'ref_distribuidor', 'ref_distribuidor');            
            $andWhere .= " AND T120.f120_referencia IN ('".  implode("','", $referencias)."')";
                                   
        }
        if($this->desc_ref_prov != ""){            
            $modelReferencias = $this->_getReferencias(
                        "ref_descripcion_proveedor LIKE '%$this->desc_ref_prov%'");
            $referencias = ArrayHelper::map($modelReferencias, 
                        'ref_distribuidor', 'ref_distribuidor');            
            $andWhere .= " AND T120.f120_referencia IN ('".  implode("','", $referencias)."')";            
        } 
        //Filtro de categorias -------------------------------------------------        
        if($this->categoria_prov != ""){                                                                  
            $modelReferencias = $this->_getReferencias(
                    "categoria_proveedor LIKE '%$this->categoria_prov%'");                
            $referencias = ArrayHelper::map($modelReferencias, 
                        'ref_distribuidor', 'ref_distribuidor');            
            $andWhere .= " AND T120.f120_referencia IN ('"
                    .  implode("','", $referencias)."')";            
        }
        if($this->categoria_dist != ""){                                    
            $modelReferencias = $this->_getReferencias(
                    "categoria_distribuidor LIKE '%$this->categoria_dist%'");                
            $referencias = ArrayHelper::map($modelReferencias, 
                        'ref_distribuidor', 'ref_distribuidor');            
            $andWhere .= " AND T120.f120_referencia IN ('"
                    .  implode("','", $referencias)."')";            
        }
        //Consultamos los datos en siesa ---------------------------------------
        $arrayResult = $b2bSiesa->getInventario($referencias, $andWhere);    
        //Validamos si se presentan todos los datos ----------------------------
        $paramShowAll = '_tog';
        $exportAll = 'all';        
        $pagination = ['pageSize' => 20];
        foreach ($_GET as $key => $value) {
            if (strpos($key, $paramShowAll) !== false){
                if($value == $exportAll){
                    $pagination = false;        
                }
            }                    
        }
        //Agregamso las referencias --------------------------------------------
        if(count($arrayResult)>0){
            $tempRefencias = ArrayHelper::index($modelReferencias, 'ref_distribuidor');
            foreach ($arrayResult as $value) { 
                if(isset($tempRefencias[$value['referencia_dist']]['ref_proveedor'])){
                    $value['referencia_prov'] = $tempRefencias[$value['referencia_dist']]['ref_proveedor'];
                }else{
                    $value['referencia_prov'] = '';
                }
                
                if(isset($tempRefencias[$value['referencia_dist']]['ref_descripcion_proveedor'])){
                    $value['desc_ref_prov'] = $tempRefencias[$value['referencia_dist']]['ref_descripcion_proveedor'];                    
                }else{
                    $value['desc_ref_prov'] = '';                        
                }
                $tempArrayResult[] = $value;
            }                
            $arrayResult = $tempArrayResult;
        }                                 
        //Retornamos los datos -------------------------------------------------        
        return new \yii\data\ArrayDataProvider([
            'allModels' => $arrayResult,
            'sort'  => [
                'attributes' => ['referencia_dist', 'desc_ref_dist', 
                    'referencia_prov', 'desc_ref_prov', 'cantidad', 'bodega', 
                    'co', 'categoria_prov', 'categoria_dist'],
            ],            
            'pagination' => $pagination
        ]);        
    }
    
    /**
     * Retorna le listaod de las referencias por defecto segun el tipo de 
     * usuario
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getReferencias($addAndCondicion = ''){
        $modelReferencias = Referencias::find();        
        if (\Yii::$app->user->identity->isProveedor()){
            $idProveedor = Yii::$app->user->identity->getIdProveedor();
            $modelReferencias->where(['proveedor_id'=>$idProveedor]);                
        }elseif (\Yii::$app->user->identity->isDistribuidor()){
            $idDistribuidor = Yii::$app->user->identity->getIdDistribuidor();
            $modelReferencias->where(['distribuidor_id'=>$idDistribuidor]);
        } 
        
        if(!empty($addAndCondicion)){            
            $modelReferencias->andWhere($addAndCondicion);
        }
        return $modelReferencias->asArray()->all();        
    }
    
    /**
     * Retorna el string para la consulta de categorias por proveedor
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getStrCategoriaProv(){
        $referencas = $this->_getReferencias();        
        $groupCategoria = ArrayHelper::index($referencas, 'id', ['categoria_proveedor']);        
        $str = "(CASE";
        if(count($groupCategoria) > 0){            
            foreach ($groupCategoria as $categoria => $data ) {                
                $referencias = ArrayHelper::map($data, 'id', 'ref_distribuidor');
                $nomCategroria = !empty($categoria) ? trim($categoria) : 'null';                
                $str .= " WHEN f120_referencia IN('".  implode("','", $referencias)."') THEN '".$nomCategroria."' ";                
            }
        }
        $str .= "ELSE 'null' END)";
        return $str;
    }
    
    /**
     * Retorna el string para la consulta de categorias por distribuidor
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getStrCategoriaDist(){
        $referencas = $this->_getReferencias();        
        $groupCategoria = ArrayHelper::index($referencas, 'id', ['categoria_distribuidor']); 
        $str = "";
        if(count($groupCategoria) > 0){            
            $str .= "(CASE";
            foreach ($groupCategoria as $categoria => $data ) {                
                $referencias = ArrayHelper::map($data, 'id', 'ref_distribuidor');
                $nomCategroria = !empty($categoria) ? trim($categoria) : null;                
                $str .= " WHEN f120_referencia IN('".  implode("','", $referencias)."') THEN '".$nomCategroria."' ";                
            }
            $str .= "ELSE null END)";
        }                
        return $str;
    }
}
