<?php

namespace app\models;

use Yii;
use yii\behaviors\AttributeBehavior;
use yii\db\ActiveRecord;


/**
 * This is the model class for table "log_cargas".
 *
 * @property integer $id
 * @property integer $fila
 * @property string $mensaje
 * @property integer $usuario_id
 *
 * @property Usuarios $usuario
 */
class Logcargas extends ActiveRecord
{
    public $nomUsuario;
    
    public function behaviors() {
        return [
            'usuarios' => [
                'class' => AttributeBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_VALIDATE => ['usuario_id'],
                    ActiveRecord::EVENT_BEFORE_INSERT => ['usuario_id'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['usuario_id'],
                ],
                'value' => function ($event) {                            
                if(isset(\Yii::$app->user->identity)){                    
                    return (!\Yii::$app->user->identity->isSuperAdmin()) ?
                        \Yii::$app->user->identity->id :
                        $event->sender->usuario_id;                                        
                }            
        },
            ],
        ];
    }
    
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'log_cargas';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['fila', 'mensaje', 'usuario_id'], 'required'],
            [['fila', 'usuario_id'], 'integer'],
            [['mensaje', 'nomUsuario'], 'string'],
            [['usuario_id'], 'exist', 'skipOnError' => true, 'targetClass' => Usuarios::className(), 'targetAttribute' => ['usuario_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'fila' => Yii::t('app', 'Fila'),
            'mensaje' => Yii::t('app', 'Mensaje'),
            'nomUsuario' => Yii::t('app', 'Usuario ID'),
            'usuario_id' => Yii::t('app', 'Usuario ID'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUsuario()
    {
        return $this->hasOne(Usuarios::className(), ['id' => 'usuario_id']);
    }
}
