<?php

namespace app\models;

use Yii;
use yii\behaviors\AttributeBehavior;
use \yii\db\ActiveRecord;

/**
 * This is the model class for table "proyecciones".
 *
 * @property integer $id
 * @property integer $anio
 * @property string $mes
 * @property string $meta_compras
 * @property string $meta_ventas
 * @property integer $usuario_id
 *
 * @property Usuarios $usuario
 */
class Proyecciones extends ActiveRecord
{
    public $nomUsuario;
    
    public function behaviors() {
        return [
            'usuarios' => [
                'class' => AttributeBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_VALIDATE => ['usuario_id'],
                    ActiveRecord::EVENT_BEFORE_INSERT => ['usuario_id'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['usuario_id'],
                ],
                'value' => function ($event) {
                    if(isset(\Yii::$app->user->identity)){
                        return (!\Yii::$app->user->identity->isSuperAdmin()) ?  \Yii::$app->user->identity->id : $event->sender->usuario_id;
                    }            
                },
            ],
        ];
    }
    
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'proyecciones';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['anio', 'mes', 'usuario_id', 'meta_compras', 'meta_ventas'], 'required'],
            [['anio', 'meta_compras', 'meta_ventas', 'usuario_id'], 'integer'],
            [['nomUsuario', 'mes'], 'string'],
            [['usuario_id'], 'exist', 'skipOnError' => true, 'targetClass' => Usuarios::className(), 'targetAttribute' => ['usuario_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'anio' => Yii::t('app', 'Anio'),
            'mes' => Yii::t('app', 'Mes'),
            'meta_compras' => Yii::t('app', 'Meta Compras'),
            'meta_ventas' => Yii::t('app', 'Meta Ventas'),
            'usuario_id' => Yii::t('app', 'Usuario'),
            'nomUsuario' => Yii::t('app', 'Usuario'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUsuario()
    {
        return $this->hasOne(Usuarios::className(), ['id' => 'usuario_id']);
    }
    
    /**
     * Retorna el listado de los años 
     * @return array
     */
    public function getListAnios(){        
        return [
            date('Y')-1 => date('Y')-1,
            date('Y') => date('Y'),
            date('Y')+1 => date('Y')+1,            
            date('Y')+2 => date('Y')+2,            
            date('Y')+3 => date('Y')+3,            
        ];
    }
    
    /**
     * Retorna el listado de meses
     * @return array
     */
    public function getListMeses(){
        return [
            'Enero'=>'Enero',                        
            'Febrero'=>'Febrero',                        
            'Marzo'=>'Marzo',                        
            'Abril'=>'Abril',                        
            'Mayo'=>'Mayo',                        
            'Junio'=>'Junio',                        
            'Julio'=>'Julio',                        
            'Agosto'=>'Agosto',                        
            'Septiembre'=>'Septiembre',                        
            'Octubre'=>'Octubre',                        
            'Noviembre'=>'Noviembre',                        
            'Diciembre'=>'Diciembre',                        
        ];
    }
}
