<?php

namespace app\models;

use Yii;
use yii\behaviors\AttributeBehavior;
use \yii\db\ActiveRecord;

/**
 * This is the model class for table "referencias".
 *
 * @property integer $id
 * @property string $ref_proveedor
 * @property string $ref_descripcion_proveedor
 * @property string $categoria_proveedor
 * @property string $ref_distribuidor
 * @property string $ref_descripcion_distribuidor
 * @property string $categoria_distribuidor
 * @property integer $distribuidor_id
 * @property integer $proveedor_id
 *
 * @property Proveedores $proveedor
 * @property Distribuidores $distribuidor
 */
class Referencias extends ActiveRecord
{
    public $fileInput;
    public $nomDistribuidor;
    public $nomProveedor;
    
    
    public function behaviors() {
        return [
            'usuarios' => [
                'class' => AttributeBehavior::className(),
                'attributes' => [
                    ActiveRecord::EVENT_BEFORE_VALIDATE => ['proveedor_id'],
                    ActiveRecord::EVENT_BEFORE_INSERT => ['proveedor_id'],
                    ActiveRecord::EVENT_BEFORE_UPDATE => ['proveedor_id'],
                ],
                'value' => function ($event) {                            
                    if(isset(\Yii::$app->user->identity)){    
                        $usuario = Usuarios::findOne(Yii::$app->user->identity->id);                    
                        if (\Yii::$app->user->identity->isProveedor()){
                            if(isset($usuario->proveedor_id) && !empty($usuario->proveedor_id))
                            return $usuario->proveedor_id;
                        }elseif (\Yii::$app->user->identity->isDistribuidor()){
                            if(isset($usuario->distribuidor_id) && !empty($usuario->distribuidor_id))
                            return $usuario->distribuidor_id;
                        }else{
                            return $event->sender->proveedor_id; 
                        }
                    }            
                },
            ],
        ];
    }
    
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'referencias';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['ref_proveedor', 'ref_descripcion_proveedor', 'ref_distribuidor', 'distribuidor_id', 'proveedor_id'], 'required'],
            [['distribuidor_id', 'proveedor_id'], 'required', 'on'=>'importar'],
            [['distribuidor_id', 'proveedor_id'], 'integer'],
            [['fileInput'], 'required', 'on'=>'importar'],                     
            [['fileInput'], 'file', 'extensions' => 'xlsx', 'on'=>'importar'],
            [['ref_proveedor', 'ref_distribuidor'], 'string', 'max' => 45],
            [['nomDistribuidor', 'nomProveedor'], 'string', 'max' => 45],
            [['ref_descripcion_proveedor', 'ref_descripcion_distribuidor'], 'string', 'max' => 300],
            [['categoria_proveedor', 'categoria_distribuidor'], 'string', 'max' => 150],
            [['proveedor_id'], 'exist', 'skipOnError' => true, 'targetClass' => Proveedores::className(), 'targetAttribute' => ['proveedor_id' => 'id']],
            [['distribuidor_id'], 'exist', 'skipOnError' => true, 'targetClass' => Distribuidores::className(), 'targetAttribute' => ['distribuidor_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'ref_proveedor' => Yii::t('app', 'Ref Proveedor'),
            'ref_descripcion_proveedor' => Yii::t('app', 'Ref Descripcion Proveedor'),
            'categoria_proveedor' => Yii::t('app', 'Categoria Proveedor'),
            'ref_distribuidor' => Yii::t('app', 'Ref Distribuidor'),
            'ref_descripcion_distribuidor' => Yii::t('app', 'Ref Descripcion Distribuidor'),
            'categoria_distribuidor' => Yii::t('app', 'Categoria Distribuidor'),
            'distribuidor_id' => Yii::t('app', 'Distribuidor'),
            'nomDistribuidor' => Yii::t('app', 'Distribuidor'),
            'proveedor_id' => Yii::t('app', 'Proveedor'),
            'nomProveedor' => Yii::t('app', 'Proveedor'),
            'fileInput' => Yii::t('app', 'Archivo'),
        ];
    }
    
    public function scenarios()
    {
        $scenarios = parent::scenarios();        
        $scenarios['importar'] = ['fileInput', 'distribuidor_id', 'proveedor_id'];        
        return $scenarios;
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProveedor()
    {
        return $this->hasOne(Proveedores::className(), ['id' => 'proveedor_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDistribuidor()
    {
        return $this->hasOne(Distribuidores::className(), ['id' => 'distribuidor_id']);
    }
    
    /**
     * Metodo para limpiar los datos
     * @param type $data
     * @return type
     */
    public function limpiarData($data){        
        if(count($data)>0){
            foreach ($data as $key => $value) {
                if ($value instanceof \DateTime) {
                    $data[$key] = $value->format('Ymd');
                }else{
                    //Quitamos comillas simples o dobles -----------------------
                    $data[$key] = str_replace(["'", '"', '/', "\\"], "", $value);
                }                                    
            }
            //Creamos array para insertar --------------------------------------
            $data[0] = (string)(isset($data[0]) ? $data[0]: null);
            $data[1] = isset($data[1]) ? $data[1]: null;
            $data[2] = isset($data[2]) ? $data[2]: null;            
            $data[3] = (string)(isset($data[3]) ? $data[3]: null);
            $data[4] = isset($data[4]) ? $data[4]: null;
            $data[5] = isset($data[5]) ? $data[5]: null;                                              
        }        
        return $data;
    }
}
