<?php

namespace app\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use app\models\Referencias;

/**
 * ReferenciasSearch represents the model behind the search form about `app\models\Referencias`.
 */
class ReferenciasSearch extends Referencias
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'distribuidor_id', 'proveedor_id'], 'integer'],
            [['ref_proveedor', 'ref_descripcion_proveedor', 
                'categoria_proveedor', 'ref_distribuidor', 
                'ref_descripcion_distribuidor', 
                'categoria_distribuidor', 'nomDistribuidor', 
                'nomProveedor'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');

        $query = Referencias::find();
        $query->joinWith('proveedor');
        $query->joinWith('distribuidor');

        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);
        
        $dataProvider->sort->attributes['nomDistribuidor'] = [            
            'asc' => ['distribuidores.razon_social' => SORT_ASC],
            'desc' => ['distribuidores.razon_social' => SORT_DESC],
        ];
        
        $dataProvider->sort->attributes['nomProveedor'] = [            
            'asc' => ['proveedores.razon_social' => SORT_ASC],
            'desc' => ['proveedores.razon_social' => SORT_DESC],
        ];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'distribuidor_id' => $this->distribuidor_id,
            'proveedor_id' => $this->proveedor_id
        ]);

        if(empty($branchOffice['providers']) == false){
            $query->andWhere(['in', 'proveedor_id', $branchOffice['providers']]);
        }

        $query->andFilterWhere(['like', 'ref_proveedor', $this->ref_proveedor])
            ->andFilterWhere(['like', 'ref_descripcion_proveedor', $this->ref_descripcion_proveedor])
            ->andFilterWhere(['like', 'categoria_proveedor', $this->categoria_proveedor])
            ->andFilterWhere(['like', 'ref_distribuidor', $this->ref_distribuidor])
            ->andFilterWhere(['like', 'ref_descripcion_distribuidor', $this->ref_descripcion_distribuidor])
            ->andFilterWhere(['like', 'distribuidores.razon_social', $this->nomDistribuidor])
            ->andFilterWhere(['like', 'proveedores.razon_social', $this->nomProveedor])
            ->andFilterWhere(['like', 'categoria_distribuidor', $this->categoria_distribuidor]);

        return $dataProvider;
    }
}
