<?php

namespace app\models;

use yii\db\ActiveRecord;
use yii\db\mssql\PDO;
use app\models\Usuarios;
use yii\web\IdentityInterface;

class User extends ActiveRecord implements IdentityInterface {

    public $id;
    public $username;
    public $password;
    public $authKey;
    public $accessToken;
    public $fullName;
    private static $users = [
        '100' => [
            'id' => '100',
            'username' => 'admin',
            'password' => 'admin',
            'authKey' => 'test100key',
            'accessToken' => '100-token',
        ],
        '101' => [
            'id' => '101',
            'username' => 'demo',
            'password' => 'demo',
            'authKey' => 'test101key',
            'accessToken' => '101-token',
        ],
    ];

    /**
     * @inheritdoc
     */
    public static function findIdentity($id) {
        $fnIdentity = \Yii::$app->session['fnIdentity'];
        if (empty($fnIdentity)) {
            $sql = "SELECT id, username, "
                . "username as authKey, username as accessToken, "
                . "nombre as fullName, password "
                . "FROM usuarios u "
                . "WHERE id = :id";
            $command = \Yii::$app->db->createCommand($sql);
            $command->bindParam(":id", $id, PDO::PARAM_STR);
            $user = $command->queryOne();
            \Yii::$app->session['fnIdentity'] = $user;
        } else {
            $user = \Yii::$app->session['fnIdentity'];
        }

        if ($user) {
            return new static($user);
        }

        return null;
    }

    /**
     * @inheritdoc
     */
    public static function findIdentityByAccessToken($token, $type = null) {
        foreach (self::$users as $user) {
            if ($user['accessToken'] === $token) {
                return new static($user);
            }
        }

        return null;
    }

    /**
     * Finds user by username
     *
     * @param  string      $username
     * @return static|null
     */
    public static function findByUsername($username) {
        $sql = "SELECT id, username, "
                . "username as authKey, username as accessToken, "
                . "nombre as fullName, password "
                . "FROM usuarios u "
                . "WHERE username = :username AND activo = 'SI'";
        $command = \Yii::$app->db->createCommand($sql);
        $command->bindParam(":username", $username, PDO::PARAM_STR);
        $user = $command->queryOne();

        if ($user) {
            return new static($user);
        }

        return null;
    }

    /**
     * @inheritdoc
     */
    public function getId() {
        return $this->id;
    }        
    
    /**
     * Retorna el nombre completo del usuario registrado
     * 
     * @return string     
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version Release: $Id$
     */
    public function getFullName() {
        return $this->fullName;
    }
    
    /**
     * @inheritdoc
     */
    public function getAuthKey() {
        return $this->authKey;
    }

    /**
     * @inheritdoc
     */
    public function validateAuthKey($authKey) {
        return $this->authKey === $authKey;
    }

    /**
     * Validates password
     *
     * @param  string  $password password to validate
     * @return boolean if password provided is valid for current user
     */
    public function validatePassword($password) {
        return $this->password === md5($password);
    }
    
    /**
     * Valida si el usaurio tiene perfil de super administrador
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co     
     */
    public function isSuperAdmin() {        
        $arrRoles = \Yii::$app->authManager->getRolesByUser(\Yii::$app->user->getId());        
        if(array_key_exists(Usuarios::SUPER_ADMIN, $arrRoles)){
            return true;
        }
        return false;
    }
    
    /**
     * Valida si el usaurio tiene perfil de administrador
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co     
     */
    public function isAdmin() {        
        $arrRoles = \Yii::$app->authManager->getRolesByUser(\Yii::$app->user->getId());
        if(array_key_exists(Usuarios::ADMIN, $arrRoles)){
            return true;
        }
        return false;
    }
    
    /**
     * Valida si el usaurio tiene perfil de administrador
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co     
     */
    public function isGeneral() {        
        $arrRoles = \Yii::$app->authManager->getRolesByUser(\Yii::$app->user->getId());
        if(array_key_exists(Usuarios::GENERAL, $arrRoles)){
            return true;
        }
        return false;
    }            

    /**
     * Compara los roles para saber si el usuario es Rol Proveedor
     * 
     * @return boolean
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co     
     */
    public function isProveedor(){
        $arrRoles = \Yii::$app->authManager->getRolesByUser(\Yii::$app->user->getId());
        if(array_key_exists(Usuarios::PROVEEDOR, $arrRoles)){
            return true;
        }
        return false;
    }
    
    /**
     * Retorna el Id del proveedor del usuario logueado
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co     
     */
    public function getIdProveedor(){
        $usuario = Usuarios::findOne(\Yii::$app->user->getId());        
        return $usuario->proveedor_id;
    }
    
    /**
     * Compara los roles para saber si el usuario es Rol Distribuidor
     * 
     * @return boolean
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co     
     */
    public function isDistribuidor(){
        $arrRoles = \Yii::$app->authManager->getRolesByUser(\Yii::$app->user->getId());
        if(array_key_exists(Usuarios::DISTRIBUIDOR, $arrRoles)){
            return true;
        }
        return false;
    }
    
    /**
     * Retorna el Id del distribuidor del usuario logueado
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co     
     */
    public function getIdDistribuidor(){
        $usuario = Usuarios::findOne(\Yii::$app->user->getId());        
        return $usuario->distribuidor_id;
    }
    
}
