<?php

namespace app\models;

use app\components\B2BISiesa;
use Yii;

/**
 * This is the model class for table "usr_branch_office".
 *
 * @property int $id
 * @property int $id_user
 * @property string $code_provider
 * @property string $name_provider
 * @property string $code_branch_office
 * @property string $name_branch_office
 * @property string $code_city
 * @property string $name_city
 * @property User $user
 */
class UsrBranchOffice extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'usr_branch_office';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id_user', 'code_provider', 'name_provider', 'code_branch_office', 
                'name_branch_office', 'code_city'], 'required'],
            [['id_user'], 'integer'],
            [['code_provider'], 'string', 'max' => 25],
//            [['name_provider', 'name_city'], 'string', 'max' => 250],
            [['name_provider'], 'string', 'max' => 250],
            [['name_city'], 'string',],
            [['id_user'], 'exist', 'skipOnError' => true, 'targetClass' => Usuarios::className(), 'targetAttribute' => ['id_user' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'id_user' => Yii::t('app', 'User ID'),
            'code_provider' => Yii::t('app', 'Code Provider'),
            'name_provider' => Yii::t('app', 'Name Provider'),
            'code_branch_office' => Yii::t('app', 'Code Branches Office'),
            'name_branch_office' => Yii::t('app', 'Name Branches Office'),
            'code_city' => Yii::t('app', 'City'),
            'name_city' => Yii::t('app', 'City'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(Usuarios::className(), ['id' => 'id_user']);
    }

    /**
     * Retorna el listado de ls compañias relacionadas en PDV
     * @return array
     */
    public function getSucursalesDrop($nit)
    {
        $siesa = new B2BISiesa();

        try{
            return $siesa->getSucursales($nit);
        }catch (\Exception $e){
            return false;
        }
    }

    /**
     * Retorna el listado de ls compañias relacionadas en PDV
     * @return array
     */
    public function getCitiesDrop()
    {
        $siesa = new B2BISiesa();

        try{
            return $siesa->getCities();
        }catch (\Exception $e){
            return false;
        }
    }

    /**
     * Retorna el listado de las ciudades con pedidos
     * @return array
     */
    public function getClients($description)
    {
        $siesa = new B2BISiesa();
        try{
            return $siesa->getClients($description);
        }catch (\Exception $e){
            return false;
        }
    }
    
    /**
     * Retorna el listado de las ciudades con pedidos
     * @return array
     */
    public function getRequestedCitiesDrop()
    {
        $siesa = new B2BISiesa();

        try{
            return $siesa->getRequestedCitiesDrop();
        }catch (\Exception $e){
            return false;
        }
    }
}
