<?php

namespace app\models;

use Yii;
use yii\db\ActiveRecord;
use yii\db\Expression;
use yii\behaviors\BlameableBehavior;
use yii\behaviors\AttributeBehavior;

/**
 * This is the model class for table "usuarios".
 *
 * @property integer $id
 * @property string $nombre
 * @property string $username
 * @property string $password
 * @property string $email
 * @property string $activo
 * @property string $created
 * @property string $created_by
 * @property string $modified
 * @property string $modified_by
 * @property string $proveedor_id
 * @property string $distribuidor_id
 *
 */
class Usuarios extends BeforeModel {

    public $password_repeat;         
    public $nomDistribuidor;
    public $nomProveedor;
    const SUPER_ADMIN = 'Super Administrador';
    const ADMIN = 'Administrador';  
    const GENERAL = 'General';      
    const PROVEEDOR = 'Proveedor';      
    const DISTRIBUIDOR = 'Distribuidor';      

    /**
     * @inheritdoc
     */
    public static function tableName() {
        return 'usuarios';
    }

    /**
     * @inheritdoc
     */
    public function rules() {
        return [
            //CREAR USUARIOS
            [['nombre', 'username', 'password', 'email', 'password_repeat', 'activo', 'tipo'], 'required', 'on' => ['crearuUs']],            
            [['nombre', 'username', 'password', 'email', 'password_repeat', 'activo', 'tipo', 'proveedor_id'], 'required', 'on' => ['crearuUsProv']],            
            [['nombre', 'username', 'password', 'email', 'password_repeat', 'activo', 'tipo', 'distribuidor_id'], 'required', 'on' => ['crearuUsDis']],            
            //EDITAR USUARIO
            [['nombre', 'username', 'email', 'tipo', 'activo'], 'required', 'on' => ['updateUs']],
            [['nombre', 'username', 'email', 'tipo', 'activo', 'proveedor_id'], 'required', 'on' => ['updateUsProv']],
            [['nombre', 'username', 'email', 'tipo', 'activo', 'distribuidor_id'], 'required', 'on' => ['updateUsDis']],
            ['email', 'email'],
            [['distribuidor_id', 'proveedor_id'], 'integer','on' => ['crearuUs', 'crearuUsProv', 'crearuUsDis', 'updateUsProv', 'updateUsDis']],
            [['distribuidor_id', 'proveedor_id'], 'integer'],
            [['username', 'email'], 'unique', 'on' => ['updateUs', 'crearuUs']],
            ['password', 'compare', 'on' => ['updateUs', 'crearuUs']],
            ['username', 'match', 'not' => true, 'pattern' => '/[^a-z0-9._-]/',
                'on' => ['crearuUs', 'updateUs'],
                'message' => 'No se permiten espacios o caracteres especiales'],
            [['activo', 'tipo'], 'string'],
            [['created', 'modified'], 'safe'],
            [['nombre', 'password'], 'string', 'max' => 100],
            [['nomDistribuidor', 'nomProveedor'], 'string', 'max' => 50],
            [['username', 'email', 'created_by', 'modified_by'], 'string', 'max' => 45],           
            //[['sede_id'], 'exist', 'skipOnError' => true, 'targetClass' => Sedes::className(), 'targetAttribute' => ['sede_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels() {
        return [
            'id' => Yii::t('app', 'ID'),
            'nombre' => Yii::t('app', 'Nombres y Apellidos'),
            'username' => Yii::t('app', 'Nombre de usuario'),
            'password' => Yii::t('app', 'Contraseña'),
            'email' => Yii::t('app', 'Correo Electrónico'),
            'activo' => Yii::t('app', 'Activo'),
            'created' => Yii::t('app', 'Creado'),
            'created_by' => Yii::t('app', 'Creado por'),
            'modified' => Yii::t('app', 'Modifiado'),
            'modified_by' => Yii::t('app', 'Modificado por'),            
            'password_repeat' => Yii::t('app', 'Repetir Contraseña'),            
            'Tipo' => Yii::t('app', 'Tipo'),            
            'distribuidor_id' => Yii::t('app', 'Distribuidor ID'),
            'nomDistribuidor' => Yii::t('app', 'Distribuidor ID'),
            'proveedor_id' => Yii::t('app', 'Proveedor ID'),
            'nomProveedor' => Yii::t('app', 'Proveedor ID'),
        ];
    }
    
    /**
     * 
     * @return string
     */
    public function scenarios()
    {
        $scenarios = parent::scenarios();        
        $scenarios['crearuUs'] = ['nombre', 'username', 'password', 'email', 'password_repeat', 'activo', 'tipo', 'proveedor_id', 'distribuidor_id'];        
        $scenarios['crearuUsProv'] = ['nombre', 'username', 'password', 'email', 'password_repeat', 'activo', 'tipo', 'proveedor_id'];        
        $scenarios['crearuUsDis'] = ['nombre', 'username', 'password', 'email', 'password_repeat', 'activo', 'tipo','distribuidor_id'];        
        $scenarios['updateUs'] = ['nombre', 'username', 'email', 'tipo', 'activo', 'password', 'password_repeat', 'proveedor_id', 'distribuidor_id'];        
        $scenarios['updateUsProv'] = ['nombre', 'username', 'email', 'tipo', 'activo', 'password', 'password_repeat', 'proveedor_id'];        
        $scenarios['updateUsDis'] = ['nombre', 'username', 'email', 'tipo', 'activo','password', 'password_repeat', 'distribuidor_id'];        
        return $scenarios;
    }            

    /**
     * 
     * @param type $id
     * @return type
     */
    public static function findIdentity($id) {
        return Usuarios::findOne($id);
    }
    
    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProveedor()
    {
        return $this->hasOne(Proveedores::className(), ['id' => 'proveedor_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDistribuidor()
    {
        return $this->hasOne(Distribuidores::className(), ['id' => 'distribuidor_id']);
    }

}
