<?php

namespace app\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use app\models\Usuarios;

/**
 * UsuariosSearch represents the model behind the search form about `app\models\Usuarios`.
 */
class UsuariosSearch extends Usuarios {

    /**
     * @inheritdoc
     */
    public function rules() {
        return [
            [['id'], 'integer'],
            [['username', 'email'], 'unique'],
            [['nomDistribuidor', 'nomProveedor'], 'string', 'max' => 50],
            [['nombre', 'username', 'password', 'email', 'activo', 'created', 
                'created_by', 'modified', 'modified_by', 'tipo', 
                'nombreSede', 'nombreCliente', 'distribuidor_id', 
                'proveedor_id','nomDistribuidor', 'nomProveedor'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios() {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params) {        
        $query = Usuarios::find();      
        $query->joinWith('proveedor');
        $query->joinWith('distribuidor');

        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);
        
        $dataProvider->sort->attributes['nomDistribuidor'] = [            
            'asc' => ['distribuidores.razon_social' => SORT_ASC],
            'desc' => ['distribuidores.razon_social' => SORT_DESC],
        ];
        
        $dataProvider->sort->attributes['nomProveedor'] = [            
            'asc' => ['proveedores.razon_social' => SORT_ASC],
            'desc' => ['proveedores.razon_social' => SORT_DESC],
        ];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'created' => $this->created,
            'modified' => $this->modified,            
        ]);

        $query->andFilterWhere(['like', 'nombre', $this->nombre])
                ->andFilterWhere(['like', 'username', $this->username])
                ->andFilterWhere(['like', 'password', $this->password])
                ->andFilterWhere(['like', 'email', $this->email])
                ->andFilterWhere(['like', 'activo', $this->activo])
                ->andFilterWhere(['like', 'distribuidores.razon_social', $this->nomDistribuidor])
                ->andFilterWhere(['like', 'proveedores.razon_social', $this->nomProveedor])
                ->andFilterWhere(['like', 'created_by', $this->created_by])
                ->andFilterWhere(['like', 'modified_by', $this->modified_by]);

        return $dataProvider;
    }

}
