<?php

namespace app\models;

use Yii;
use yii\base\Model;
use app\components\B2BISiesa;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "ventas".
 *
 * @property integer $id
 * @property string $codigo_cliente
 * @property string $nit
 * @property string $razon_social
 * @property string $cod_sucursal
 * @property string $sucursal
 * @property string $referencia_dist
 * @property string $desc_ref_dist
 * @property string $fecha
 * @property string $factura
 * @property string $cantidad
 * @property double $total
 * @property string $tipo
 * @property double $costo
 * @property string $unidad_medida
 * @property string $pedido
 * @property string $co
 * @property string $codigo_vendedor
 * @property string $ciudad
 * @property integer $distribuidor_id
 *
 * @property Distribuidores $distribuidor
 */
class Ventas extends Model {
    
    const NM_REPORTE = 'ventas';
        
    public $codigo_cliente;
    public $nit;
    public $razon_social;
    public $cod_sucursal;
    public $sucursal;
    public $referencia;
    public $referencia_dist;
    public $desc_ref_dist;
    public $referencia_prov;
    public $desc_ref_prov;
    public $categoria_prov;
    public $categoria_dist;
    public $fecha;
    public $factura;
    public $cantidad;
    public $total;
    public $tipo;
    public $costo;
    public $unidad_medida;
    public $pedido;
    public $co;
    public $mes;
    public $anio;
    // public $codigo_vendedor;
    public $ciudad_sucursal; 
    public $vendedor; 
    public $descuento; 
    public $impuesto;
    public $precio;
    
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'ventas';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [            
            [['codigo_cliente', 'nit', 'cantidad', 'codigo_vendedor'], 'integer'],            
            [['total', 'costo', 'anio', 'mes', 'descuento', 'impuesto', 'precio'], 'number'],
            [['razon_social'], 'string', 'max' => 200],
            [['cod_sucursal'], 'string', 'max' => 15],
            [['sucursal', 'referencia_dist', 'tipo', 'unidad_medida', 'co', 
                'categoria_prov', 'categoria_dist'], 'string', 'max' => 45],
            [['factura', 'pedido'], 'string', 'max' => 100],
            [['ciudad_sucursal', 'desc_ref_dist', 'desc_ref_prov','referencia_dist', 
                'referencia_prov', 'vendedor'], 'string', 'max' => 50],                          
            [['fecha','nit','razon_social','factura','referencia_dist','cantidad',
                'cod_sucursal','sucursal','tipo','costo','unidad_medida',
                'pedido','co', 'desc_ref_dist', 'total', 'referencia_prov', 
                'desc_ref_prov', 'anio', 'mes', 'categoria_prov', 'categoria_dist'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'codigo_cliente' => Yii::t('app', 'Codigo Cliente'),
            'nit' => Yii::t('app', 'Nit'),
            'razon_social' => Yii::t('app', 'Razon Social'),
            'cod_sucursal' => Yii::t('app', 'Cod Sucursal'),
            'sucursal' => Yii::t('app', 'Sucursal'),
            'referencia_dist' => Yii::t('app', 'Referencia Dist'),
            'referencia_prov' => Yii::t('app', 'Referencia Prov'),
            'desc_ref_dist' => Yii::t('app', 'Desc Referencia Dist'),
            'desc_ref_prov' => Yii::t('app', 'Desc Referencia Prov'),            
            'categoria_prov' => Yii::t('app', 'Categoria Prov'),            
            'categoria_dist' => Yii::t('app', 'Categoria Dist'),            
            'fecha' => Yii::t('app', 'Fecha'),
            'factura' => Yii::t('app', 'Factura'),
            'cantidad' => Yii::t('app', 'Cantidad'),
            'total' => Yii::t('app', 'Total'),
            'tipo' => Yii::t('app', 'Tipo'),
            'costo' => Yii::t('app', 'Costo'),
            'unidad_medida' => Yii::t('app', 'Unidad Medida'),
            'pedido' => Yii::t('app', 'Pedido'),
            'co' => Yii::t('app', 'Co'),
            'codigo_vendedor' => Yii::t('app', 'Codigo Vendedor'),
            'ciudad_sucursal' => Yii::t('app', 'Ciudad sucursal'),            
            'mes' => Yii::t('app', 'Mes'),            
            'anio' => Yii::t('app', 'Anio'),            
            'vendedor' => Yii::t('app', 'Vendedor'),            
            'descuento' => Yii::t('app', 'Descuento'),            
            'impuesto' => Yii::t('app', 'Impuesto'),            
            'precio' => Yii::t('app', 'Precio'),            
        ];
    }
    
    /**
     * Crea el reporte de ventas 
     * 
     * @return \yii\data\ArrayDataProvider
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function reporteVentas($valueListDefecto, $params){
        $b2bSiesa = new B2BISiesa();        
        $andWhere = '';
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');

        $this->load($params);
        #Parametros de filtradp ------------------------------------------------
        if($this->nit != "") 
            $andWhere .= " AND T200.f200_nit LIKE '%$this->nit%'";
        if($this->razon_social != "") 
            $andWhere .= " AND T200.f200_razon_social LIKE '%$this->razon_social%'";        
        if($this->factura != "") 
            $andWhere .= " AND f350_consec_docto LIKE '%$this->factura%'";        
        if($this->cod_sucursal != "") 
            $andWhere .= " AND FV.F461_ID_SUCURSAL_FACT LIKE '%$this->cod_sucursal%'";        
        if($this->sucursal != "") 
            $andWhere .= " AND C201.F201_DESCRIPCION_SUCURSAL LIKE '%$this->sucursal%'";        
        if($this->desc_ref_dist != "") 
            $andWhere .= " AND T120.f120_descripcion LIKE '%$this->desc_ref_dist%'";        
        if($this->costo != "") 
            $andWhere .= " AND T132.f132_ultimo_costo_local LIKE '%$this->costo%'";        
        if($this->precio != "") 
            $andWhere .= " AND MFV.f470_precio_uni LIKE '%$this->precio%'";        
        if($this->cantidad != "") 
            $andWhere .= " AND MFV.f470_cant_1 LIKE '%$this->cantidad%'";        
        if($this->tipo != "") 
            $andWhere .= " AND f350_id_tipo_docto LIKE '%$this->tipo%'";        
        if($this->pedido != "") 
            $andWhere .= " AND PV.f430_consec_docto LIKE '%$this->pedido%'";         
        if($this->co != "") 
            $andWhere .= " AND f285_descripcion LIKE '%$this->co%'"; 
        if($this->ciudad_sucursal != "") 
            $andWhere .= " AND T103.f013_descripcion LIKE '%$this->ciudad_sucursal%'";
        if($this->vendedor != "") 
            $andWhere .= " AND T200VEN.f200_razon_social LIKE '%$this->vendedor%'";
        //if($this->descuento != "") 
        //    $andWhere .= " AND MFV.f470_vlr_dscto_linea LIKE '%$this->descuento%'";
        if($this->mes != "")             
            if(is_array($this->mes) && count($this->mes)>0){
                foreach ($this->mes as $value){
                   $tempMes[] = $value +1; 
                }
                $andWhere .= " AND CONVERT(VARCHAR,MONTH(DOCFAC.f350_fecha)) IN('"
                        .implode("','",$tempMes)."')";                
            }                                       
        if($this->anio != ""){
            $tempAnio = explode(',', $this->anio);
            $andWhere .= " AND CONVERT(VARCHAR,YEAR(DOCFAC.f350_fecha)) IN('"
                    .implode("','",$tempAnio)."')";                
        }            
        //Creamos las fechas ---------------------------------------------------
        if($this->fecha != ""){             
            list($fechaIn, $fechaFin) = explode(' - ', $this->fecha);
            $andWhere .= " AND DOCFAC.f350_fecha BETWEEN '$fechaIn' AND '$fechaFin'";                      
        }else{            
            if($this->mes == '' && $this->anio == ''){
                $fechaFin = date('d-m-Y');                
                $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));            
                $andWhere .= " AND DOCFAC.f350_fecha BETWEEN '$fechaIn' AND '$fechaFin'";                      
            }            
        }
        //Obtenemos las referencias --------------------------------------------
        if(empty(Yii::$app->session['proveedor_id'])){              
            $modelReferencias = $this->_getReferencias();
        }else{
            //Buscamos las referencias del proveedor en sesion -----------------
            $modelReferencias = Referencias::find()
                    ->where(['proveedor_id'=>Yii::$app->session['proveedor_id']])
                    ->asArray()->all();  
        }        
        $referencias = ArrayHelper::map($modelReferencias, 
                        'ref_distribuidor', 'ref_distribuidor');
        $andWhere .= " AND T120.f120_referencia IN ('"
                .  implode("','", $referencias)."')";
        //Filtrado de referencias ----------------------------------------------
        if($this->referencia_dist != ""){                       
            $modelReferencias = $this->_getReferencias(
                    "ref_distribuidor LIKE '%$this->referencia_dist%'");            
            if(count($referencias)>0){
                $andWhere .= " AND T120.f120_referencia LIKE '%$this->referencia_dist%'";
            }else{
                $andWhere .= " AND T120.f120_referencia = '0'";
            }         
        }
        //Referencias proveedor ------------------------------------------------        
        if($this->referencia_prov != ""){            
            if(strpos($valueListDefecto, 'referencia_prov') !== false ){                                
                $modelReferencias = $this->_getReferencias(
                        "ref_proveedor LIKE '%$this->referencia_prov%'");
                $referencias = ArrayHelper::map($modelReferencias, 
                            'ref_distribuidor', 'ref_distribuidor');            
                $andWhere .= " AND T120.f120_referencia IN ('"
                        .  implode("','", $referencias)."')";
            }                        
        }
        if($this->desc_ref_prov != ""){                        
            if(strpos($valueListDefecto, 'desc_ref_prov') !== false ){                                
                $modelReferencias = $this->_getReferencias(
                        "ref_descripcion_proveedor LIKE '%$this->desc_ref_prov%'");
                $referencias = ArrayHelper::map($modelReferencias, 
                            'ref_distribuidor', 'ref_distribuidor');            
                $andWhere .= " AND T120.f120_referencia IN ('"
                        .  implode("','", $referencias)."')";
            }            
        }
        //Filtro de categorias -------------------------------------------------        
        if($this->categoria_prov != ""){                        
            if(strpos($valueListDefecto, 'categoria_prov') !== false ){                                
                $modelReferencias = $this->_getReferencias(
                        "categoria_proveedor LIKE '%$this->categoria_prov%'");                
                $referencias = ArrayHelper::map($modelReferencias, 
                            'ref_distribuidor', 'ref_distribuidor');            
                $andWhere .= " AND T120.f120_referencia IN ('"
                        .  implode("','", $referencias)."')";
            }            
        }
        if($this->categoria_dist != ""){                        
            if(strpos($valueListDefecto, 'categoria_dist') !== false ){                                
                $modelReferencias = $this->_getReferencias(
                        "categoria_distribuidor LIKE '%$this->categoria_dist%'");                
                $referencias = ArrayHelper::map($modelReferencias, 
                            'ref_distribuidor', 'ref_distribuidor');            
                $andWhere .= " AND T120.f120_referencia IN ('"
                        .  implode("','", $referencias)."')";
            }            
        }
        //Consultamos las ventas en SIESA --------------------------------------
        $select = $this->_getListSelect($valueListDefecto);
        $groupBy = $this->_getListGroupBy($valueListDefecto);
        $orderBy = $this->_getListGroupBy($valueListDefecto); //Los parametros del group y ordern son similares        
        //Consultamos los datos en siesa                 
        $arrayResult = $b2bSiesa->getVentas($referencias, $select, $andWhere, 
                $groupBy, $orderBy, $branchOffice);
        //Validamos si se presentan todos los datos ----------------------------
        $paramShowAll = '_tog';
        $exportAll = 'all';        
        $pagination = ['pageSize' => 20];
        foreach ($_GET as $key => $value) {
            if (strpos($key, $paramShowAll) !== false){
                if($value == $exportAll){
                    $pagination = false;        
                }
            }                    
        }
        //Adicionamos las referencias del proveedor ----------------------------
        if(strpos($valueListDefecto, 'referencia_prov') !== false){  
            $tempArrayResult = [];
            //Agregamos las referencias del proveedor al resultado -------------
            if(count($arrayResult)>0){
                $tempRefencias = ArrayHelper::index($modelReferencias, 'ref_distribuidor');                
                foreach ($arrayResult as $value) {                    
                    if(isset($value['referencia_dist']) 
                        && isset($tempRefencias[$value['referencia_dist']]['ref_proveedor'])){
                        $value['referencia_prov'] = $tempRefencias[$value['referencia_dist']]['ref_proveedor'];
                    }else{
                        $value['referencia_prov'] = '';
                    }
                    $tempArrayResult[] = $value;
                }                
                $arrayResult = $tempArrayResult;
            }
        }               
        if(strpos($valueListDefecto, 'desc_ref_prov') !== false ){  
            $tempArrayResult = [];
            //Agregamos las referencias del proveedor al resultado -------------            
            if(count($arrayResult)>0){
                $tempRefencias = ArrayHelper::index($modelReferencias, 'ref_distribuidor');                    
                foreach ($arrayResult as $value) {                    
                    if(isset($value['referencia_dist']) 
                        && isset($tempRefencias[$value['referencia_dist']]['ref_descripcion_proveedor'])){
                        $value['desc_ref_prov'] = $tempRefencias[$value['referencia_dist']]['ref_descripcion_proveedor'];                    
                    }else{
                        $value['desc_ref_prov'] = '';                        
                    }
                    $tempArrayResult[] = $value;
                }                
                $arrayResult = $tempArrayResult;                
            }
        }           
        //Retornamos los datos -------------------------------------------------        
        return new \yii\data\ArrayDataProvider([
            'allModels' => $arrayResult,
            'sort'  => [
                'attributes' => explode(',', $valueListDefecto),
            ],            
            'pagination' => $pagination
        ]);        
    }
    
    /**
     * Retorna le listaod de las referencias por defecto segun el tipo de 
     * usuario
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getReferencias($addAndCondicion = ''){
        $modelReferencias = Referencias::find();
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');

        if (\Yii::$app->user->identity->isProveedor()){
            $idProveedor = Yii::$app->user->identity->getIdProveedor();
            $modelReferencias->where(['proveedor_id'=>$idProveedor]);                
        }elseif (\Yii::$app->user->identity->isDistribuidor()){
            $idDistribuidor = Yii::$app->user->identity->getIdDistribuidor();
            $modelReferencias->where(['distribuidor_id'=>$idDistribuidor]);
        }

        if(empty($branchOffice['providers']) == false){
            $modelReferencias->where(['in', 'proveedor_id', $branchOffice['providers']]);
        }
        
        if(!empty($addAndCondicion)){            
            $modelReferencias->andWhere($addAndCondicion);
        }
        return $modelReferencias->asArray()->all();        
    }
    
    /**
     * Listado de los parametros por defecto sin espacios
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsDefecto(){                                  
        return 'fecha,nit,razon_social,factura,referencia_dist,referencia_prov,'
        . 'cantidad';
    }
    
    /**
     * Listado de los parametros adicionales sin espacios
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsAdd(){
        return 'cod_sucursal,sucursal,total,tipo,costo,unidad_medida,'
            . 'pedido,co,desc_ref_dist,desc_ref_prov,mes,anio,categoria_prov,'
            . 'categoria_dist,vendedor,descuento,impuesto,ciudad_sucursal,precio';
    }
    
    /**
     * Lisrados de parametros configurables en el reporte
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParams(){
        return [
            'nit' => ['content' => Yii::t('app', 'Nit')],           
            'razon_social' => ['content' => Yii::t('app', 'Razon Social')],           
            'factura' => ['content' => Yii::t('app', 'Factura')],  
            'referencia_dist' => ['content' => Yii::t('app', 'Referencia Dist')],   
            'desc_ref_dist' => ['content' => Yii::t('app', 'Desc Referencia Dist')],   
            'referencia_prov' => ['content' => Yii::t('app', 'Referencia Prov')],   
            'desc_ref_prov' => ['content' => Yii::t('app', 'Desc Referencia Prov')],   
            'categoria_prov' => ['content' => Yii::t('app', 'Categoria Prov')],   
            'categoria_dist' => ['content' => Yii::t('app', 'Categoria Dist')],   
            'cantidad' => ['content' => Yii::t('app', 'Cantidad')],    
            'cod_sucursal' => ['content' => Yii::t('app', 'Cod Sucursal')],           
            'sucursal' => ['content' => Yii::t('app', 'Sucursal')],                               
            'fecha' => ['content' => Yii::t('app', 'Fecha')],                                 
            'total' => ['content' => Yii::t('app', 'Total')],                                 
            'tipo' => ['content' => Yii::t('app', 'Tipo')],                                 
            'costo' => ['content' => Yii::t('app', 'Costo')],                                 
            'unidad_medida' => ['content' => Yii::t('app', 'Unidad Medida')],                                 
            'pedido' => ['content' => Yii::t('app', 'Pedido')],                                 
            'co' => ['content' => Yii::t('app', 'Co')],     
            'mes' => ['content' => Yii::t('app', 'Mes')],     
            'anio' => ['content' => Yii::t('app', 'Anio')],     
            'ciudad_sucursal' => ['content' => Yii::t('app', 'Ciudad Sucursal')],     
            'vendedor' => ['content' => Yii::t('app', 'Vendedor')],     
            'descuento' => ['content' => Yii::t('app', 'Descuento')],     
            'impuesto' => ['content' => Yii::t('app', 'Impuesto')],     
            'precio' => ['content' => Yii::t('app', 'Precio')],     
        ];
    }
    
    /**
     * Metodo que retorna el listado de las columnas para el grid
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function listParamsGrid(){
        //Traemos todos los meses para el filtro -------------------------------
        $meses = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 
            'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
        return [
            'nit' => 'nit',           
            'razon_social' => 'razon_social',           
            'factura' => 'factura',  
            'referencia_dist' => 'referencia_dist',                 
            'referencia_prov' => 'referencia_prov',  
            'desc_ref_dist' => [                                
                'attribute' => 'desc_ref_dist',
                'value' => 'desc_ref_dist',                                
            ],
            'desc_ref_prov' => [                                
                'attribute' => 'desc_ref_prov',
                'value' => 'desc_ref_prov',                                
            ],
            'categoria_prov' => 'categoria_prov',   
            'categoria_dist' => 'categoria_dist',   
            'cantidad' => 
            [                
                'attribute' => 'cantidad',
                'value' => 'cantidad',                
                'format'=>['decimal', 2],
                'pageSummary'=>true
            ],
            'mes' =>
            [                
                'attribute' => 'mes',
                'width' => '150px',
                'filter'=>$meses, 
                'filterType'=>  \kartik\grid\GridView::FILTER_SELECT2,
                'filterWidgetOptions'=>[
                    'options' => ['multiple' => true],
                    'pluginOptions'=>['allowClear'=>true,'tags' => true,],
                ],
                'value' => function($model) use($meses){
                    return $meses[$model['mes']-1];
                },                                
                'group'=>true, 
                'groupedRow'=>false,
                'subGroupOf'=>'anio', // supplier column index is the parent group,
                'groupFooter'=>function ($model, $key, $index, $widget) { // Closure method                    
                    return [
                        'mergeColumns'=>[['total', 'cantidad', 'anio']], // columns to merge in summary
                        'content'=>[// content to show in each summary cell
                            'mes' => Yii::t('app', 'Total Mes'),
                            'total'=>  \kartik\grid\GridView::F_SUM,
                            'cantidad'=>  \kartik\grid\GridView::F_SUM,
                        ],
                        'contentFormats' => [ // content reformatting for each summary cell
                            'total' => ['format' => 'number', 'decimals' => 0],
                            'cantidad' => ['format' => 'number', 'decimals' => 0],
                        ],                  
                        'options' => ['class' => 'danger', 'style' => 'font-weight:bold;']
                    ];
                }
            ],
            'anio' =>
            [                
                'attribute' => 'anio',
                'value' => 'anio',                                
                'group'=>true, 
                'groupedRow'=>false,                
                'groupFooter'=>function ($model, $key, $index, $widget) { // Closure method                    
                    return [
                        'mergeColumns'=>[['total', 'cantidad', 'mes']], // columns to merge in summary
                        'content'=>[// content to show in each summary cell 
                            'anio' => Yii::t('app', 'Total').' '.Yii::t('app', 'Anio'),
                            'total'=>  \kartik\grid\GridView::F_SUM,
                            'cantidad'=>  \kartik\grid\GridView::F_SUM,
                        ],
                        'contentFormats' => [ // content reformatting for each summary cell
                            'total' => ['format' => 'number', 'decimals' => 0],
                            'cantidad' => ['format' => 'number', 'decimals' => 0],
                        ],                  
                        'options' => ['class' => 'danger', 'style' => 'font-weight:bold;']
                    ];
                }
            ],
            'cod_sucursal' => 'cod_sucursal',           
            'sucursal' => 'sucursal',
            'ciudad_sucursal' => 'ciudad_sucursal',
            'vendedor' => 'vendedor',
            'vendedor' => 'vendedor',
            'fecha' => 
            [
                'attribute' => 'fecha',
                'filterType' => \kartik\grid\GridView::FILTER_DATE_RANGE,
                'filterWidgetOptions' => [
                    'options' => ['placeholder' => 'Rango Fechas', 'readonly'=>true],
                    'pluginOptions' => [
                        'allowClear' => true,
                        'locale'=>[                            
                            'format' => 'DD-MM-YYYY',
                            'separator'=>' - ',
                        ],                        
                    ],
                ],                
                'width' => '200px',
            ],                                 
            'impuesto' =>
            [                
                'attribute' => 'impuesto',
                'value' => 'impuesto',                
                'format'=>['decimal', 2],
                'pageSummary'=>true,
                'mergeHeader'=>true,
                'contentOptions' => ['style'=>'vertical-align: top !important;'],
                'headerOptions' => ['style'=>'vertical-align: top !important;'],
            ],
            'descuento' =>
            [                
                'attribute' => 'descuento',
                'value' => 'descuento',                
                'format'=>['decimal', 2],
                'pageSummary'=>true,
                'mergeHeader'=>true,
                'contentOptions' => ['style'=>'vertical-align: top !important;'],
                'headerOptions' => ['style'=>'vertical-align: top !important;'],
            ],
            'total' =>
            [                
                'attribute' => 'total',
                'value' => 'total',                
                'format'=>['decimal', 2],
                'pageSummary'=>true,
                'mergeHeader'=>true,
                'contentOptions' => ['style'=>'vertical-align: top !important;'],
                'headerOptions' => ['style'=>'vertical-align: top !important;'],
            ],
            'tipo' => 'tipo',            
            'costo' =>            
            [                
                'attribute' => 'costo',
                'value' => 'costo',                
                'format'=>['decimal', 2],
                'pageSummary'=>true
            ],
            'precio' =>            
            [                
                'attribute' => 'precio',
                'value' => 'precio',                
                'format'=>['decimal', 2],
                'pageSummary'=>true,
            ],
            'unidad_medida' => 'unidad_medida',
            'pedido' => 'pedido',
            'co' => 'co'
        ];
    }
    
    
    
    /**
     * Retorna los parametrso que van en el select del B2BiSiesa
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsSelect(){
        return [
            'id'=>'f470_rowid AS id',
            'codigo_cliente' => '-- T200.f200_rowid AS codigo_cliente',
            'nit'=> 'T200.f200_nit as nit', 
            'razon_social' => 'T200.f200_razon_social as razon_social',
            'codigo_vendedor' => '-- FV.f461_rowid_tercero_vendedor AS codigo_vendedor',   
            'factura' => "RTRIM(LTRIM(f350_id_tipo_docto)) + '-' + CONVERT(VARCHAR,(REPLICATE('0', (8 - LEN(f350_consec_docto))) + CONVERT(VARCHAR,f350_consec_docto))) AS factura",            
            'cod_sucursal' => 'FV.F461_ID_SUCURSAL_FACT as cod_sucursal',
            'sucursal' => 'C201.F201_DESCRIPCION_SUCURSAL AS sucursal',
            'desc_ref_dist' => 'T120.f120_descripcion as desc_ref_dist',
            'desc_ref_prov' => 'RTRIM(LTRIM(T120.f120_referencia)) as referencia_dist,T120.f120_descripcion as desc_ref_dist',
            'categoria_prov' => $this->_getStrCategoriaProv().' as categoria_prov',
            'categoria_dist' => $this->_getStrCategoriaDist().' as categoria_dist',
            'fecha' => 'CONVERT(VARCHAR,DOCFAC.f350_fecha,105) as fecha',
            'anio' => 'YEAR(DOCFAC.f350_fecha) as anio',            
            'mes' => 'MONTH(DOCFAC.f350_fecha) as mes',            
            'referencia_dist' => 'RTRIM(LTRIM(T120.f120_referencia)) as referencia_dist',
            'referencia_prov' => 'RTRIM(LTRIM(T120.f120_referencia)) as referencia_dist',
            'costo' => 'ISNULL(T132.f132_ultimo_costo_local,0) AS costo', 
            'precio' => "SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_precio_uni*-1 ELSE MFV.f470_precio_uni END)) AS precio", 
            'cantidad' => "SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_cant_1*-1 ELSE MFV.f470_cant_1 END)) AS cantidad",
            //'total' => "SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea END)) AS total",
            //'total' => "SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_bruto)*-1 ELSE MFV.f470_vlr_bruto END)) AS total",
            'total' => "SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto)*-1 ELSE MFV.f470_vlr_neto END)) AS total",
            'co' => 'f285_descripcion AS co',
            'unidad_medida' => "'UNID' AS unidad_medida", 
            'tipo' => "(CASE WHEN RTRIM(LTRIM(f350_id_tipo_docto)) = 'FV' THEN 0 ELSE 1 END) AS tipo",
	    'pedido' => "RTRIM(LTRIM(f430_id_tipo_docto)) + '-' + CONVERT(VARCHAR,(REPLICATE('0', (8 - LEN(PV.f430_consec_docto))) + CONVERT(VARCHAR,PV.f430_consec_docto)))  AS pedido",
	    'vendedor' => "T200VEN.f200_razon_social as vendedor",
	    'ciudad_sucursal' => "T103.f013_descripcion as ciudad_sucursal",
	    'descuento' => "SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_dscto_linea END)) AS descuento",
	    'impuesto' => "SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_imp END)) AS impuesto",
        ];
    }
    
    /**
     * Retorna los parametros que van el el Group By de B2BiSiesa
     * 
     * @return type
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _paramsGroupBy(){
        #Es necesaio el / para la separacion de columnas no debe ser modificado
        return [
            'id'=>'f470_rowid',
            'codigo_cliente' => 'T200.f200_rowid',
            'nit'=> 'T200.f200_nit', 
            'razon_social' => 'T200.f200_razon_social',
            'codigo_vendedor' => 'FV.f461_rowid_tercero_vendedor',   
            //'factura' => "f470_rowid,f350_id_tipo_docto,f350_consec_docto",
            'factura' => "f350_id_tipo_docto / f350_consec_docto",
            'cod_sucursal' => 'FV.F461_ID_SUCURSAL_FACT',
            'sucursal' => 'C201.F201_DESCRIPCION_SUCURSAL',            
            'referencia_dist' => 'T120.f120_referencia',
            'referencia_prov' => 'T120.f120_referencia',
            'desc_ref_dist' => 'T120.f120_descripcion',
            'desc_ref_prov' => 'T120.f120_referencia / T120.f120_descripcion',
            'categoria_prov' => $this->_getStrCategoriaProv(),
            'categoria_dist' => $this->_getStrCategoriaDist(),
            'fecha' => 'DOCFAC.f350_fecha',
            'anio' => 'YEAR(DOCFAC.f350_fecha)',            
            'mes' => 'MONTH(DOCFAC.f350_fecha)',            
            'costo' => 'T132.f132_ultimo_costo_local', 
            //'cantidad' => "SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_cant_1*-1 ELSE MFV.f470_cant_1 END)) AS cantidad",
            //'total' => "SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp-MFV.f470_vlr_dscto_linea END)) AS total",
            'co' => 'f285_descripcion',            
            'tipo' => "f350_id_tipo_docto",
	    //'numero_pedido' => "f470_rowid,f430_id_tipo_docto,PV.f430_consec_docto"
	    'pedido' => "f430_id_tipo_docto / PV.f430_consec_docto",
	    'vendedor' => "T200VEN.f200_razon_social",
	    'ciudad_sucursal' => "T103.f013_descripcion",
	    //'descuento' => "MFV.f470_vlr_dscto_linea",
        ];
    }
    
    /**
     * Retorna el listado a partir de un array y un string dado
     * 
     * @param string $string String separado por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getTempList($string){        
        return $this->_getDataFromList($string, $this->listParams());
    }
    
    /**
     * Retorna el array de los parametros que iran el en select de la consulta 
     * de ventas
     * 
     * @param string $params Parametros separados por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListSelect($params){
        return $this->_getDataFromList($params, $this->_paramsSelect());
    }
    
    /**
     * Retorna el array de los parametros que iran el en group de la consulta 
     * de ventas
     * 
     * @param string $params Parametros separados por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getListGroupBy($params){
        return $this->_getDataFromList($params, $this->_paramsGroupBy());
    }
    
    /**
     * Retorna el array de los parametros que iran el en grid de la vista     
     * 
     * @param string $params Parametros separados por comas
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getListGrid($params){
        return $this->_getDataFromList($params, $this->listParamsGrid());
    }        
    
    /**
     * Funcion que busca los datos de un string en un array
     * 
     * @param string $string Datos separados por comas    
     * @param array  $list   Array donde se buscaran los datos
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getDataFromList($string, $list){
        $array = explode(',', $string);
        $arrayResul = [];
        if(count($array)>0){            
            foreach ($array as $value) {
                $trimValue = trim($value);
                if(isset($list[$trimValue])){
                    $arrayResul[$trimValue] = $list[$trimValue];
                }                
            }
        }
        return $arrayResul;
    }
    
    /**
     * Retorna la informacion para el dashboard
     * 
     * @param integer $idProveedorPost Id del proveedor
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getInfoDasboard($idProveedorPost = null){
        $andWhere = '';
        $b2bSiesa = new B2BISiesa;
        $fechaFin = date('d-m-Y');                
        $fechaIn = date('d-m-Y', mktime(0, 0, 0, date('m'), 1, date('Y')));
        $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');
        $modelReferencias = $this->getReferences($idProveedorPost, $branchOffice);
        $referencias = ArrayHelper::map($modelReferencias, 
                    'ref_distribuidor', 'ref_distribuidor');
        $ultVentas = $b2bSiesa->getUltimasVentasMeses($referencias, 3, $branchOffice);

        //Traemos las metas del proveedor --------------------------------------
        $proyecciones = [];
        $arrayProy = [];        
        if(is_null($idProveedorPost)){
            $proyecciones = Proyecciones::find()->where(
                ['usuario_id'=>Yii::$app->user->identity->id])->asArray()->all();            
        }else{
            $usuario = Usuarios::findOne(['proveedor_id'=>$idProveedorPost]);
            if(!is_null($usuario)){
                $proyecciones = Proyecciones::find()
                    ->where(['usuario_id'=>$usuario->id])->asArray()->all();            
            }            
        }
        
        if(count($proyecciones)>0){
            $arrayProy = ArrayHelper::index($proyecciones, 'mes', ['anio']);
        }        
        //----------------------------------------------------------------------
        $dataMeses = [];
        $dataTotales = [];
        if(count($ultVentas)>0){ 
            $meses = ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 
            'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre'];
            foreach ($ultVentas as $anio => $datos) { 
                foreach ($datos as $data) {                 
                    $dataMeses[$anio][] = $meses[$data['mes']-1];                
                    $dataTotales[$anio][] = $data['total'];                               
                }            
            }            
        }
        $infoDashboard['meses_grafica']= $dataMeses; 
        $infoDashboard['totales_grafica']= $dataTotales; 
        $infoDashboard['proyecciones']= $arrayProy; 
        //TotalVenta -----------------------------------------------------------
        $infoDashboard['totalVeta'] = 0;
        $infoDashboard['cliente_nit'] = '';
        $infoDashboard['cliente_razon_social'] = '';
        $infoDashboard['cliente_total'] = '0';
        $infoDashboard['referencia_ref'] = '';
        $infoDashboard['referencia_desc'] = '';
        $infoDashboard['referencia_total'] = '0';
        
        if(isset($refVenta['referencia']) && isset($refVenta['descripcion']) && isset($refVenta['total'])){
            $infoDashboard['referencia_ref'] = $refVenta['referencia'];
            $infoDashboard['referencia_desc'] = $refVenta['descripcion'];
            $infoDashboard['referencia_total'] = $refVenta['total'];
        }
        
        return $infoDashboard;
    } 
        
    /**
     * Retorna el string para la consulta de categorias por proveedor
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getStrCategoriaProv(){
        $referencas = $this->_getReferencias();        
        $groupCategoria = ArrayHelper::index($referencas, 'id', ['categoria_proveedor']);        
        $str = "(CASE";
        if(count($groupCategoria) > 0){            
            foreach ($groupCategoria as $categoria => $data ) {                
                $referencias = ArrayHelper::map($data, 'id', 'ref_distribuidor');
                $nomCategroria = !empty($categoria) ? trim($categoria) : 'null';                
                $str .= " WHEN f120_referencia IN('".  implode("','", $referencias)."') THEN '".$nomCategroria."' ";                
            }
        }
        $str .= "ELSE 'null' END)";
        return $str;
    }
    
    /**
     * Retorna el string para la consulta de categorias por distribuidor
     * 
     * @return string
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getStrCategoriaDist(){
        $referencas = $this->_getReferencias();        
        $groupCategoria = ArrayHelper::index($referencas, 'id', ['categoria_distribuidor']); 
        $str = "";
        if(count($groupCategoria) > 0){            
            $str .= "(CASE";
            foreach ($groupCategoria as $categoria => $data ) {                
                $referencias = ArrayHelper::map($data, 'id', 'ref_distribuidor');
                $nomCategroria = !empty($categoria) ? trim($categoria) : null;                
                $str .= " WHEN f120_referencia IN('".  implode("','", $referencias)."') THEN '".$nomCategroria."' ";                
            }
            $str .= "ELSE null END)";
        }                
        return $str;
    }
    
    /**
     * Retorna las referencias 
     * 
     * @param integer $idProveedor  
     * @param string  $branchOffice
     * 
     * @return array
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2016 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function getReferences($idProveedor, $branchOffice){
        $modelReferencias = [];
        if(is_null($idProveedor) || empty($idProveedor)){
            if (\Yii::$app->user->identity->isProveedor()){
                $idProveedor = Yii::$app->user->identity->getIdProveedor();
                $modelReferencias = Referencias::find()
                        ->where(['proveedor_id'=>$idProveedor])
                        ->asArray()->all();                
            }elseif (\Yii::$app->user->identity->isDistribuidor()){
                $idDistribuidor = Yii::$app->user->identity->getIdDistribuidor();

                if(empty($branchOffice['providers']) == false){
                    $modelReferencias = Referencias::find()
                        ->where(['in', 'proveedor_id', $branchOffice['providers']])
                        ->andWhere(['distribuidor_id'=>$idDistribuidor])
                        ->asArray()->all();
                }else{
                    $modelReferencias = Referencias::find()
                        ->where(['distribuidor_id'=>$idDistribuidor])
                        ->asArray()->all();
                }
            }else{
                if(empty($branchOffice['providers']) == false){
                    $modelReferencias = Referencias::find()
                        ->asArray()
                        ->where(['in', 'proveedor_id', $branchOffice['providers']])
                        ->all();
                }else{
                    $modelReferencias = Referencias::find()
                        ->asArray()->all();
                }
            }
        }else{
            if(empty($branchOffice['providers']) == false){
                $modelReferencias = Referencias::find()
                    ->where(['proveedor_id'=>$idProveedor])
                    ->andWhere(['in', 'proveedor_id', $branchOffice['providers']])
                    ->asArray()->all();
            }else{
                $modelReferencias = Referencias::find()
                    ->where(['proveedor_id'=>$idProveedor])
                    ->asArray()->all();
            }
        }
        return $modelReferencias;
    }
    
}
