<?php

use yii\helpers\Html;
use kartik\grid\GridView;
use kartik\export\ExportMenu;
use kartik\sortinput\SortableInput;
use yii\bootstrap\ActiveForm;
use yii\helpers\ArrayHelper;
use kartik\select2\Select2;
use app\models\Proveedores;

/* @var $this yii\web\View */
/* @var $model app\models\Ventas */
/* @var $dataProvider yii\data\ActiveDataProvider */

#Busqueda de proveedores para el superadministrador ----------------------------
if (\Yii::$app->user->identity->isSuperAdmin()){
    $branchOffice = Yii::$app->runAction('usr-branch-office/get-branch-user');

    if(empty($branchOffice['providers']) == false){
        $peoveedores = ArrayHelper::map(Proveedores::find()->where(['in', 'id', $branchOffice['providers']])->all(), 'id', 'razon_social');
    }else{
        $peoveedores = ArrayHelper::map(Proveedores::find()->all(), 'id', 'razon_social');
    }
}

if(!empty(Yii::$app->session['proveedor_id'])){    
    $this->title = Yii::t('app', 'Reporte de Ventas').': '
            .$peoveedores[Yii::$app->session['proveedor_id']];
}else{
    $this->title = Yii::t('app', 'Reporte de Ventas');
}
//------------------------------------------------------------------------------

$btnRefress = Html::a("<i class='glyphicon glyphicon-refresh'></i> "
    .Yii::t('app', 'Refrescar'), ['ventas'], ['class' => 'btn btn-default']);
?>
<?php if (\Yii::$app->user->identity->isSuperAdmin()): ?>
    <?php $form = ActiveForm::begin([
        'layout' => 'horizontal',
        'fieldConfig' => [
            'template' => " <div class=\"form-group form-md-line-input\">{label}\n{beginWrapper}\n{input}<div class=\"form-control-focus\"> </div>\n{error}\n</div>{endWrapper}",
            'horizontalCssClasses' => [
                'label' => 'col-md-2 control-label',
                'offset' => 'col-sm-offset-4',
                'wrapper' => 'col-sm-10',
                'error' => 'has-error',
                'hint' => 'help-block',
            ],
        ],]); ?>
        <div class="box box-info collapsed-box">
            <div class="box-header with-border">
                <h3 class="box-title"><?= Yii::t('app', 'Proveedores') ?></h3>
                <div class="box-tools pull-right">
                    <button type="button" class="btn btn-box-tool" data-widget="collapse"><i class="fa fa-plus"></i>
                    </button>
                </div>        
            </div>    
            <div class="box-body">   
                <div class="row">
                    <div class="col-lg-offset-2 col-sm-7 ">
                    <?= Select2::widget([
                        'name' => 'proveedor_id',
                        'data' => $peoveedores,
                        'size' => Select2::MEDIUM,
                        'options' => ['placeholder' => 'Seleccione ...', 'prompt'=>'Todos'],
                        'pluginOptions' => [
                            'allowClear' => true
                        ],
                    ]);
                    ?>
                    </div>                    
                    <div class="col-sm-3">
                        <?= Html::submitButton(Yii::t('app', 'Buscar'), ['class' => 'btn btn-primary']) ?>                            
                    </div>                    
                </div>                                 
            </div>                                 
        </div>
    <?php ActiveForm::end(); ?>    
<?php endif; ?>

<?php $form = ActiveForm::begin(); ?>
<div class="box box-info collapsed-box">
    <div class="box-header with-border">
        <h3 class="box-title"><?= Yii::t('app', 'Parametros') ?></h3>
        <div class="box-tools pull-right">
            <button type="button" class="btn btn-box-tool" data-widget="collapse"><i class="fa fa-plus"></i>
            </button>
        </div>        
    </div>    
    <div class="box-body">
        <h5><?= Yii::t('app', 'Parametros Adicionales') ?></h5>     
        <?php 
            echo SortableInput::widget([
                'name'=>'listAdd',
                'value'=>$valueListAdd,
                'sortableOptions' => ['type'=>'grid'], 
                'items' => $listParamsAdd,
                'hideInput' => true,
                'sortableOptions' => [
                    'itemOptions'=>['style' => 'min-height: 10%; font-size: 85%;'],   
                    'connected'=>true,
                    'type'=>'grid'
                ],
                'options' => ['class'=>'form-control', 'readonly'=>true]
            ]);
        ?>
    </div>                        
    <div class="box-footer">     
        <h5><?= Yii::t('app', 'Parametros Reporte') ?></h5>
        <?php 
        echo SortableInput::widget([
            'name'=>'listDefecto',
            'value'=>$valueListDefecto,
            'sortableOptions' => ['type'=>'grid'], 
            'items' => $listParamsDefecto,
            'hideInput' => true,
            'sortableOptions' => [
                'itemOptions'=>['class'=>'alert alert-info', 'style' => 'min-height: 10%; font-size: 85%;'],                    
                'connected'=>true,
                'type'=>'grid'
            ],
            'options' => ['class'=>'form-control', 'readonly'=>true]
        ]);

        echo Html::resetButton('Reiniciar', ['class'=>'btn btn-default']);
        echo '&nbsp;&nbsp;';
        echo Html::submitButton('Crear Reporte', ['class' => 'btn btn-primary'])
        ?>            
    </div>      
</div>
<?php ActiveForm::end(); ?>

<?php
//Listas sortable
$gridColumns = $columns;
//$gridColumns = [
//    [
//        'class'=>'kartik\grid\SerialColumn',
//        'contentOptions'=>['class'=>'kartik-sheet-style'],
//        'width'=>'36px',
//        'header'=>'',
//        'headerOptions'=>['class'=>'kartik-sheet-style']
//    ],
//    [
//        'class' => 'kartik\grid\ExpandRowColumn',
//        //'width'=>'50px',
//        'value' => function ($model, $key, $index, $column) {             
//            return GridView::ROW_COLLAPSED;
//        },
//        'header' => '&nbsp;',                
//        'extraData' => ['fecha_venta'=>$fechaVenta],                
//        //'headerOptions'=>['class'=>'kartik-sheet-style'],                             
//        'detailUrl' => yii\helpers\Url::to(['reportes/detallecliente']).'',
//        'expandOneOnly' => true,
//        'expandIcon' => '<span class="fa fa-chevron-right" style="color: #00144C; font-size: 70%"></span>',
//        'collapseIcon' => '<span class="fa fa-chevron-down" style="color: #00144C; font-size: 70%"></span>',
//        'width' => '20px',
//        'detailRowCssClass' => GridView::TYPE_INFO
//    ],            
//    $columns
//];
                                
    $fullExportMenu = ExportMenu::widget([
        'dataProvider' => $dataProvider,
        'columns' => $gridColumns,
        'columnSelectorOptions' => [
            'label' => Yii::t('app', 'Columnas'),
        ],
        'dropdownOptions' => [
            'label' => Yii::t('app', 'Exportar'),
            'class' => 'btn btn-default'
        ],
        //'fontAwesome' => true,
        'showConfirmAlert' => false,
        'target' => '_blank',
        'filename' => $this->title . '_' . date('Ymd'),
        'exportRequestParam' => 'exportar',
        'columnBatchToggleSettings' => [
            'label' => Yii::t('app', 'Todo')
        ],
        'exportConfig' => [
            ExportMenu::FORMAT_TEXT => false,
            ExportMenu::FORMAT_PDF => false,
            ExportMenu::FORMAT_HTML => false,
        ]
    ]);
?>
<div class="maquinas-index box box-primary">     
    <?php
    $layout = <<< HTML
    <div class="box-header with-border" >
    {$fullExportMenu} {toggleData} {$btnRefress}
    </div>
        {summary}
    <div class="clearfix"></div>
        {items}
    {pager}
HTML;
    
    ?>    
    <div class="box-body">
        <small>
            <?= GridView::widget([
                //'id' => 'data-ventas-maquinas',
                'dataProvider' => $dataProvider,
                'filterModel' => $model,
                //'responsive' => true,
                //'responsiveWrap' => false,
                'showPageSummary' => true,
                'resizableColumns'=>true,
                
                'floatHeader'=>true,
                //'floatHeaderOptions'=>['scrollingTop'=>'0'],
                'perfectScrollbar'=>true,
                'perfectScrollbarOptions'=>[
                    //'suppressScrollX '=>true,
                    'useBothWheelAxes   '=>true,
                    //'scrollXMarginOffset   '=>100,
                ],
                'layout' => $layout,
                'toolbar'=> [            
                    '{export}',
                    '{toggleData}',
                ],                    
                'export'=>[
                    'fontAwesome'=>true
                ],   
        //        'floatHeader'=>true,
        //        'floatHeaderOptions'=>['scrollingTop'=>'0'],
                'exportConfig' => [            
                    GridView::EXCEL => true
                ],
        //        'pjax' => true, // pjax is set to always true for this demo
        //        'pjaxSettings'=>['neverTimeout'=>true,],
                'columns' => $gridColumns,
            ]); ?>
        </small>
    </div>
</div>
<script>
    $('body').on('dragend', 'li', function(){
        $("ul[id$='sortable'").trigger('sortupdate');
    });
</script>